/**
 * App State
 */

 var App = (function($) {

 	var isVersionChecked = false;

 	function offlineHandler(e) {

		fsm.handle('offline');
 	};

 	function onlineHandler(e) {
 		Stations.update();
 		fsm.handle('online');
 	};

 	function pauseHandler(e) {
		fsm.handle('pause');
 	};

 	function resumeHandler(e) {
 		fsm.handle('resume');
 	};

 	var fsm = new machina.Fsm({

		initialState : "uninitialized",

		states: {
			uninitialized: {},
			initializing: {

				_onEnter: function() {
					
					

					var self = this;

					document.addEventListener('pause', pauseHandler, false);
					document.addEventListener('resume', resumeHandler, false);

					
					// Setup Local Notification Listeners
					if(typeof cordova !== 'undefined' && cordova.plugins && cordova.plugins.notification && cordova.plugins.notification.local) {

						cordova.plugins.notification.local.on('trigger', function (notification) {
						    
						    JET.trigger('localnotification.trigger', [notification.id, notification.state, notification.data]);
						    
						}, this);

						cordova.plugins.notification.local.on('click', function (notification) {

						    JET.trigger('localnotification.click', [notification.id, notification.state, notification.data]);

						}, this);

					}

					// ------------------------------------------------------------------------------------------

					// Initialise BG Location
					Radar.init(); //XXX
                    
                    
					// Register Push Notifications
					if(localStorage['iospushtoken'] || localStorage['gcmpushtoken']) {
						JET.registerPush();
					}

					// Check, ob und welche Navigationsapps installiert sind.
					if(window['appAvailability']) {

						if(device.platform == 'Android' || device.platform == 'android') {

							appAvailability.checkBool('com.google.android.apps.maps', function(availability) {
							    // availability is either true or false
							    JET.googleMaps = !!availability;
							});

							JET.navigon = JET.tomtom = JET.appleMaps = false;

						} else {

							appAvailability.checkBool('comgooglemaps://', function(availability) {
							    // availability is either true or false
							    JET.googleMaps = !!availability;
							});
							appAvailability.checkBool('tomtomhome://', function(availability) {
							    // availability is either true or false
							    JET.tomtom = !!availability;
							});
							appAvailability.checkBool('navigon://', function(availability) {
							    // availability is either true or false
							    JET.navigon = !!availability;
							});
							JET.appleMaps = true;

						}
						
					}

					// Update StationData (if needed)
					// Check LoginStatus
					// Load UserData

					Stations.update().done(function() {
						$('#initial-loading').remove();
						self.transition('bindListeners');
					}).fail(function(jqXHR) {
						$('#main-body').removeClass('is-loading');

						var message;
						if(jqXHR && isOfflineError(jqXHR)) {
							message = '<b>Derzeit keine Internetverbindung!</b><br />Die Daten können nicht geladen werden, da aktuell keine ausreichende Internetverbindung besteht.';
						} else {
							message = '<b>Ein Fehler ist aufgetreten!</b><br />Die Daten konnten nicht geladen werden. <br />Bist Du mit dem Internet verbunden?';
						}


						$('#initial-loading')
							.css('top', 0)
							.html('<span style="font-size:14px">'+message+'</span>')
							.after('<div id="initial-reload"><button class="button-primary" onclick="location.reload()">Erneut versuchen</button></div>');
					});

				},

				'*': function() {
					this.deferUntilTransition();
				}
				
			},
			bindListeners: {
				_onEnter: function() {

					JET.trigger('app.bindListeners');

					
					document.addEventListener("offline", offlineHandler, false);
					document.addEventListener("online", onlineHandler, false);



					this.transition('ready');
				},
				'*': function() {
					this.deferUntilTransition();
				}
			},
			unbindListeners: {
				_onEnter: function() {

					JET.trigger('app.unbindListeners');

					document.removeEventListener("offline", offlineHandler, false);
					document.removeEventListener("online", onlineHandler, false);

					this.transition('paused');	

				},
				'*': function() {
					this.deferUntilTransition();
				}
			},
			ready: {
				_onEnter: function() {
					
					console.log('ready');
					JET.trigger('app.ready');

					if(!isVersionChecked) {

						$.ajax({

				    		type : "GET",
				    		url: JET.backendURL,
				    		data: {
				    			'action': 'check_version',
				    			'version': app_version
				    		},
							dataType : "json",
							timeout : 5000,
							cache: false

				    	}).done(function(result) {
				    		if(result.status !== 'OK') {
				    			UI.alert(result.msg);
				    		}
				    	});
				    	isVersionChecked = true;

					}

				},
				'pause': 'unbindListeners',
				'*': function() {
					this.deferUntilTransition();
				}
			},
			paused: {
				_onEnter: function() {
					JET.trigger('app.paused');
				},
				'resume': 'bindListeners',
				'*': function() {
					this.deferUntilTransition();
				}
			}
		},

		init: function() {
			this.transition('initializing');
		}

	});

	return fsm;

}(jQuery));