var JET = (function($) {

	/**
	 * Hier werden alle registrierten Controller gespeichert.
	 */
	var controllers = {};

	/**
	 * Registriert einen Controller.
	 *
	 * @var string   key        Der Name des Controllers
	 * @var function controller Die Controllerfunktionen
	 * @return object 
	 */
	function ViewController(key, controller) {

		    // In diesen Container wird der Controllerview geladen.
		var $container = $('<div class="view hidden" />'), 
			// Hierin wird die Rückgabe des Controllers gespeichert
			config,
			// Hilfsvariable, die prüft, ob der HTML-View geladen ist.
			isLoaded = false;

		/**
		 * Wird aufgerufen, wenn ein Controller angezeigt werden soll.
		 * Lädt den View und stellt diesen als Template zur Verfügung (falls noch nicht geschehen)
		 */
		function viewWillAppear(args) {

			var _e;

			// Füge Container zum DOM hinzu
			$('#main-body').append($container);

			// Trigger reflow / repaint um einen Androidbug zu umgehen, der unter manchen Umständen eine nicht korrekte Darstellung zur Folge hatte
			_e = $container[0].offsetHeight;

			// View already loaded?
			if(isLoaded) {
				config.viewWillAppear && config.viewWillAppear(args);
				return this;
			}

			// Erstelle den Controller und speichere Rückgabe
			config = controller($container);

			// Lade Template
			getTemplate(config.template).done(function(html, shouldReload) {

				// Hilfsvariable
				var $wrapper;

				// Sind "Subtemplates" definiert?
				if(html.indexOf('data-template')) {

					// Die Subtemplates werden als "models" im config Objekt gespeichert.
					config.models = config.models || {};	

					// jQuery benötigt ein umgebendes HTML-Element um auf die Kinder zugreifen zu können.
					$wrapper = $('<div />').html(html);

					// Finde alle Templates
					$('[data-template]', $wrapper).each(function() {

						var $this = $(this), 
							tpl_name = $this.data('template'),
							template = _.template(_.unescape($this.html()));

						// Erzeuge Funktion zum rendern des Templates
						config.models[tpl_name] = function(data) {
							$container.find('[data-template="'+tpl_name+'"]').hide().html(template(data)).show();
						}

						// Rendere Template mit initialen Daten (nicht genutzt)
						$this.html(template(
							config.template_defaults && config.template_defaults[tpl_name] || null
						));

					});

					// Füge HTML zum Container hinzu
					$container.html($wrapper.html());

				} else {

					// Nein, keine Subtemplates.
					// Füge den gesamten Quellcode zum Container hinzu.
					$container.html(html);

				}

				// Trigger reflow / repaint um einen Androidbug zu umgehen, der unter manchen Umständen eine nicht korrekte Darstellung zur Folge hatte	
				_e = $container[0].offsetHeight;

				// Rufe Callback auf
				config.templateLoaded && config.templateLoaded();

				// Falls es einen Fehler beim Laden eines Templates gab (siehe getTemplate()), wird anstatt des Templates eine Fehlermeldung zurückgegeben.
				// In diesem Fall soll beim nächsten Aufruf erneut versucht werden, das Template zu laden.
				if(shouldReload === true) {
					isLoaded = false;
				}

			});

			// Setze Variable, damit das Template nicht erneut geladen werden muss.
			isLoaded = true;
			
			// Call Lifecycle Method (falls definiert)
			config.viewWillAppear && config.viewWillAppear(args);

			return this;
		};

		// Der Controller ist dem DOM hinzugefügt
		function viewDidAppear() {
			config.bindEvents && config.bindEvents();
			return this;
		};

		// Der Controller wird gleich entfernt werden
		function viewWillDisappear() {
			config.unbindEvents && config.unbindEvents();
			return this;
		};

		// Der Controller wird entfernt
		function viewDidDisappear() {

			JET.off('.'+key);

			$container.toggleClass('active hidden');

			config.viewDidDisappear && config.viewDidDisappear();

			$container.remove();

			return this;
		};

		// Public API
		return {

			refresh: function() {
				config['refresh'] && config.refresh();
			},

			getView: function() {
				return $container;
			},
			getTitle: function() {
				return config.title;
			},

			viewWillAppear: viewWillAppear,
			viewDidAppear: viewDidAppear,
			viewWillDisappear: viewWillDisappear,
			viewDidDisappear: viewDidDisappear

		};

	};

	// Lädt ein Template mit dem angegebenen Namen
	// Gibt einen jQuery Promise zurück
	function getTemplate(name) {

		var deferred = $.Deferred();

		$.ajax({
			cache: true,
			dataType: 'text',
			isLocal: true,
			timeout: 5000,
			type: 'GET',
			url: name.substr(0, 4) === 'http' ? name : 'tpl/'+name
		}).done(function(html) {

			deferred.resolve(html);

		}).fail(function(jqXHR, textStatus, errorThrown) {

			var errorText = '<b>Ein Fehler ist aufgetreten!</b><br />Die Seite kann nicht angezeigt werden.';
			if(isOfflineError(jqXHR)) {
				errorText = '<b>Derzeit keine Internetverbindung!</b><br />Die Seite kann nicht angezeigt werden, da aktuell keine ausreichende Internetverbindung besteht. Bitte versuche es später erneut.';
			}

			deferred.resolve('<p class="template-error padded template-error-'+textStatus+'">'+errorText+'</p>', true);

		});


		return deferred.promise();

	}

	// Registriert die App für Push-Mitteilungen.
	function registerPush() {

		if(typeof PushNotification === 'undefined') return;

		var push = PushNotification.init({
			android: {
				senderID: '616614095654',
				iconColor: '#fbba00'
			},
			ios: {
				sound: true
			}
		});

		push.on('registration', function(data) {

			if(isAndroid()) {
				localStorage['gcmpushtoken'] = data.registrationId;
				User.setData('gcm_token', data.registrationId);
				User.setData('ios_token', '');
			} else {
				localStorage['iospushtoken'] = data.registrationId;
				User.setData('ios_token', data.registrationId);
				User.setData('gcm_token', '');
			}
			User.setData('push_token', '1');
			savePushToken();

		});

		push.on('notification', function(data) {
			if(data.additionalData.foreground) {
				
				
				UI.alert({
					body: data.message,
					okbutton: 'Anzeigen',
					cancelbutton: 'Abbrechen'
				}, function showStation() {
					UI.closeNav();
					UI.show('Tankstellensuche', data.additionalData.station);
				}, function noop() {});
				
			} else {

				UI.show('Tankstellensuche', data.additionalData.station);

			}
		});

		push.on('error', function(e) {

			UI.alert(e.message);

		});

		
	};

	// Dieser Callback wird aufgerufen, wenn eine Pushnachricht empfangen wird (iOS)
	function pushCallbackAPN(e) {
		if(e.station) {
			// Zeige Seite "Tankstellensuche", übergebe Stationsnummer als Argument.
			UI.show('Tankstellensuche', e.station);
		} else if(e.alert || e.body) {
			// Kann dieser Code jemals aufgerufen werden?
			// Zeige Nachricht
			UI.alert({
				body: e.alert || e.body,
				okbutton: 'Anzeigen',
				cancelbutton: 'Abbrechen'
			}, function() {
				// Klick auf "Anzeigen" ruft die Seite "Tankstellensuche" auf und übergibt die angegebene Stationsnummer
				UI.show('Tankstellensuche', e.station);
			});
		}
	};

	// Dieser Callback wird aufgerufen, wenn eine Pushnachricht empfangen wird (Android).
	// Ausserdem wird die Funktion aufgerufen, wenn ein Pushtoken registriert wurde oder ein Fehler aufgetreten ist.
	function pushCallbackGCM(e) {
		switch(e.event) {

			case 'registered': // Pushtoken registriert

				if(e.regid.length > 0) {
					JET.trigger('GCM.token', e.regid);
					// Speichere Token in Localstorage
					window.localStorage['gcmpushtoken'] = e.regid;
					// Sende Token an Server
					savePushToken();
				}

			break;

			case 'message': // Pushnachricht empfangen
				
				if(e.payload.type == 'radar') { // Dies gibt es so nicht mehr...
					UI.show('Tankstellensuche', e.payload.id);
				} else if(e.payload.type == 'preisalarm') {
					// Zeige Seite "Tankstellensuche", übergebe Stationsnummer als Argument.
					UI.show('Tankstellensuche', e.payload.station);
				} else {
					// Hm... 
					// Eigentlich gibt es nur die Preisalarmmessages.
					// Wahrscheinlich ein Überbleibsel aus Testzeiten
					UI.alert(e.payload.message);
				}
			break;

			case 'error': // Fehler, Pushtoken nicht erhalten
				UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />'+e.msg);

				// Speichere Token in Localstorage
				window.localStorage['gcmpushtoken'] = '';
				// Sende Token an Server
				savePushToken();

			break;

			default: // Unbekannter Fehler

				UI.alert('<b>Bei dem Versuch Ihr Gerät für Pushnachrichten zu registrieren, ist ein unbekannter Fehler aufgetreten!</b>');

				// Speichere Token in Localstorage
				window.localStorage['gcmpushtoken'] = '';
				// Sende Token an Server
				savePushToken();

			break;

		}
	};

	return $.extend({

		backendURL: 'https://www.jet-tankstellen.de/app_api/',
		baseURL: 'https://www.jet-tankstellen.de/',
		defaultProfilePicture: 'img/klicke-profile-picture.png',
		defaultCarPicture: 'img/klicke-nocar.png',
		pricecode: function(code) {

			var _name;

			switch(code) {
				case 'L001':
					_name = 'Super Plus';
				break;
				case 'L003':
					_name = 'Diesel';
				break;
				case 'L004':
					_name = 'Super';
				break;
				case 'L005':
					_name = 'Super E10';
				break;
				case 'Hybrid':
					_name = 'Hybrid';
				break;
				case 'CNG':
					_name = 'Erdgas';
				break;
				case 'LPG':
					_name = 'Autogas';
				break;
			}

			return _name;

		},

		formatPrice: function(price) {

			return price.slice(0,-1) + price.slice(-1).sup();

		},

		controller: function(name, spec) {
			controllers[name] = new ViewController(name, spec);
		},
		getController: function(name) {
			return controllers[name];
		},
		isNewRegistration: false,

		registerPush: registerPush,
		pushCallbackAPN: pushCallbackAPN,
		pushCallbackGCM: pushCallbackGCM,
		googleMaps: false,
		tomtom: false,
		navigon: false,
		appleMaps: false

	}, $({}));

}(jQuery));
