var LocationService = (function($, JET) {

	    // google.maps.Geocoder
	var geocoder,

		// deprecated
		watchID = null,

		// hier werden die aktuellen lat/lng bounds der Google Map auf der Seite "Tankstellenfinder" gecacht.
		map_bounds,

		// jQuery Cache für den "Bitte meine Position ermitteln"-Button
		$locateButton,

		// Ist ein geolocation request gestartet?
		isLocating = false,

		// Letzte bekannte Position
		lastPosition = null;

	// Im Tankstellensuche-Controller wird dieses Event ausgelöst,
	// wann immer die Karte bewegt wird.
	// Die aktuellen google.maps.LatLngBounds werden hier gespeichert.
	JET.on('map.bounds', function(e, bounds) {
		map_bounds = bounds;
	});

	/**
	 * Startet eine google.maps.Geocoder.geocode Abfrage
	 *
	 * @var string Der Suchbegriff
	 */
	function search(string) {

		// Löse Event aus.
		JET.trigger('geocoder.start', [string]);

		// Erstelle Geocoder Objekt
		if(!geocoder) {
			geocoder = new google.maps.Geocoder();
		}

		var geo_options = {
			address: string,
			region: 'de',
			componentRestrictions: {
				country: 'de'
			}
		};

		// Falls wir gecachte LatLngBounds haben,
		// wird die Suche auf diesen Bereich eingeschränkt.
		// Dadurch werden genauere Ergebnisse geliefert.
		if(map_bounds) {
			geo_options.bounds = map_bounds;
		}

		// Los geht's
		geocoder.geocode(geo_options, geocoder_result);
	};

	/**
	 * Diese Funktion wird als Callback durch google.maps.Geocoder.geocode aufgerufen
	 */
	function geocoder_result(results, status) {

		if(status == google.maps.GeocoderStatus.OK) {

			
			if(results.length > 1) { 
			
				// Nicht eindeutiges Ergebnis

				var returnValues = [];

				for(var i=0, result; result = results[i]; ++i) {

					returnValues.push({
						address: result.formatted_address,
						location: result.geometry.location,
						viewport: result.geometry.viewport
					});

				}

				JET.trigger('geocoder.multiple_results', [returnValues]);

			} else if(results[0].formatted_address === 'Deutschland') {

				/**
				Spezialfall:
				Da wir die Geocodersuche auf "DE" einschränken,
				bekommen wir für alle nicht gefundenen Adressen "Deutschland" zurückgeliefert...

				Das sehe ich mal als Fehler an!

				*/

				console.log('Location not found error', results);

				JET.trigger('geocoder.error', 'Die Adresse konnte nicht gefunden werden.');

			} else {

				// Wir haben ein eindeutiges Ergebnis.

				var result = results[0], geometry = result.geometry;

				JET.trigger('geocoder.result', {
					'address': result.formatted_address,
					location: geometry.location,
					viewport: geometry.viewport
				});

			}


		} else {

			// Fehler

			var msg = '';

			switch(status) {
				
				case google.maps.GeocoderStatus.ZERO_RESULTS:
					msg = 'Die Adresse konnte nicht gefunden werden.';
				break;

				default:

					if(!isConnected()) {
						msg = '<b>Derzeit keine Internetverbindung!</b><br />Die Aktion kann nicht korrekt ausgeführt werden, da aktuell keine ausreichende Internetverbindung besteht. Bitte versuche es später erneut.';
					} else {

						msg = '<b>Ein Fehler ist aufgetreten!</b><br />Die Abfrage kann derzeit nicht ausgeführt werden. Bitte versuche es später erneut.';

					}

				break;
			}

			JET.trigger('geocoder.error', msg);
		}

	};

	/**
	 * Versucht die Position des Benutzers mit hoher Genauigkeit zu ermitteln.
	 */
	function locate() {

		if(isLocating) {
			return;
		}
		isLocating = true;

		JET.trigger('geolocation.start');

		Radar.getCurrentPosition(geoSuccess, geoError);
		
	};

	/**
	 * Versucht auf die schnellste Art die Position des Benutzers zu ermitteln.
	 * Ist eine letzte Position bekannt (setLastPosition), wird diese genutzt.
	 * Andernfalls wird versucht, die Position mit geringer Genaugkeit zu ermitteln.
	 */
	function getPosition() {
		
		if(isLocating === true) {
			return;
		}

		isLocating = true;

		JET.trigger('geolocation.start');	

		if(lastPosition !== null) {

			return geoSuccess({
				coords: {
					latitude: lastPosition.lat,
					longitude: lastPosition.lng
				}
			});
		}

		Radar.getCurrentPosition(geoSuccess, geoError);
		
	};

	function androidSwitchToHighAccuracy(error) {

		if(error.code == 2 || error.code == 3) { // Modul ist aus

			console.log('Switching to High Accuracy');

			// Versuchen wir unser Glück mit GPS.
			// Manchmal funktioniert's...
			setTimeout(function() {

				navigator.geolocation.getCurrentPosition(geoSuccess, geoError, {
					enableHighAccuracy: true,
					maximumAge: 1000 * 60 * 30,
					timeout: 1000 * 30
				});

			}, 100);

		} else {

			// Nix gefunden ODER nicht erlaubt (findet bei Android nicht statt?)
			geoError(error);
		}

	};

	// Wird zur Zeit nicht genutzt.
	// Schaltet eine Geolocation-Abfrage von enableHighAccuracy: true auf false.
	function androidSwitchToLowAccuracy(error) {

		if(error.code == 2 || error.code == 3) {
			
			console.log('Switching to Low Accuracy');

			setTimeout(function() {

				navigator.geolocation.getCurrentPosition(geoSuccess, geoError, {
					enableHighAccuracy: false,
					maximumAge: 1000,
					timeout: 1000 * 10
				});

			}, 100);

		} else {
			geoError(error);
		}

	}

	/**
	 * Callback, wenn eine Geoposition gefunden wurde.
	 */
	function geoSuccess(position) {
		
		isLocating = false;

		var coords = position.coords;
		setLastPosition(coords.latitude, coords.longitude);
		JET.trigger('geolocation.success', coords);

	};

	/**
	 * Callback, wenn keine Position gefunden wurde.
	 */
	function geoError(error) {

		isLocating = false;

		var msg = '';

		switch(error.code) {
			case 1:
				msg = 'Bitte erlaube der JET App den Zugriff auf Deine Positionsdaten.';
				if(isAndroid() === false) {
					msg += ' Einstellungen > Datenschutz > Ortungsdienste';
				}
			break;

			default:
				
				msg = '<b>Deine Position wurde nicht gefunden.</b><br />';
				if(isAndroid()) {


					msg += 'Bitte GPS / Standortzugriff aktivieren und anschließend "Nochmal versuchen".';


				} else {
					// iOS
					msg += 'Möchtest Du es "Nochmal versuchen"?';
				}
			break;

		};

		
		// deprecated
		if(watchID) {
			navigator.geolocation.clearWatch(watchID);
			watchID = null;

			if(!$locateButton) {
				$locateButton = $('#locate');
			}

			$locateButton.removeClass('is-active');
			JET.trigger('geolocation.stop');
		}

		JET.trigger('geolocation.error', [msg, error]);


	};


	/**
	 * Setzt die letzte Position des Users.
	 */
	function setLastPosition(lat, lng) {
		lastPosition = {
			lat: lat,
			lng: lng
		};
	};


	return {
		search: search,
		locate: locate,
		getPosition: getPosition,
		setLastPosition: setLastPosition
	}


}(jQuery, JET));