var Radar = (function() {

		// Die zuletzt gefunden Tankstelle
	var currentStation = null,

		// Return value von navigator.geolocation.watchPosition
		radarWatchID = null,

		// ID der local notification alle Fehlermeldungen.
		// Dadurch, das wir immer die gleiche ID nutzen, wird die Meldung überschrieben und es erscheinen nicht so viele.
		errorNotificationMessageID = '47110815',

		_noop = function() {},

		backgroundGeoLocation;

	

	/**
	 * Callback Funktion, wenn eine neue Position gefunden wurde.
	 */
	function bgLocationSuccessCallback(position, taskID) {

		if(position.hasOwnProperty('extras') && position.extras.hasOwnProperty('radar') && position.extras.radar === false) {
			backgroundGeoLocation.finish(taskID);
			return;
		}

		var location = position.coords;

		console.log('Location Update '+location.latitude+' '+location.longitude);

		// Lösche alle Local Notifications
		cordova.plugins.notification.local.cancel(errorNotificationMessageID);
		
		// Der folgende Code ist (mehr oder weniger) aus Stations.findNearestStation kopiert.
		// Da wir hier aber einen Callback haben, der im Hintergrund läuft, wollte ich sichergehen,
		// das die Funktion auch da ist, wenn Memory knapp wird.
		// Vielleicht funktioniert es aber auch Prima, wenn man einfach 
		// Stations.findNearestStation(location.latitude, location.longitude, 50) aufruft.

		// Suche die nächstgelegene Station im Umkreis von 50 km (auf dem Land sind manchmal nur wenige JET)

		var R = 6371,  // km
			P = Math.PI / 180,

			sin  = Math.sin,
			cos  = Math.cos,
			sqrt = Math.sqrt,

			dLat, dLng, lat_a, lat_b, a, c,

			filtered = [],

			stations = Stations.stations(),
			station_number,
			station;

		for(station_number in stations) {

			station = stations[station_number];

			dLat = sin(((station.lat - location.latitude) * P) / 2);
			dLng = sin(((station.lng - location.longitude) * P) / 2);
			lat_a = cos(location.latitude * P);
			lat_b = cos(station.lat * P);
			
			a = dLat * dLat + dLng * dLng * lat_a * lat_b;
			c = 2 * Math.atan2(sqrt(a), sqrt(1-a));

			station.distance = R * c;

			if(station.distance < 50) {
				filtered.push(station);
			}
		}

		if(filtered.length > 0) {

			filtered.sort(function(a, b) {

				return a.distance < b.distance ? -1 : a.distance > b.distance ? 1 : 0;

			});

			// Wenn die nächste JET nicht die ist, die wir gerade anzeigen
			if(filtered[0].station_number != currentStation) {

				if(App.state === 'paused') {

					// Hintergrund
					// Initiiere LocalNotification
					cordova.plugins.notification.local.schedule({
						id: (10000 + parseInt(filtered[0].station_number.split('XJ0')[1])),
						title: 'JET Tankstellenradar',
						text: filtered[0].street + ' ('+format('#.##0,0', filtered[0].distance)+' km)',
						icon: 'res://ic_stat_jet_plain.png',
						led: 'FBBA00',
						data: {
							type: 'radar',
							id: filtered[0].station_number,
							lat: location.latitude, 
							lng: location.longitude
						}
					});



				} else {

					// App ist im Vordergrund
					// Zeige Alert

					var alertBody = '<b>JET Tankstellenradar</b><br />';
					alertBody += filtered[0].street + ' ('+format('#.##0,0', filtered[0].distance)+' km)';

					UI.alert({
						body: alertBody,
						okbutton: 'Anzeigen',
						cancelbutton: 'Abbrechen'
					}, function() {
						UI.closeNav();
						UI.show('Tankstellensuche', {
							station_number: filtered[0].station_number,
							lat: location.latitude,
							lng: location.longitude
						});
					}, _noop);
				}
				
				
				if(currentStation) {
					cordova.plugins.notification.local.cancel((10000 + parseInt(currentStation.split('XJ0')[1])), _noop)
				}
				

				currentStation = filtered[0].station_number;
				
			}
		}

		backgroundGeoLocation.finish(taskID);

	};

	/**
	 * Fehler bei der Geocodierung
	 */
	function bgLocationErrorCallback(error) {
		console.log('BackgroundLocationError', error);

		if(error == 1 || error.code == 20) { 
			// Keine Rechte
			disable();
			if(App.state == 'ready') {
				UI.alert('Bitte der JET App die Berechtigung geben, auf Deine Positionsdaten zuzugreifen.');
				
			} else {
				
				cordova.plugins.notification.local.schedule({
					id: errorNotificationMessageID,
					title: 'JET Tankstellenradar',
					text: 'Das JET Tankstellenradar wurde deaktiviert.',
					icon: 'res://ic_stat_jet_plain.png',
					led: 'FBBA00',
					data: {
						type: 'radarerror',
						msg: 'Bitte der JET App die Berechtigung geben, auf Deine Positionsdaten zuzugreifen. Das JET Tankstellenradar wurde deaktiviert.'
					}
				});
			}
			return;
		} else if(error == 408 || error.code == 2 || error.code == 3) {
			
			// Timeout
			// Mal fragen....

			if(App.state == 'ready') {

				// Vordergrund

				var msg = '<b>Deine Position wurde nicht gefunden.</b><br />';
				if(isAndroid()) {


					msg += 'Ist das GPS aktiviert? ';


				} 
				msg += 'Das Radar versucht weiterhin Deine Position zu ermitteln.';

				UI.alert({
					body:msg,
					okbutton: 'OK',
					cancelbutton:'Radar beenden'
				}, _noop, function() {
					Radar.disable();
				});
				
			} else {

				// Hintergrund

				var msg = 'Deine Position wurde nicht gefunden.';

				var msg2 = '<b>Deine Position wurde nicht gefunden.</b><br />';
				if(isAndroid()) {


					msg2 += 'Ist das GPS aktiviert? ';


				} 
				msg2 += 'Das Radar versucht weiterhin Deine Position zu ermitteln.';

				
				cordova.plugins.notification.local.schedule({
					id: errorNotificationMessageID,
					title: 'JET Tankstellenradar',
					text: msg,
					sound: null,
					icon: 'res://ic_stat_jet_plain.png',
					led: 'FBBA00',
					data: {
						type: 'radarerror',
						msg: msg2
					}
				});
			}
			return;
		}

	};
	
	/**
	 * Starte das Radar
	 */ 
	function enable(forceLowAccuracy) {

		

		if(localStorage['radarWatchID'] !== null) {
			navigator.geolocation.clearWatch(radarWatchID);
			radarWatchID = null;
		}

		

		backgroundGeoLocation.start(function bgGeoStartSuccess() {
		
			backgroundGeoLocation.getCurrentPosition(_noop, _noop, {
				maximumAge: 1000 * 60 * 5
			});

		}, function bgGeoStartError() {


			var msg = '<b>Das Radar konnte nicht gestartet werden..</b><br />';
			if(isAndroid()) {


				msg += 'Ist das GPS aktiviert? ';


			} 
			msg += 'Das JET Tankstellenradar wurde deaktiviert.';

			if(App.state == 'ready') {
				

				UI.alert({
					body:msg,
					okbutton: 'OK'
				}, function() {
					Radar.disable();
				});

			} else {

				cordova.plugins.notification.local.schedule({
					id: errorNotificationMessageID,
					title: 'JET Tankstellenradar',
					text: 'Das JET Tankstellenradar wurde deaktiviert.',
					icon: 'res://ic_stat_jet_plain.png',
					led: 'FBBA00',
					data: {
						type: 'radarerror',
						msg: msg
					}
				});

			}

		});	
		radarWatchID = 1;


		// Aktualisiere UI
		var $radarButton = $('#nav ul').find('[data-page="Tankstellenradar"] button'),
			$radarIcon = $radarButton.find('.fa'),
			$radarText = $radarButton.find('span');

		$radarIcon.removeClass('fa-square-o').addClass('fa-check-square');
		$radarText.text('an');
		
		// Speichere Status und ID
		localStorage['radar'] = 'enabled';
		localStorage['radarWatchID'] = radarWatchID;
		
		// Analytics
		ga.trackEvent('Radar', 'enable');

	};

	/**
	 * Staoppe Radar
	 */
	function disable() {

		console.log('radarWatchID', radarWatchID);

		// Beende Prozess
		navigator.geolocation.clearWatch(radarWatchID);
		backgroundGeoLocation.stop();
		backgroundGeoLocation.resetOdometer();
		backgroundGeoLocation.clearDatabase();
		radarWatchID = null;

		// Aktualisiere UI
		var $radarButton = $('#nav ul').find('[data-page="Tankstellenradar"] button'),
			$radarIcon = $radarButton.find('.fa'),
			$radarText = $radarButton.find('span');

		$radarIcon.removeClass('fa-check-square').addClass('fa-square-o');
		$radarText.text('aus');

		// Speichere Status und lösche ID
		localStorage['radar'] = 'disabled';
		localStorage['radarWatchID'] = null;

		currentStation = null;

	};


	function init() {
		
		backgroundGeoLocation = window.BackgroundGeolocation;

		/*
		Konfiguration des Plugins (https://github.com/cliqueApp/cordova-background-geolocation/blob/master/docs/api.md)
		*/
		backgroundGeoLocation.configure(bgLocationSuccessCallback, bgLocationErrorCallback, {
			// Geolocation config
	        desiredAccuracy: 10, // 0, 10, 100, 1000
	        distanceFilter: 30,
	        stopAfterElapsedMinutes: 0,
	        stationaryRadius: 10,
	        disableElasticity: true, // <-- [iOS] Default is 'false'.  Set true to disable speed-based distanceFilter elasticity
	        activityType: 'AutomotiveNavigation',
	        useSignificantChangesOnly: false,
	        preventSuspend: true,
	        heartbeatInterval: 60,

	        activityRecognitionInterval: 10000,
	        stopTimeout: 1,  // Wait x miutes to turn off location system after stop-detection
	        minimumActivityRecognitionConfidence: 50,   // 0-100%.  Minimum activity-confidence for a state-change 
	        stopDetectionDelay: 0,  // Wait x minutes to engage stop-detection system
	        disableMotionActivityUpdates: true,
	        
	        locationUpdateInterval: 5000,
	        fastestLocationUpdateInterval: 5000,
	        activityRecognitionInterval: 10000,

	        // Application config
	        debug: false, // <-- enable this hear sounds for background-geolocation life-cycle.
	        stopOnTerminate: true,              // <-- [Android] Allow the background-service to run headless when user closes the app.
	        forceReloadOnLocationChange: false,  // <-- [Android] If the user closes the app **while location-tracking is started** , reboot app when a new location is recorded (WARNING: possibly distruptive to user) 
	        forceReloadOnMotionChange: false,    // <-- [Android] If the user closes the app **while location-tracking is started** , reboot app when device changes stationary-state (stationary->moving or vice-versa) --WARNING: possibly distruptive to user) 
	        forceReloadOnGeofence: false,        // <-- [Android] If the user closes the app **while location-tracking is started** , reboot app when a geofence crossing occurs --WARNING: possibly distruptive to user) 
	        startOnBoot: false
		});
	}

	return {

		init: init,
		enable: enable,
		disable: disable,
		success: bgLocationSuccessCallback,
		error: bgLocationErrorCallback,
		getCurrentPosition: function(success, error) {
			backgroundGeoLocation.getCurrentPosition(function successCallback(location, taskID) {
				backgroundGeoLocation.finish(taskID);
				success(location);
			}, function errorCallback(e) {
				error(e);
			}, {
				timeout: 30,
				maximumAge: 1000 * 30,
				extras: { radar: false }
			});
		},
		status: function() {
			var isRadarEnabled = localStorage['radar'];

			return isRadarEnabled === 'enabled' ? 'enabled' : 'disabled';
		}

	};

}());