JET.controller('BoxenstoppNoLogin', function(el) {

	var config = {
		title: 'Boxenstopp',
		template: 'Boxenstopp_nologin.html'
	};

	return config;

});

JET.controller('Boxenstopp', function(el) {

	var templateLoaded = false,
		isInitialView = true,

		// Garage
		$collectionView,

		// Autodetail
		$detailView,

		fuelChart,
		expenseChart,

		currentCar,
		expenses = null;

	var config = {
		title: 'Boxenstopp',
		template: 'Boxenstopp.html',
		templateLoaded: function() {

			templateLoaded = true;

			// Cache die Views
			$collectionView = el.find('#car-collection');
			$detailView = el.find('#car-detail');

			render();

		},
		viewWillAppear: function() {

			$('#back-search').addClass('is-search').removeClass('is-back');

			if(templateLoaded) {
				render();
			}

		},
		refresh: function() {

			if(currentCar) {

				// Wenn das Auto gelöscht wurde, leite zurück auf Garage

				currentCar = _.find(User.getData('cars'), function(car) {
					return car.id == currentCar.id;
				});

				if(!currentCar) {
					el.removeClass('show-now');
					hideCarDetail();
				}
			}

			render();
		},
		bindEvents: function() {

			// Für den Scale-Effekt
			el.addClass('z');

			// Eventhandler der das Bearbeitungsformular anzeigt.
			el.on('click.boxenstopp', '[data-modal="BoxenstoppEditCar"]', function(e) {

				// Wenn der User auf "Auto hinzufügen" geklickt hat, haben wir keine CarID
				// Andernfalls wird diese an das BoxenstoppEditCar übergeben

				UI.modal(
					'BoxenstoppEditCar',
					this.getAttribute('data-carid') || null
				);

			});

			// Eventhandler für die Auswahl eines Autos
			el.on('click.boxenstopp', 'li[data-index]', function(e) {

				currentCar = User.getData('cars')[parseInt(this.getAttribute('data-index'), 10)];

				// Zeige Detailseite
				showCarDetail(currentCar);

			});

			// Eventhandler für "Neuer Tankvorgang"
			el.on('click.boxenstopp', '.verbrauch-info .button-primary', function(e) {
				UI.modal(
					'BoxenstoppAddFuel',
					{car:currentCar, fuel: null}
				);
			});

			// Eventhandler der die Liste aller Tankvorgänge anzeigt
			el.on('click.boxenstopp', '.verbrauch-info .button-secondary', function(e) {
				toggleFuelList();
			});

			// Eventhandler für "Neue Ausgabe"
			el.on('click.boxenstopp', '.kosten-info .button-primary', function(e) {
				UI.modal(
					'BoxenstoppAddExpense',
					{car:currentCar, expense: null}
				);
			});

			// Eventhandler der die Liste aller Ausgaben anzeigt
			el.on('click.boxenstopp', '.kosten-info .button-secondary', function(e) {
				toggleExpenseList();
			});

			// Eventhandler, der eine Ausgabe löscht
			el.on('click.boxenstopp', '.button-delete', function(e) {

				var $row = $(this).parents('tr'),
					id = $row.data('id');

				// Kein Internet?
				if(!isConnected()) {
					return displayNoConnectionError();
				}

				// API Call
				$.post(JET.backendURL, {
					action: 'deleteExpense',
					id: id,
					token: User.getToken(),
					car_id: currentCar.id
				}).done(function(data) {

					// Entferne Zeile aus der Tabelle
					$row.remove();

					// Aktualisiere Grafiken
					getExpenseCallback(data);

				}).fail(function(jqXHR) {

					// Internet?
					if(isOfflineError(jqXHR)) {
						return displayNoConnectionError();
					}

					// Zeige Fehlermeldung
					UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />Die Daten konnten nicht gelöscht werden.');

				});

			});

			// Eventhandler zum bearbeiten eines Tankvorgangs
			el.on('click.boxenstopp', '#fuellist td:not(.expense-item-delete)', function(e) {

				var $row = $(this).parents('tr'),
					id = $row.data('id');

				showExpenseModal('Fuel', id);


			});

			// Eventhandler zum bearbeiten einer Ausgabe
			el.on('click.boxenstopp', '#expenselist td:not(.expense-item-delete)', function(e) {

				var $row = $(this).parents('tr'),
					id = $row.data('id');

				showExpenseModal('Expense', id);


			});

			// Eventhandler der aus der Detailansicht zurück zur Garage geht
			JET.on('UI.back.click.boxenstopp', function(e) {
				el.removeClass('show-now');
				hideCarDetail();
			});

			// Der Klick auf ein Element der Säulengrafiken zeigt einen Tooltip
			// Ein Klick auf den Tooltip öffnet BoxenstoppAdd(Expense|Fuel)
			el.on('click.boxenstopp', '.tooltipLink', function(e) {
				
				// ID des Eintrags
				var item_id = this.getAttribute('data-id');
				var type = this.getAttribute('data-type') == 'Kraftstoff' ? 'Fuel' : 'Expense';
				showExpenseModal(type, item_id);


			});
		},
		unbindEvents: function() {

			// Entferne Eventhandler
			el.off('.boxenstopp');
			JET.off('.boxenstopp');
			$(document).off('.boxenstopp');
			// Für den Scale-Effekt
			el.removeClass('z');

			// Lösche Charts
			el.find('#fuelchart').empty();
			el.find('#expensechart').empty();

			fuelChart = null;
			expenseChart = null;
		}
	};

	/**
	 * Zeigt ein Modal zum bearbeiten einer Ausgabe an
	 * @param type Fuel|Expense
	 * @param id ID der Ausgabe
	 */
	function showExpenseModal(type, id) {

		var type_lower = type.toLowerCase();
		var expense = null;

		if(id) {
			expense = _.find(expenses[type_lower], function(item) {
				return item.id == id;
			});
		}

		var controller = 'BoxenstoppAdd' + type;
		var data = {
			car: currentCar
		};
		data[type_lower] = expense;

		UI.modal(controller, data);


	}

	/**
	 * Rendert den View
	 */
	function render() {
		
		var cars = User.getData('cars'), 
			showDetail = el.hasClass('show-detail');

		// Aktualisiere View
		config.models.carList({cars:cars});

		// Erster Aufruf und nur ein Auto?
		if(isInitialView && cars.length == 1) {
			currentCar = cars[0];
			showDetail = true;
		}

		// Soll Detailseite angezeigt werden?
		if(showDetail) {
			el.addClass('show-now');
			showCarDetail(currentCar);
			isInitialView = false;
		}

	}

	/**
	 * Zeigt Detailansicht eines Autos
	 * @param car Object mit den Autoinfos
	 */
	function showCarDetail(car) {

		config.models.carDetail({car: car});
		
		setTimeout(function() {

			// Zeige Backbutton
			UI.handle('show.back');
			UI.setTitle('Detail');
		
			el.addClass('show-detail');

			// Lade Ausgaben vom Server
			getCarExpenses();

		}, 0);

		

	};

	/**
	 * Lädt die Ausgaben für das aktuelle Auto und erzeugt die Charts
	 * @returns {*}
	 */
	function getCarExpenses() {

		// Kein Internet=
		if(!isConnected()) {
			// Wenn wieder Online, lade erneut
			$(document).one('online.boxenstopp', getCarExpenses);

			// Erstelle leere Charts
			getExpenseCallback({expenses: {
				fuel:[],
				expense: [],
				fuelaverage:0,
				fueltotal:0,
				expensetotal:0,
				expenseaverage:0
			}});

			// Zeige Fehlermeldung
			return displayNoConnectionError(true);
		}

		// API Call
		$.post(JET.backendURL, {
			action: 'getCarExpenses',
			car_id: currentCar.id,
			token: User.getToken()
		}).done(getExpenseCallback).fail(function(jqXHR) {

			// Fehler

			// Internet?
			if(isOfflineError(jqXHR)) {

				displayNoConnectionError(true);

				// Wenn wieder Online, lade erneut
				$(document).one('online.boxenstopp', getCarExpenses);

			} else {

				UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />Die Daten konnten nicht geladen werden.');
			}

			// Erstelle leere Charts
			getExpenseCallback({expenses: {
				fuel:[],
				expense: [],
				fuelaverage:0,
				fueltotal:0,
				expensetotal:0,
				expenseaverage:0
			}});
		});
	};

	/**
	 * Erstellt die Charts
	 * @param data
	 */
	function getExpenseCallback(data) {

		// Cache das Ergebnis
		expenses = data.expenses;

		// Erstelle Charts
		renderFuelChart(data.expenses.fuel);
		renderExpenseChart(data.expenses.expense);

		setTimeout(function() {
			// Zeige Durchschnittsverbrauch und Kostensumme
			el.find('.verbrauch-durchschnitt').text(format('#.##0,0', data.expenses.fuelaverage));
			el.find('.kosten-summe').html(
				format('#.##0,00', data.expenses.expensetotal) + '<span style="font-weight:normal;font-size:10px">, ' +
				'€ ' + 
				format('#.##0,00', data.expenses.expenseaverage) + ' / Monat</span>'
			);
		}, 0);

		if(data['user']) {

			setTimeout(function() {
				// Aktualisiere User
				User.setData(data.user);
			}, 0);

		}

	};

	/**
	 * Bereitet die Rohdaten für die Charts auf
	 * @param input Array mit den einzelnen Ausgaben
	 * @param type fuel|expenses
	 * @returns {{data: Array, maximum: number, sum: number, numMonths: number}}
	 */
	function prepareChartData(input, type) {

		var data = [],
			i, j,

			startOfMonth, 
			formattedStartOfMonth,

			expense_date, 
			expense_type,


			_temp = [],
			_yearmonth,
			_dataPoints,

			_value,

			sum = 0,
			maximum = 0,

			numMonths = 0,
			firstMonth = -1,
			lastMonth = -1;
		
		
		for(i=0, j=input.length; i<j; ++i) {

			// Ausgabenart (Kraftstoff / Steuer / Versicherung / Pflege / Werkstatt)
			expense_type = input[i].expense_type;

			// Erstelle Array für die Ausgabenart
			if(!data[expense_type]) {
				data[expense_type] = [];
			}

			// Datum der Ausgabe
			expense_date = moment(input[i].expense_date);

			// 1. des Monats der Ausgabe
			startOfMonth = moment(expense_date).startOf('month');

			// 1. des Monats der Ausgabe formatiert
			formattedStartOfMonth = startOfMonth.format('YYYYMM');

			// Erstelle Array für den Monat
			if(!data[expense_type][formattedStartOfMonth]) {
				data[expense_type][formattedStartOfMonth] = [];
			}

			// Setze Monat der ersten Ausgabe
			if(startOfMonth < firstMonth || firstMonth === -1) {
				firstMonth = startOfMonth;
			}

			// Setze Monat der letzen Ausgabe
			if(startOfMonth > lastMonth || lastMonth === -1) {
				lastMonth = startOfMonth;
			}

			// Erstelle den Datensatz
			// Doku: http://canvasjs.com/docs/charts/chart-options/data/datapoints/
			data[expense_type][formattedStartOfMonth].push({

				y: expense_type == 'Kraftstoff' ? parseFloat(input[i].amount) : parseFloat(input[i].price),
				x: startOfMonth.toDate(),
				name: expense_date.format('DD. MMMM YYYY'),
				label: getChartLabel(input[i], type),

				toolTipContent: '<div class="tooltipLink" data-id="'+input[i].id+'" data-type="'+expense_type+'"><b>{name}</b><br />{label}</div>',
				

				legendText: expense_type
			});

		}

		// Erstelle Datenarray
		for(expense_type in data) {

			_dataPoints = [];

			for(_yearmonth in data[expense_type]) {

				for(i=0, j=data[expense_type][_yearmonth].length; i<j; ++i) {

					_dataPoints.push(data[expense_type][_yearmonth][i]);	

					// Berechne Summe
					sum += data[expense_type][_yearmonth][i].y;

					// Setze Maximalwert des Charts
					if(data[expense_type][_yearmonth][i].y > maximum) {
						maximum = data[expense_type][_yearmonth][i].y;
					}

				}
				
			}

			// Erstelle Datensatz
			_temp.push({
				dataPoints: _dataPoints,
				type: 'stackedColumn',
				legendText: expense_type,
				showInLegend: !(type == 'fuel'),
				
				color: (type == 'expenses' && expense_type == 'Kraftstoff') ? '#666' : getExpenseColor(expense_type)
			});

		}

		// Anzahl der anzuzeigenden Monate
		// Hieraus errechnet sich die Anzahl der darzustellenden Monate auf der X-Achse
		// Siehe getChartInstance
		if(lastMonth === -1) {
			numMonths = 12;
		} else {
			numMonths = Math.ceil(lastMonth.diff(firstMonth, 'months', true)) + 1;
		}

		return {
			data: _temp,
			maximum: maximum,
			sum: sum,
			numMonths: numMonths
		};

	};

	/**
	 * Gibt das Label zurück, welches bei Klick auf einen Datensatz in den Charts angezeigt wird.
	 * @param input Object
	 * @param type String fuel|expenses
	 * @returns {string}
	 */
	function getChartLabel(input, type) {

		if(input.expense_type == 'Kraftstoff' && type == 'fuel') {
			return format('#.###,0', parseFloat(input.amount)) + 'l / € '+format('#.###,00', parseFloat(input.price));
		}

		return '<b>'+input.expense_type + ':</b> € '+format('#.###,00', parseFloat(input.price));
	};

	/**
	 * Gibt die Farbe für eine Ausgabenart zurück.
	 * @param type
	 * @returns {string}
	 */
	function getExpenseColor(type) {
		switch(type) {
			case 'Kraftstoff': return '#fdb813';
			case 'Steuer': return '#183065';
			case 'Versicherung': return '#fdb813';
			case 'Pflege': return '#7483a3';
			case 'Werkstatt': return '#fdd471';
			case 'sonstiges': default: return '#7483a3';
                
//            case 'Kraftstoff': return '#fbba00';
//			case 'Steuer': return '#fb0';
//			case 'Versicherung': return '#369';
//			case 'Pflege': return '#F5003D';
//			case 'Werkstatt': return '#00F53D';
//			case 'sonstiges': default: return '#FF3366';
		}
	};

	/**
	 * Gibt ein CanvasJS Chart zurück
	 * @param element_id ID des DOM-Elements
	 * @param chartData
	 * @param options Chartoptions
	 * @returns {defaultOptions.Chart}
	 */
	function getChartInstance(element_id, chartData, options) {

		var defaults = {
		
			culture:'de',
		
			zoomEnabled:false,
		
			backgroundColor:'#f3f3f1',

			axisY: {
				labelFontFamily: '"Helvetica Neue", HelveticaNeue, Helvetica, Arial, sans-serif',
				labelFontSize: 10,
				labelFontColor: "#666",
				gridThickness: 1,
				gridColor:'#ccc',
				tickThickness: 1,
				tickColor: '#ccc',
				lineThickness: 0,
				titleFontSize: 10,
				titleFontFamily: '"Helvetica Neue", HelveticaNeue, Helvetica, Arial, sans-serif'
			},
		
			axisX: {
				labelFontFamily: '"Helvetica Neue", HelveticaNeue, Helvetica, Arial, sans-serif',
				labelFontSize: 10,
				labelFontColor: "#666",
				valueFormatString: "MM/YY",
				intervalType: 'month',
				interval: chartData.numMonths > 24 ? 12 : Math.ceil(chartData.numMonths / 12),
				lineColor:'#ccc',
				lineThickness: 2,
				tickThickness: 2,
				tickColor: '#ccc'
			}

		};


		options = options && $.extend(true, {}, defaults, options) || defaults;
		options.data = chartData.data;
		options.axisX.margin = options.legend ? -10 : -20

		return new CanvasJS.Chart(element_id, options);

	};

	/**
	 * Erstellt das Kraftstoffchart
	 * @param fuel
	 */
	function renderFuelChart(fuel) {

		// Aktualisiere View (Kraftstoffliste)
		config.models['car-fuel']({
			fuel: fuel
		});

		// Erstelle Datensatz
		var dataPoints = prepareChartData(fuel, 'fuel');

		// Erstelle Chart
		// Achtung! Global!
		fuelChart = getChartInstance('fuelchart', dataPoints, {axisY:{title:'Liter'}});
		fuelChart.render();

		//window.fuelchart = fuelChart;

	};


	/**
	 * Erstellt das Ausgabenchart
	 * @param expenses
	 */
	function renderExpenseChart(expenses) {

		// Aktualisiere View (Ausgabenliste)
		config.models['car-expenses']({
			expenses: expenses
		});

		// Erstelle Datensatz
		var dataPoints = prepareChartData(expenses, 'expenses');

		// Erstelle Chart
		// Achtung! Global!
		expenseChart = getChartInstance('expensechart', dataPoints, {
			legend: {
				horizontalAlign: "right", // "center" , "right"
				verticalAlign: "top",  // "top" , "bottom"
				fontSize: 10,
				fontFamily: '"Helvetica Neue", HelveticaNeue, Helvetica, Arial, sans-serif',
				fontColor:'#666'
			},
			axisY: {
				title: 'Euro'
			}
		});
		expenseChart.render();

	};

	/**
	 * Kraftstoffliste auf/zu
	 */
	function toggleFuelList() {
		el.find('#fuellist').toggleClass('is-visible');
	};

	/**
	 * Ausgabenliste auf/zu
	 */
	function toggleExpenseList() {
		el.find('#expenselist').toggleClass('is-visible');
	};


	/**
	 * Zurück zur Garage
	 */
	function hideCarDetail() {
		
		el.removeClass('show-detail');
		UI.handle('hide.back');
		UI.setTitle(config.title);
		setTimeout(function() {
			render();
		}, 300);
	};

	return config;

});

