

JET.controller('BoxenstoppAddExpense', function(el) {

	var isTemplateLoaded = false,
		info_obj,
		$form;

	var config = {
		title: 'Ausgabe',
		template: 'Boxenstopp_editExpense.html',
		templateLoaded: function() {
			isTemplateLoaded = true;

			$form = el.find('#expense-edit');

			// Android hat ein Problem mit der Darstellung von input[type=number].
			// Daher wandeln wir diese Element in input[type=text]
			if(isAndroid()) {
				$form.find('input[type="number"]').attr('type', 'text');
			}

			renderForm();
		},
		viewWillAppear: function(info) {

			// Setze Beschriftung für den Modalbutton rechts
			$('#modal-primary').text('Speichern');

			// Lösche ggf. vorhandene Fehlermeldungen
			el.find('.error').remove();
			el.find('.is-error').removeClass('is-error');

			// Wenn info übergeben wurde, bearbeiten wir einen bestehenden Eintrag
			if(info) {
				info_obj = info;
			}

			if(isTemplateLoaded) {
				renderForm();
			}

		},
		bindEvents: function() {

			// Eventhandler zum Speichern des Formulars
			el.on('submit.boxenstopp_addexpense', '#expense-edit', saveForm);
			JET.on('modal.primary.click.boxenstopp_addexpense', saveForm);

			// Eventhandler zum Schliessen des Modals
			JET.on('modal.secondary.click.boxenstopp_addexpense', function(e) {
				UI.modalClose();
			});

			// Eventhandler zum löschen der Ausgabe
			el.on('click.boxenstopp_addexpense', '.button-delete', function(e) {

				// Confirm
				UI.alert('<b>Bist Du sicher?</b><br>Der Eintrag wird unwiderruflich gelöscht!', function() {

					// Internet?
					if(!isConnected()) {
						return displayNoConnectionError();
					}

					// Zeige Loadingspinner
					UI.toggleLoading();

					// API Call
					$.post(JET.backendURL, {
						action: 'deleteExpense',
						id: el.find('#id').val(),
						token: User.getToken(),
						car_id: info_obj.car.id
					}).done(function(){
						
						// Schliesse Modal und aktualisisere den darunterliegenden View (Boxenstopp)
						UI.modalClose(true);

					}).fail(function(jqXHR) {

						// Internet?
						if(isOfflineError(jqXHR)) {
							return displayNoConnectionError();
						} 

						// Zeige Fehlermeldung
						UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />Die Daten konnten nicht gelöscht werden.');

					}).always(function() {
						// Verberge Loadingspinner
						UI.toggleLoading();
					});
					

				}, function() {
					// Abbrechen
					// Do nothing
				});

			});
		},
		unbindEvents: function() {
			// Entferne Eventhandler
			el.off('.boxenstopp_addexpense');
			JET.off('.boxenstopp_addexpense');
			// Setze Formular zurück
			$form[0].reset();
			car = null;
		}
	};

	/**
	 * Rendert den View
	 */
	function renderForm() {

		var expense;

		if(!info_obj || !info_obj['expense']) {

			// Neuer Eintrag
			// Default:
			expense = {
				expense_date: moment().format('YYYY-MM-DD'),
				mileage: info_obj.car.km,
				id: ''
			}

			// Verberge "Löschen"-Button
			$form.find('.button-delete').hide();
		} else {

			// Da wir alle input[type=number] durch input[type=text] ersetzt haben, müssen die Werte manuell formatiert werden
			var androidFix = {};
			if(isAndroid()) {

				// Preis in deutsches Format wandeln
				androidFix.price = info_obj.expense.price.toString().split('.').join(',');

			}

			// Originalwerte überschreiben
			expense = $.extend({}, info_obj.expense, androidFix);

			// Zeige "Löschen"-Button
			$form.find('.button-delete').show();
		}

		setTimeout(function() {

			if(isAndroid()) {

				// Da Android Probleme mit der Darstellung von input[type=date] hat,
				// wandeln wir diese Felder in input[type=text]
				// Entsprechend müssen die Werte manuell formatiert werden.

				var originalValue = ''+expense.expense_date;

				expense.expense_date = moment(expense.expense_date).format('DD.MM.YYYY');

				// Datumsfelder nur noch readonly
				$form.find('input[type="date"]').attr({
					'type':'text', 
					'readonly':'readonly'
				}).addClass('pseudo-date');

				// Eventhandler zur Datumsauswahl
				el.on('click.boxenstopp_addexpense', '.pseudo-date', function(e) {
					e.preventDefault();

					var element = this,
						presetDate;

					if(originalValue.length > 0) {
						presetDate = new Date(originalValue);
					} else {
						presetDate = new Date();
					}

					// Zeige Datepicker
					datePicker.show({
						date: presetDate,
						mode: 'date',
						allowFutureDates: false
					}, function(date) {

						// Wandele 2015/12/24 in 24.12.2015
						var array = date.split('/');
						if(array[1] < 10) {
							array[1] = '0'+array[1];
						}

						if(array[2] < 10) {
							array[2] = '0'+array[2];
						}

						originalValue = array.join('-');

						array.reverse();

						element.value = array.join('.');
						element.blur();


						
					});
				});
			}

			// Setze Formularwerte
			$form.formParams(expense);

		}, 0);

	};

	/**
	 * Speichert die Formularwerte
	 * @param e
	 * @returns {*}
	 */
	function saveForm(e) {

		// Abschicken verhindern
		e.preventDefault();

		// Hole Werte
		var params = $form.formParams(),
			errors = {};

		// Formularevents löschen
		$form.off('focus.form_helper');

		// Werteumwandlung
		params.price = string2number(params.price);
		params.mileage = string2number(params.mileage, 0);

		// Errorcheck
		if(params.expense_date == '') {
			errors['#expense_date'] = 'Bitte gib das Datum ein!';
		}

		if(params.mileage == '') {
			errors['#mileage'] = 'Bitte gib den Kilometerstand oder die gefahrenen Kilometer ein!';
		}
		if(params.expense_type == '') {
			errors['#expense_type'] = 'Bitte wähle die Kostenart!';
		}
		if(params.price == '') {
			errors['#price'] = 'Bitte gib den Gesamtpreis ein!';
		}

		// Zeige Fehlermeldung
		if(!_.isEmpty(errors)) {
			$form.on('focus.form_helper', '.is-error', function() {
				$(this).removeClass('is-error');
			})
			return showError(errors, $form);
		}

		// API Parameter
		params.action = 'editExpense';
		params.car_id = info_obj.car.id;
		params.token = User.getToken();

		// Internet?
		if(!isConnected()) {
			return displayNoConnectionError();
		}

		// Zeige Loadingspinner
		UI.toggleLoading();

		// API Call
		$.post(JET.backendURL, params, 'json').done(function(data) {

			if(data.response.status == 'success') {
				// Aktualisiere Userdaten
				User.setData(data.user);
				// Schliesse Modal
				UI.modalClose(true);
			} else {
				// Zeige Fehler
				showError([data.response.message], $form);
			}

		}).fail(function(jqXHR, textStatus, errorThrown) {
			
			// Internet?
			if(isOfflineError(jqXHR)) {
				return displayNoConnectionError();
			}

			// Zeige Fehlermeldung
			var response = jqXHR.responseText && JSON.parse(jqXHR.responseText) || {msg: "API Fehler " + errorThrown};
			UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />' + response.msg);

		}).always(function() {
			// Verberge Loadingspinner
			UI.toggleLoading();
		});


	}

	return config;

});