
JET.controller('BoxenstoppAddFuel', function(el) {

	var isTemplateLoaded = false,
		info_obj,
		$form;

	var config = {
		title: 'Tankvorgang',
		template: 'Boxenstopp_editFuel.html',
		templateLoaded: function() {
			isTemplateLoaded = true;

			$form = el.find('#fuel-edit');

			// Android hat ein Problem mit der Darstellung von input[type=number].
			// Daher wandeln wir diese Element in input[type=text]
			if(isAndroid()) {
				$form.find('input[type="number"]').attr('type', 'text');
			}

			renderForm();
		},
		viewWillAppear: function(info) {

			// Text für den Modalbutton rechts
			$('#modal-primary').text('Speichern');

			// Entferne ggf. vorhandene Fehlermeldungen
			el.find('.error').remove();
			el.find('.is-error').removeClass('is-error');

			// Wenn info übergeben wurde, bearbeiten wir einen bestehenden Eintrag
			if(info) {
				info_obj = info;
			}

			if(isTemplateLoaded) {
				renderForm();
			}

		},
		bindEvents: function() {

			// Eventhandler zum Speichern des Formulars
			el.on('submit.boxenstopp_addfuel', '#fuel-edit', saveForm);
			JET.on('modal.primary.click.boxenstopp_addfuel', saveForm);

			// Eventhandler zum löschend es Eintrags
			el.on('click.boxenstopp_addfuel', '.button-delete', function(e) {

				// Confirm
				UI.alert('<b>Bist Du sicher?</b><br>Der Eintrag wird unwiderruflich gelöscht!', function() {

					// Internet?
					if(!isConnected()) {
						return displayNoConnectionError();
					}

					// Zeige Loadingspinner
					UI.toggleLoading();

					// API Call
					$.post(JET.backendURL, {
						action: 'deleteExpense',
						id: el.find('#id').val(),
						token: User.getToken(),
						car_id: info_obj.car.id
					}).done(function(){
						
						// Schliesse Modal und aktualisiere den darunterliegenden View (Boxenstopp)
						UI.modalClose(true);

					}).fail(function(jqXHR) {

						// Internet?
						if(isOfflineError(jqXHR)) {
							return displayNoConnectionError();
						}

						// Zeige Fehlermeldung
						UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />Die Daten konnten nicht gelöscht werden.');

					}).always(function() {
						// Verberge Loadingspinner
						UI.toggleLoading();
					});
					

				}, function() {
					// Abbrechen
					// Do nothing
				});

			});


			// Eventhandler für den "Tankbeleg scannen" Button
			el.on('click.boxenstopp_addfuel', '#scan-receipt', function() {

				// Zeige Hinweis
				UI.alert({
					body: 'Bitte achte darauf, das Foto möglichst formatfüllend, hell, scharf und im Hochformat aufzunehmen. <br />Beispielaufnahme:<br /><img src="img/scan.jpg" style="max-width:50%; margin-top:10px; border:2px solid #000" />',
					okbutton: 'Jetzt Belegscanner starten'
				}, scanReceipt);
			});

		},
		unbindEvents: function() {

			// Entferne Eventhandler
			el.off('.boxenstopp_addfuel');
			JET.off('.boxenstopp_addfuel');

			// Formular leeren
			$form[0].reset();
			car = null;
		}
	};

	/**
	 * Rendert den View
	 */
	function renderForm() {

		var fuel;

		if(!info_obj || !info_obj['fuel']) {

			// Neuer Eintrag
			// Default:
			fuel = {
				expense_date: moment().format('YYYY-MM-DD'),
				fuel_type: info_obj.car.fuel_type,
				mileage: info_obj.car.km,
				id: ''
			}

			// "Löschen"-Button verbergen
			$form.find('.button-delete').hide();

			// Bei neuen Einträgen kann der User auswählen, ob der die gefahrenen Kilometer oder den aktuellen Tachostand eingeben möchte.
			$form.find('label[for="mileage"]').hide();
			$form.find('select[name="mileage-type"]').parent().show();

			// Zeige Tankbelegscanner
			$('#scan-form').show();

		} else {

			// Bestehender Eintrag

			var androidFix = {};
			if(isAndroid()) {
				// Werte für Android formatieren, da die entsprechenden Felder normale input[type=text] sind.
				androidFix.amount = info_obj.fuel.amount.toString().split('.').join(',');
				androidFix.price = info_obj.fuel.price.toString().split('.').join(',');

			}

			fuel = $.extend({}, info_obj.fuel, androidFix);

			// Zeige "Löschen" Button
			$form.find('.button-delete').show();

			// Bei bestehenden Einträgen können die gefahrenen Kilometer nicht mehr sinnvol genutzt werden,
			// deshalb verbergen wir die Selectliste und zeigen statt dessen ein <label /> für das Eingabefeld
			$form.find('label[for="mileage"]').show();
			$form.find('select[name="mileage-type"]').parent().hide();

			// Tankbelegscanner verbergen
			$('#scan-form').hide();
		}

		setTimeout(function() {

			if(isAndroid()) {

				// Da Android Probleme mit der Darstellung von input[type=date] hat,
				// wandeln wir diese Felder in input[type=text]
				// Entsprechend müssen die Werte manuell formatiert werden.

				var originalValue = ''+fuel.expense_date;

				fuel.expense_date = moment(fuel.expense_date).format('DD.MM.YYYY');

				// Datumsfelder nur noch readonly und input[type=text]
				$form.find('input[type="date"]').attr({
					'type':'text', 
					'readonly':'readonly'
				}).addClass('pseudo-date');

				// Eventhandler zur Auswahl eines Datums
				el.on('click.boxenstopp_addfuel', '.pseudo-date', function(e) {
					e.preventDefault();

					var element = this,
						presetDate;

					if(originalValue.length > 0) {
						presetDate = new Date(originalValue);
					} else {
						presetDate = new Date();
					}

					// Zeige Datepicker
					datePicker.show({
						date: presetDate,
						mode: 'date',
						allowFutureDates: false
					}, function(date) {

						// Wandele 2015/12/24 in 24.12.2015
						var array = date.split('/');
						if(array[1] < 10) {
							array[1] = '0'+array[1];
						}

						if(array[2] < 10) {
							array[2] = '0'+array[2];
						}

						originalValue = array.join('-');

						// Setze Formularwert
						element.value = array.reverse().join('.');

						element.blur();
						
					});
				});
			}

			// Setze Formularwerte
			$form.formParams(fuel);

		}, 0);

	};

	/**
	 * Speichert das Formular
	 * @param e
	 * @returns {*}
	 */
	function saveForm(e) {

		// Formsubmit unterbinden
		e.preventDefault();

		var params = $form.formParams(),
			errors = {};

		// Formevents löschen
		$form.off('focus.form_helper');

		// Wandele Formularwerte in float
		params.amount = string2number(params.amount);
		params.price = string2number(params.price);
		params.mileage = string2number(params.mileage, 0);

		// Errorcheck
		if(params.expense_date == '') {
			errors['#expense_date'] = 'Bitte gib das Datum ein!';
		}
		if(params.mileage == '') {
			errors['#mileage'] = 'Bitte gib den Kilometerstand oder die gefahrenen Kilometer ein!';
		}
		if(params.amount == '') {
			errors['#amount'] = 'Bitte gib die getankte Menge ein!';
		}
		if(params.price == '') {
			errors['#price'] = 'Bitte gib den Gesamtpreis ein!';
		}

		// Zeige Fehlermeldung
		if(!_.isEmpty(errors)) {
			$form.on('focus.form_helper', '.is-error', function() {
				$(this).removeClass('is-error');
			})
			return showError(errors, $form);
		}

		// API Parameter
		params.action = 'editFuel';
		params.car_id = info_obj.car.id;
		params.token = User.getToken();

		// Verberge Tastatur
		$form.find('input').blur();

		// Internet?
		if(!isConnected()) {
			return displayNoConnectionError();
		}

		// Zeige Loadingspinner
		UI.toggleLoading();

		// API Call
		$.post(JET.backendURL, params, 'json').done(function(data) {

			if(data.response.status == 'success') {
				// Aktualisiere User
				User.setData(data.user);

				// Schliesse Modal und aktualisiere den darunterliegenden View (Boxenstopp)
				UI.modalClose(true);
			} else {

				// Zeige Fehlermeldung
				showError([data.response.message], $form);
			}


		}).fail(function(jqXHR, textStatus, errorThrown) {

			// Internet?
			if(isOfflineError(jqXHR)) {
				return displayNoConnectionError();
			}

			// Zeige Fehlermeldung
			var response = jqXHR.responseText && JSON.parse(jqXHR.responseText) || {msg: "API Fehler " + errorThrown};
			UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />' + response.msg);

		}).always(function() {
			// Verberge Loadingspinner
			UI.toggleLoading();
		});


	}

	/**
	 * Nimmt ein Foto auf und sendet es an den Server zur OCR-Erkennung.
	 * Aktualisiert das Formular im Erfolgsfall oder zeigt eine Fehlermeldung
	 *
	 * @returns {*}
	 */
	function scanReceipt() {

		// Internet?
		if(!isConnected()) {
			return displayNoConnectionError();
		}

		// Camera-Plugin
		var Camera = navigator.camera;

		// https://github.com/apache/cordova-plugin-camera#module_camera
		// Da die Qualität der Android Kamera nicht so gut ist, wie die iOS Kamera,
		// werden für Android etwas höhere Werte für die Qualität und die Auflösung gesetzt.
		Camera.getPicture(scanSuccess, scanFail, {
			quality:            isAndroid() ? 80 : 40,
			destinationType:    Camera.DestinationType.FILE_URI,
			sourceType:         Camera.PictureSourceType.CAMERA,
			encodingType:       Camera.EncodingType.JPEG,
			targetWidth:        isAndroid() ? 600 : 480,
			targetHeight:       isAndroid() ? 800 : 640,
			correctOrientation: true,
			saveToPhotoAlbum:   false
		});

	};

	/**
	 * Erfolgscallbackfunktion für Camera.getPicture
	 * @param imageURI
	 * @returns {*}
	 */
	function scanSuccess(imageURI) {

		// Kein Internet?
		if(!isConnected()) {
			return displayNoConnectionError();
		}

		// Zeige Loadingspinner
		UI.toggleLoading();

		// API Call
		// https://github.com/apache/cordova-plugin-file-transfer
		var ft, options;

		options = new FileUploadOptions();
		options.fileKey = 'scan';
		options.mimeType = 'image/jpeg';

		ft = new FileTransfer();
		ft.upload(imageURI, JET.backendURL+'ocr/scan.php', function(result) {

			// Upload Success!

			// Verberge Loadingspinner
			UI.toggleLoading();

			var data = JSON.parse(result.response),
				alert_msg = [];

			if(data.status == 'success') {

				// Aktualisiere Formular
				$form.find('#amount').val(data.l);
				$form.find('#fuel_type').val(data.type);
				$form.find('#price').val(data.total);

				// Das Datum ist nicht notwendig, um als Success zu gelten
				if(data.date != '') {
					$form.find('#expense_date').val(data.date);
				}

				// Da die Texterkennung nicht 100% zuverlässig ist, informieren wir den User, das er das Ergebnis bitte manuell prüfen soll
				alert_msg.push('Bitte überprüfe das Ergebnis:<br />');
				alert_msg.push('<b>'+format('#.##0,00', data.l)+' Liter '+JET.pricecode(data.type)+' für €'+format('#.##0,00', data.total)+'.</b><br />');
				if(data.date != '') {
					alert_msg.push('Datum: '+data.date.split('-').reverse().join('.')+'<br />');
					alert_msg.push('Vergiss bitte nicht, den Kilometerstand einzutragen.');
				} else {
					alert_msg.push('Vergiss bitte nicht, den Kilometerstand und das Datum einzutragen.');
				}

				// Zeige Hinweis
				UI.alert(alert_msg.join(''));

				// Tracke Scanergebnis in Google Analytics
				ga.trackEvent('Tankbelegscanner', 'success');

			} else {
				// Scanfehler

				// Zeige Message
				UI.alert(data.msg);

				// Tracke Scanergebnis in Google Analytics
				ga.trackEvent('Tankbelegscanner', 'failure', data.msg);

				
			}


			if(!isAndroid()) {
				// iOS muss das temporäre Foto aufräumen
				var noop = function() {};
				navigator.camera.cleanup(noop, noop);

			}


		}, function(error) {

			// Upload Fehler
			console.log(error);

			// Verberge Loadinspinner
			UI.toggleLoading();

			// Internet?
			if(!isConnected()) {
				return displayNoConnectionError();
			}

			// Zeige Fehlermeldung
			UI.alert('<b>Ein Fehler ist aufgetreten</b><br />Bitte versuche es erneut.');

			// Tracke Fehler in Google Analytics
			ga.trackEvent('Tankbelegscanner', 'failure', error.code);

		}, options);


	};

	/**
	 * Fehlercallbackfunktion für Camera.getPicture
	 * @param message
	 */
	function scanFail(message) {
		// Zeige Fehlermeldung
		UI.alert('<b>Ein Fehler ist aufgetreten</b><br />'+message);
	}

	return config;

});