
JET.controller('BoxenstoppEditCar', function(el) {

	var isTemplateLoaded = false,
		car;

	var config = {
		title: 'Auto bearbeiten',
		template: 'Boxenstopp_editCar.html',
		templateLoaded: function() {

			isTemplateLoaded = true;

			// Android hat ein Problem mit input[type=number], daher wandeln wir diese Elemente in input[type=text]
			if(isAndroid()) {
				$('#car-edit').find('input[type="number"]').attr('type', 'text');
			}

			renderForm();
		},
		viewWillAppear: function(car_id) {

			// Setze Buttontext für den Modalbutton rechts
			$('#modal-primary').text('Speichern');

			// Entferne Fehlermeldungen, die ggf. von einem vorhergehenden Aufruf übrig geblieben sind.
			el.find('.error').remove();
			el.find('.is-error').removeClass('is-error');

			if(car_id) {

				// Auto bearbeiten
				var cars = User.getData('cars');
				for(var i=0, j=cars.length; i<j; ++i) {

					if(cars[i].id == car_id) {
						car = cars[i];
					}

				}

			} else {
				car = null;
			}

			if(isTemplateLoaded) {
				renderForm();
			}

		},
		bindEvents: function() {


			// Eventhandler für den Modalbutton rechts (Speichern)
			el.on('submit.boxenstopp_edit', '#car-edit', saveForm);
			JET.on('modal.primary.click.boxenstopp_edit', saveForm);
			
			// Eventhandler zum löschen eines Autos
			el.on('click.boxenstopp_edit', '.button-delete', function(e) {

				UI.alert('<b>Bist Du sicher?</b><br>Dein Auto wird unwiderruflich gelöscht!', function() {

					// Internet?
					if(!isConnected()) {
						return displayNoConnectionError();
					}

					// API Call
					$.post(JET.backendURL, {
						action: 'deleteCar',
						car_id: $('#car-edit').find('input[name="id"]').val(),
						token: User.getToken()
					}).done(function(data) {

						// Aktualisiere User
						User.setData(data.user);

						// Schliesse Modal und aktualisiere den darunterliegenden View (Boxenstopp)
						UI.modalClose(true);
					}).fail(function(jqXHR) {

						// Internet?
						if(isOfflineError(jqXHR)) {
							return displayNoConnectionError();
						}

						// Zeige Fehlermeldung
						UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />Die Daten konnten nicht gelöscht werden.');

					});

				}, true);

			});
			
            
			// Eventhandler zur Auswahl des KFZ-Fotos
			el.on('click.boxenstopp_edit', 'label[for="picture"]', function(e) {

				// Wir zeigen einen Alert, in dem der User auswählen kann, ob er ein neues Foto machen,
				// oder ein bestehendes Bild aus der Library auswählen möchte.
				var alertBody = [
					'<b>Autobild auswählen</b><br /><br />',
					/*'<button type="button" class="picture-choose-button choose-album"><i class="fa fa-picture-o"></i> Foto auswählen</button>',
                    '<button type="button" class="picture-choose-button choose-camera"><i class="fa fa-camera"></i> Foto aufnehmen</button>'*/
                    '<button type="button" class="button-primary choose-album button-spacer"><i class="fa fa-picture-o"></i> Foto auswählen</button>',
                    '<button type="button" class="button-secondary choose-camera"><i class="fa fa-camera"></i> Foto aufnehmen</button>'
					
				];

				// Eventhandler für die Auswahl der Bildquelle
				$('#alert-body').one('click.boxenstopp_edit', 'button', function(e) {

					var $btn = $(this), type;
					if($btn.hasClass('choose-camera')) {
						type = 'CAMERA';
					} else if($btn.hasClass('choose-album')) {
						type = 'PHOTOLIBRARY';
					}

					// Wähle Foto
					getPhoto(type).done(function(imageData) {

						// Zeige Bildvorschau
						$('#user-car-picture').attr('src', 'data:image/jpeg;base64,'+imageData);

						// Daten wurden geändert
						isDirty = true;

					}).fail(function() {

						// Zeige Fehlermeldung
						UI.alert({
							body: '<b>Das hat leider nicht funktioniert!</b>',
							okbutton:'Erneut versuchen',
							cancelbutton: 'Abbrechen'
						}, function() {

							$('label[for="picture"]', el).click();

						});

					});

					// Schliesse Alert
					$('#alert-ok').trigger('click');

				});

				// Zeige Alert
				// Der "OK"-Button hat hierbei die Funktion "Abbrechen"
				UI.alert({
					body: alertBody.join(''),
					okbutton: '<div class="photo-chooser-cancel">Abbrechen</div>'
				}, function() {
					// Lösche Eventhandler
					$('#alert-body').off('click.boxenstopp_edit');
				});



			});

		},
		unbindEvents: function() {

			// Lösche Eventhandler
			el.off('.boxenstopp_edit')
			JET.off('.boxenstopp_edit');

			// Resette Formular
			var $form = $('#car-edit');

			$form[0].reset();

			$form.find('.success').remove();
			$form.find('.error').remove();
			$form.find('.info').remove();

			// Resette Variable
			car = null;
		}
	};

	/**
	 * Rendert den View
	 */
	function renderForm() {

		if(car) {
			$('#car-edit').addClass('has-car');
		} else {
			$('#car-edit').removeClass('has-car');
		}

		setTimeout(function() {

			// Formularparameter
			var params = $.extend({id:''}, car);
			delete params.picture;

			if(params.construction_year == '0000') {
				params.construction_year = '';
			}
			if(params.power == '0') {
				params.power = '';
				params.power_type = 'ps';
			} else {
				params.power_type = params.power_type || 'kw';
			}

			// Termine aus der Vergangenheit werden nicht erlaubt.
			var today = moment();
			$('#car-edit').find('input[type="date"]').on('change blur', function() {
				var that = this;
				if(this.value != '' && today.isAfter(moment(this.value), 'day')) {
					UI.alert('Das Datum liegt in der Vergangenheit!', function() {
						that.focus();
					});
				}
			});

			if(isAndroid()) {

				// Da Android Fehler in der Darstellung von input[type=number] oder input[type="date"] hat
				// müssen wir die Werte hier entsprechend aufbereiten

				var originalValues = {
					hu: params.hu,
					oilcheck: params.oilcheck,
					insurance: params.insurance,
					tax: params.tax
				};

				// Datumsangaben formatieren
				if(params.hu && params.hu.length > 0) {
					params.hu = moment(params.hu).format('DD.MM.YYYY');
				}
				if(params.oilcheck && params.oilcheck.length > 0) {
					params.oilcheck = moment(params.oilcheck).format('DD.MM.YYYY');
				}
				if(params.insurance && params.insurance.length > 0) {
					params.insurance = moment(params.insurance).format('DD.MM.YYYY');
				}
				if(params.tax && params.tax.length > 0) {
					params.tax = moment(params.tax).format('DD.MM.YYYY');
				}

				// Wandele input[type=date] in input[type=text]
				// readonly
				$('#car-edit').find('input[type="date"]').attr({
					'type':'text', 
					'readonly':'readonly'
				}).addClass('pseudo-date');

				// Enventhandler zur Auswahl eines Datums
				el.on('click.boxenstopp_edit', '.pseudo-date', function(e) {
					e.preventDefault();

					var element = this,
						element_id = this.id,
						presetDate,
						currentValue = originalValues[element_id];

					if(currentValue && currentValue.length > 0) {
						presetDate = new Date(currentValue);
					} else {
						presetDate = new Date();
					}

					datePicker.show({
						date: presetDate,
						mode: 'date',
						allowOldDates: false
					}, function(date) {

						// Formatiere das Datum für die Darstellung (2015/12/24 => 24.12.2015)
						var array = date.split('/');
						if(array[1] < 10) {
							array[1] = '0'+array[1];
						}

						if(array[2] < 10) {
							array[2] = '0'+array[2];
						}

						// Termine aus der Vergangenheit werden nicht erlaubt.
						if(today.isAfter(moment(array.join('-')), 'day')) {
							UI.alert('Das Datum liegt in der Vergangenheit!', function() {
								$(element).click();
							});
						} else {
							originalValues[element_id] = array.join('-');

							element.value = array.reverse().join('.');

						}
						element.blur();

					});
				});

			}

			var androidFix = {};
			if(isAndroid()) {

				// Wandele Number in deutsches Format
				androidFix.pressure_front = params.pressure_front.toString().split('.').join(',');
				androidFix.pressure_back = params.pressure_front.toString().split('.').join(',');

			}

			// Setze Formularparameter
			$('#car-edit').formParams(params);

			// Setze Autobild
			if(car && car['picture']) {
				el.find('label[for="picture"] img').attr('src', JET.baseURL + 'img/290x290/'+car.picture);
			} else {
				el.find('label[for="picture"] img').attr('src', JET.defaultCarPicture);

			}

		}, 0);

	};

	/**
	 * Speichert die Formulardaten
	 * @param e
	 * @returns {*}
	 */
	function saveForm(e) {

		e.preventDefault();

		var $form = $('#car-edit'), 
			params = $form.formParams(),
			errors = {},
			data;

		// Eventhandler löschen
		$form.off('focus.boxenstopp_helper');

		// Wandele Daten
		params.power = string2number(params.power, 2);
		params.construction_year = string2number(params.construction_year, 0);
		params.km = string2number(params.km, 1);
		params.pressure_front = string2number(params.pressure_front, 2);
		params.pressure_back = string2number(params.pressure_back, 2);

		// Errorcheck
		if(params.brand_name == '') {
			errors['#brand_name'] = 'Bitte gib den Hersteller ein!';
		}
		if(params.model_name == '') {
			errors['#model_name'] = 'Bitte gib das Modell ein!';
		}

		// Zeige Fehlermeldung
		if(!_.isEmpty(errors)) {
			$form.on('focus.boxenstopp_helper', '.is-error', function() {
				$(this).removeClass('is-error');
			})
			return showError(errors, $form);
		}

		// API Daten
		data = new FormData();

		data.append('action', 'editCar');
		data.append('token', User.getToken());

		for(var key in params) {
			data.append(key, params[key]);
		}

		// Bild nur übertragen, wenn es im data:image Format vorliegt (also neu ausgewählt wurde)
		var img_src = $('#user-car-picture').attr('src');
		if(img_src.indexOf('data') === 0) {
			data.append('picture', img_src);
		}

		// Internet?
		if(!isConnected()) {
			return displayNoConnectionError();
		}

		// Zeige Loadingspinner
		UI.toggleLoading();

		// API Call
		$.ajax({
			url: JET.backendURL,
			data: data,
			type: 'POST',
			processData: false,
			contentType: false,
			cache: false
		}).done(function(data) {
			
			if(data.status == 'success') {

				// Aktualisiere User
				User.setData(data.user);

				showSuccess('Die Änderungen wurden gespeichert.', $form);
				if(data['msg']) {
					showMessage(data.msg, $form);
				}

				$form[0].scrollTop = 0;

				// Modal schliessen und darunterliegenden View aktualisiseren (Boxenstopp)
				UI.modalClose(true);

			} else {

				// Zeige Fehlermeldung
				showError(data.msg, $form);
			}


		}).fail(function(jqXHR, textStatus, errorThrown) {

			// Internet?
			if(isOfflineError(jqXHR)) {
				return displayNoConnectionError();
			}

			// Zeige Fehlermeldung
			UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />Die Daten konnten nicht gespeichert werden.');

		}).always(function() {

			// Verberge Loadingspinner
			UI.toggleLoading();
		});


	}

	return config;

});
