JET.controller('KlickeNoLogin', function(el) {

	var config = {
		title: 'Deine KLICKe',
		template: 'Klicke_nologin.html'
	};

	return config;

});

JET.controller('Klicke', function(el) {

	var isTemplateLoaded = false,
		carousel,
		messages = [];

	var config = {
		title: 'Deine KLICKe',
		template: 'Klicke.html',
		templateLoaded: function() {

			isTemplateLoaded = true;
			render();

		},
		bindEvents: function() {


			if(isTemplateLoaded) {
				render();
			}

			// Eventhandler um ein Userprofil anzuzeigen
			el.on('click.klicke', '.ts-detail-klicke-item', function(e) {

				UI.modal('KlickeUserDetail', {
					user_id: this.getAttribute('data-userid'),
					is_friend: 1
				})

			});


			// Eventhandler um eine Nachricht zu beantworten oder zu löschen
			el.on('click.klicke', '.messagelist button', function(e) {

				// Verhindere, das dieses Event weiter hoch bubblet
				e.preventDefault();
				e.stopPropagation();

				var $btn = $(this),
					$li = $btn.parents('li');

				// Nachricht löschen
				if($btn.hasClass('message-trash')) {

					// Confirm
					UI.alert('<b>Bist Du sicher?</b><br />Die Nachricht wird endgültig gelöscht.', function() {
						
						var msgid = $li.data('messageid');

						// API Call
						$.post(JET.backendURL, {
							action:'deleteMessage',
							msgid: msgid,
							token: User.getToken()
						}).done(function() {

							// Zeile löschen
							$li.slideUp(function() {
								$li.remove();
							});

							// Gelöschte Nachricht ausfiltern
							messages = _.filter(messages, function(message) {
								return message.id != msgid;
							});

							// Aktualisiere UI
							updateMessageCount();

						}).fail(function(jqXHR) {

							// Internet?
							if(isOfflineError(jqXHR)) {
								return displayNoConnectionError();
							}

							// Zeige Fehlermeldung
							UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />Die Nachricht konnte nicht gelöscht werden.');
						});
						

					}, true);

				}else{

					// Nachricht beantworten
					var msgid = $li.data('messageid'),

						// Suche Nachricht
						originalMessage =  _.find(messages, function(message) {
							return message.id == msgid;
						});

					// Zeige Editformular
					UI.modal('KlickeSendMessage', {
						to_user_id: $li.data('fromuser'),
						username: originalMessage.username,
						picture: originalMessage.picture,
						message: originalMessage.message,
						create_date: originalMessage.create_date
					});

				}

			});

			// Freundschaftsanfragen werden auch als KLICKe-Nachrichten verschickt.
			// Diese (und nur diese) Nachrichten sind HTML-formatiert und beinhalten Links zum Annehmen der Anfrage und zum Aufruf des Profils
			// Eventhandler um die Klicks abzufangen
			el.on('click.klicke', '.message a', function(e) {

				// Event nicht wieter hoch bubblen
				e.preventDefault();
				e.stopPropagation();

				// GET-Parameter
				var get = this.href.slice(this.href.indexOf('?')+1).split('&');
				var params = {};

				for(var i=0, j=get.length; i<j; ++i) {
					get[i] = get[i].split('=');
					params[get[i][0]] = get[i][1];
				}
				get = null;
				
				if(params.action == 'confirm_friend') {

					// Freundschaft bestätigen

					// API Call
					$.post(JET.backendURL, {
						
						action: 'confirm_friend',
						friend_id: params.id,
						token:User.getToken()

					}).done(function(data) {

						// Aktualisiere User
						User.setData(data.user);

						// Aktualisiere View
						config.refresh();

					}).fail(function(jqXHR, textStatus, errorThrown) {

						// Internet?
						if(isOfflineError(jqXHR)) {
							return displayNoConnectionError();
						}

						// Zeige Fehlermeldung
						var api_message = jqXHR.responseJSON || { msg:'API Fehler' };
						UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />' + api_message.msg);
					});

				} else {

					// Profil anzeigen
					UI.modal('KlickeUserDetail', {
						user_id: params.user_id,
						confirm: 1
					});

				}


			})

			// Eventhandler um eine Nachricht anzuzeigen und als gelesen zu markieren
			el.on('click.klicke', '.messagelist li', function(e) {
			
				var $li = $(this);

				// Zeige Nachrichtentext
				$li.toggleClass('is-active');

				if($li.hasClass('is-unread')) {

					// Markiere als gelesen
					$li.removeClass('is-unread');

					// ID der Nachricht
					var msgid = this.getAttribute('data-messageid'); 

					// API Call
					$.post(JET.backendURL, {
						action:'markAsRead',
						msgid: msgid
					});

					// Suche Nachricht
					var message = _.find(messages, function(item) {
						return item.id == msgid;
					});
					message.flag_read = 1;
					/*
					_.each(messages, function(item) {
						if(item.id == msgid) {
							item.flag_read = 1;
						}
					});
					*/

					// Aktualisiere UI
					updateMessageCount();
				}

			});


		},
		unbindEvents: function() {

			// Beende Carousel
			carousel && carousel.destroy();

			// Lösche Eventhandler
			el.off('.klicke');
			JET.off('.klicke');
			$(document).off('.klicke');
		},
		refresh: function() {

			// Beende Carousel
			carousel && carousel.destroy();

			render();

		}
	};

	/**
	 * Rendert den View
	 */
	function render() {

		// Zeige Loadingmessage
		el.find('[data-template="messages"]').html('<p class="padded">Die Nachrichten werden geladen&hellip;</p>');

		// Hole Freundeliste
		var friends = User.getData('friends');

		// Aktualisiere UI
		config.models.friends({friends:friends});

		setTimeout(function() {

			// Erstelle Carousel
			carousel = el.find('.owl-carousel').owlCarousel({
				items:4,
				itemsDesktop: [1000, 4],
				itemsDesktopSmall: [900, 4],
				itemsTablet: [600, 4],
				itemsMobile: [400, 4],
				scrollPerPage: true,
				pagination: true
			}).data('owlCarousel');

		}, 0);

		// Lade die Nachrichten
		setTimeout(function() {
			loadMessages();
		}, 100);

	};

	/**
	 * Lädt die Nachrichten
	 */
	function loadMessages() {

		// API Call
		$.post(JET.backendURL, {
			action: 'getMessages',
			token: User.getToken()
		}).done(function(data) {

			// Lösche Nachrichtenliste
			el.find('[data-template="messages"]').empty();

			// Lokaler Cache
			messages = data.messages;

			// Aktualisiere UI
			updateMessageCount();

			setTimeout(function() {
				// Zeige Nachrichtenliste
				config.models.messages(data);
			}, 0);

		}).fail(function(jqXHR) {

			// Lösche Nachrichtenliste
			el.find('[data-template="messages"]').empty();

			// Internet?
			if(isOfflineError(jqXHR)) {

				// Zeige Fehlermeldung
				UI.alert('<b>Derzeit keine Internetverbindung!</b><br />Deine Nachrichten können nicht geladen werden.');

				// Lade Nachrichten erneut, wenn Online
				$(document).one('online.klicke', loadMessages);
			} else {

				// Zeige Fehlermeldung
				UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />Deine Nachrichten können nicht geladen werden.');
			}
			
		});

	};

	/**
	 * Aktualisiert die Anzeige der Nachrichten insgesamt und der ungelesenen Nachrichten
	 */
	function updateMessageCount() {

		// Zähle die ungelesenen Nachrichten
		var unread_count = 0;
		_.each(messages, function(item) {
			if(item.flag_read == '0') unread_count++;
		});

		// Aktualisiere UI
		el.find('.messagecount').html(
			messages.length + ' Nachricht'+(messages.length == 1 ? '' : 'en')+' '+
			'<small>('+unread_count+' ungelesen)</small>'
		);
	};

	return config;

});