
JET.controller('KlickeSendMessage', function(el) {

	var isTemplateLoaded = false,
		info_obj,
		$form;

	var config = {
		title: 'Nachricht',
		template: 'Klicke_sendMessage.html',
		templateLoaded: function() {

			isTemplateLoaded = true;
			$form = el.find('#message-edit');
			renderForm();

		},
		viewWillAppear: function(info) {

			// Modalbuttontext
			$('#modal-primary').text('Senden');

			// Fehlermeldungen entfernen
			el.find('.error').remove();
			el.find('.is-error').removeClass('is-error');

			// Parameter
			// {to_user_id, username, picture [, message]}
			if(info) {
				info_obj = info;
			}

			if(isTemplateLoaded) {
				renderForm();
			}

		},
		bindEvents: function() {

			// Eventhandler um die Daten zu speichern
			el.on('submit.klicke_sendmessage', '#message-edit', saveForm);
			JET.on('modal.primary.click.klicke_sendmessage', saveForm);

		},
		unbindEvents: function() {

			// Enventhandler entfernen
			el.off('.klicke_sendmessage');
			JET.off('.klicke_sendmessage');

			// Resette Form
			$form[0].reset();

		}
	};

	function renderForm() {

		// Formularwerte
		var formParams = {
				to_user_id: info_obj.to_user_id,
				message: ''
			}, 
			plainText = '';

		// Nachrichtentext aufbereiten
		if(info_obj['message']) {

			// Wir nehmen den Text, fügen eine Statuszeile hinzu und wordwrappen den Text nach 60 Zeichen mit einem Einrückungszeichen

			// Statuszeile
			formParams.message = ''+"\n\n\n";
			formParams.message += ''+info_obj.username + ' schrieb am '+moment(info_obj.create_date).format('DD.MM.YYYY [um] HH:mm [Uhr]') + ':';
			formParams.message += "\n> ";

			// Strip HTML
			plainText = $('<div />').html(info_obj.message).text();

			// Füge Einrückung hinzu
			plainText = wordwrap(plainText, 60, '\n> ');

			// Doppelte Zeilenumbrüche ersetzen
			formParams.message += plainText.split("\n\n").join('\n');

		}

		// Username und Bild (Empfänger) setzen
		$form.find('.username').text(info_obj.username);
		if(info_obj.picture) {
			$form.find('.userimage')[0].src = JET.baseURL+'img/100x100/'+info_obj.picture;
		} else {
			$form.find('.userimage')[0].src = 'img/klicke-profile-picture.png';
		}

		setTimeout(function() {
			// Formularwerte setzen
			$form.formParams(formParams);
		}, 0);

	};

	/**
	 * Speichert die Daten
	 * @param e
	 * @returns {*}
	 */
	function saveForm(e) {

		// Formsubmit unterbinden
		e.preventDefault();

		// Hole Formularwerte
		var params = $form.formParams(),
			errors = {};

		// Formevent beenden
		$form.off('focus.form_helper');

		// Errorcheck
		if(params.message == '') {
			errors['#message'] = 'Bitte gib Deine Nachricht ein!';
		}

		// Zeige Fehlermeldung
		if(!_.isEmpty(errors)) {
			$form.on('focus.form_helper', '.is-error', function() {
				$(this).removeClass('is-error');
			});
			return showError(errors, $form);
		}

		// API Parameter
		params.action = 'sendMessage';
		params.token = User.getToken();

		// Internet?
		if(!isConnected()) {
			return displayNoConnectionError();
		}

		// Zeige Loading
		UI.toggleLoading();

		// API Call
		$.post(JET.backendURL, params, 'json').done(function(data) {

			// Aktualisiere User
			User.setData(data.user);

			// Schliesse Modal und aktualisiere darunterliegenden View
			UI.modalClose(true);

		}).fail(function(jqXHR, textStatus, errorThrown) {

			// Internet?
			if(isOfflineError(jqXHR)) {
				return displayNoConnectionError();
			}

			// Zeige Fehlermeldung
			var response = jqXHR.responseJSON || jqXHR.responseText && JSON.parse(jqXHR.responseText) || {msg: "API Fehler " + errorThrown};
			UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />' + response.msg);

		}).always(function() {

			// Verberge Loading
			UI.toggleLoading();
		});


	}

	return config;

});