
JET.controller('KlickeUserDetail', function(el) {

	var isTemplateLoaded = false,
		info_obj,
		user = {};

	var config = {
		title: 'KLICKe',
		template: 'Klicke_user.html',
		templateLoaded: function() {
			isTemplateLoaded = true;
			render();
		},
		viewWillAppear: function(info) {

			// Setze Text für den Modalbutton rechts
			$('#modal-primary').text('Schließen');

			// Den Modalbutton links brauchen wir nicht
			$('#modal-secondary').hide();

			// Objekt mit den Keys: user_id, is_friend oder confirm
			if(info) {
				info_obj = info;
				user = {};
			}

			if(isTemplateLoaded) {
				render();
			}

		},
		bindEvents: function() {
		
            
			// Eventhandler um das Modal zu schliessen
			JET.on('modal.primary.click.klicke_userdetail', function(e) {
				UI.modalClose();
			});

			// Eventhandler um den User zu melden
			el.on('click.klicke_userdetail', '.report-profile', function(e) {
				UI.modal('KlickeReportUser', this.getAttribute('data-userid'));
			});

			// Eventhandler um die Freundschaft zu beenden
			el.on('click.klicke_userdetail', '.end-friendship', function() {

				// Confirm
				UI.alert({
					body: '<b>Bist Du sicher?</b><br />Diese Aktion kann nicht rückgängig gemacht werden.',
					okbutton: 'Freundschaft beenden',
					cancelbutton: 'Abbrechen'
				}, function() {

					// Internet?
					if(!isConnected()) {
						return displayNoConnectionError();
					}

					// Zeige Loadingspinner
					UI.toggleLoading();

					// API Call
					$.post(JET.backendURL, {
						action: 'endFriendship',
						friend_id: info_obj.user_id,
						token: User.getToken()
					}).done(function(data) {

						// Aktualisiere User
						User.setData(data.user);

						// Schliesse Modal und aktualisiere den darunterliegenden View
						UI.modalClose(true);

					}).fail(apiFail).always(apiEnd);

				});

				

			});

			// Eventhandler um die Freundschaft zu bestätigen
			el.on('click.klicke_userdetail', '.confirm-friendship', function() {

				// Internet?
				if(!isConnected()) {
					return displayNoConnectionError();
				}

				// Zeige Loading
				UI.toggleLoading();

				// API Call
				$.post(JET.backendURL, {
					action: 'confirm_friend',
					friend_id: info_obj.user_id,
					token: User.getToken()
				}).done(function(data) {

					// Aktualisiere User
					User.setData(data.user);

					// Schliesse Modal und aktualisiere den darunterliegenden View
					UI.modalClose(true);

				}).fail(apiFail).always(apiEnd);
			});

			// Eventhandler um einen Freundschaftsanfrage zu stellen
			el.on('click.klicke_userdetail', '.request-friendship', function() {

				// Internet?
				if(!isConnected()) {
					return displayNoConnectionError();
				}

				// Zeige Loading
				UI.toggleLoading();

				// API Call
				$.post(JET.backendURL, {
					action: 'requestFriendship',
					friend_id: info_obj.user_id,
					token: User.getToken()
				}).done(function(data) {

					if(data.status == 'success') {

						// Aktualisiere User
						User.setData(data.user);

						// Zeige Meldung
						UI.alert(data.msg, function() {

							// Schliesse Modal und aktualisiere den darunterliegenden View
							UI.modalClose(true);

						});
					}

					

				}).fail(apiFail).always(apiEnd);
			});

			// Eventhandler um eine Nachricht zu senden
			el.on('click.klicke_userdetail', '.button-send-message', function(e) {

				e.preventDefault();

				// Controller Parameter
				var info = $.extend({
					to_user_id: this.getAttribute('data-userid')
				}, user);

				// Zeige Controller
				UI.modal('KlickeSendMessage', info);

			});

		},
		unbindEvents: function() {

			// Eventlistener entfernen
			el.off('.klicke_userdetail');
			JET.off('.klicke_userdetail');
		
		}
	};

	/**
	 * jQuery.ajax always Callback
	 */
	function apiEnd() {
		// Loading verbergen
		UI.toggleLoading();
	}

	/**
	 * jQuery.ajax fail Callback
	 */
	function apiFail(jqXHR) {

		// Internet?
		if(isOfflineError(jqXHR)) {
			return displayNoConnectionError();
		}

		// Zeige Fehlermeldung
		var msg = jqXHR.responseJSON && jqXHR.responseJSON.msg || 'API Fehler';
		UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />'+msg);
	}

	/**
	 * Rendert den View
	 */
	function render() {

		// Die Daten werden nicht schnell genug geladen, so dass hier ggf. noch alte Daten eines vorhergehenden Aufrufs
		// angezeigt werden.
		// Also blenden wir das ganze Div erstmal aus
		el.find('[data-template="klickeUser"]').hide();

		// Kein Internet?
		if(!isConnected()) {
			return displayNoConnectionError(true);
		}

		// ProfilID
		var uid = info_obj.user_id;

		// API Call
		$.get(JET.backendURL, {
			action: 'getUserDetails',
			uid: uid,
			token: User.getToken()
		}).done(function(data) {

			// Aktualisiere View
			config.models.klickeUser($.extend(user, info_obj, data.user, data));

			// Zeige das Div wieder an
			el.find('[data-template="klickeUser"]').show();

		}).fail(function(jqXHR) {

			// Internet?
			if(isOfflineError(jqXHR)) {
				return displayNoConnectionError(true);
			}

			// Zeige Fehlermeldung
			UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />Die Daten konnten nicht geladen werden.');

		});

	}

	return config;

});