JET.controller('Kontakt', function(el) {

	var config = {
		title: 'Kontakt',
		template: 'Kontakt.html',
		viewWillAppear: function() {

			// Text des rechten Modalbuttons
			$('#modal-primary').text('Abschicken');

		},
		bindEvents: function() {

			// Eventhandler zum Abschicken des Formulars
			JET.on('modal.primary.click.kontakt', function(e) {
				
				var $form = $('#form-kontakt'),
					data = $form.formParams(),
					errors = {};

				// Pflichtfelder prüfen
				if(!data.salutation) {
					errors['#salutation'] = 'Bitte wählen Sie Ihre Anrede!';
				}				

				if(!data.vorname) {
					errors['#vorname'] = 'Bitte geben Sie Ihren Vornamen ein!';
				}

				if(!data.nachname) {
					errors['#nachname'] = 'Bitte geben Sie Ihren Nachnamen ein!';
				}

				if(!data.email) {
					errors['#email'] = 'Bitte geben Sie Ihre E-Mail-Adresse ein!';
				} else if(!isValidEmail(data.email)) {
					errors['#email'] = 'Bitte überprüfen Sie Ihre E-Mail-Adresse!';
				}

				if(!data.betreff) {
					errors['#betreff'] = 'Bitte geben Sie den Betreff ein!';
				}

				if(!data.message) {
					errors['#message'] = 'Sie haben keine Nachricht eingegeben!';
				}

				// Fehlermeldung anzeigen
				if(!_.isEmpty(errors)) {
					$form.on('focus.kontakt', '.is-error', function() {
						$(this).removeClass('is-error');
					})
					return showError(errors, $form);
				}

				// API Parameter
				data.action = 'sendKontakt';

				// Internet?
				if(!isConnected()) {
					return displayNoConnectionError();
				}

				// Zeige Loadingspinner
				UI.toggleLoading();

				// API Call
				$.post(JET.backendURL, data).done(function(data) {

					UI.alert(data.msg, function() {

						// Formular zurücksetzen
						$form[0].reset();

						// Schliesse Modal
						UI.modalClose();

					});
				}).fail(function(jqXHR) {

					// Internet?
					if(isOfflineError(jqXHR)) {
						return displayNoConnectionError();
					}

					// Zeige Fehlermeldung
					var message = jqXHR.responseJSON.msg || '<b>Ein Fehler ist aufgetreten!</b>';
					UI.alert(message);

				}).always(function() {
					// Verberge Loadingspinner
					UI.toggleLoading();
				});

			});
		},
		unbindEvents: function() {

			// Eventhandler entfernen
			JET.off('.kontakt');
			el.off('.kontakt');

			// Fehlermeldung verbergen
			$('#form-kontakt').off('.kontakt');
			$('#form-kontakt').find('.error').remove();
			$('#modal-secondary').show();			
		}
	};

	return config;

});

JET.controller('Impressum', function(el) {

	var config = {
		title: 'Impressum',
		//template: 'Impressum.html',
		template: JET.backendURL+'?action=impressum&cc=' + (new Date()).getTime(),
		viewWillAppear: function() {

			$('#modal-primary').text('Zurück');
			$('#modal-secondary').hide();

		},
		bindEvents: function() {
			JET.on('modal.primary.click.impressum', function(e) {
				UI.modalClose();
			});
		},
		unbindEvents: function() {
			JET.off('.impressum');
			$('#modal-secondary').show();			
		}
	};

	return config;

});

JET.controller('Datenschutz', function(el) {

	var returnTo = false;

	var config = {
		title: 'AGB/Datenschutz',
		template: 'Datenschutz.html',
		viewWillAppear: function(args) {

			$('#modal-primary').text('Zurück');
			$('#modal-secondary').hide();

			// Im Registrierungsformular gibt es einen Link hierhin.
			// Damit wir die Formulareingaben wieder herstellen können, werden die bisherigen Daten übergeben.
			// Hier werden die Daten zwischengespeichert.
			if(typeof args !== 'undefined' && args['returnTo']) {
				returnTo = args.returnTo;
			}

		},
		bindEvents: function() {

			// Eventhandler Model schliessen
			JET.on('modal.primary.click.datenschutz', function(e) {

				// Wurd der Controller aus der Registrierung aufgerufen?
				if(returnTo) {
					// Dann führe dahin zurück und übergebe die Daten
					UI.modal(returnTo.controller, returnTo.args || null);
				} else {
					// Schliesse das Modal
					UI.modalClose();
				}
				returnTo = false;
			});

			// Google Analytics erlaubt?
			if(localStorage['trackMe'] === '1') {
				$('#doNotTrack').attr('checked', 'checked');
			} else {
				$('#doNotTrack').removeAttr('checked');
			}

			// Eventhandler für Änderungen an Google Analytics
			el.on('change.datenschutz', '#doNotTrack', function() {
				if($(this).attr('checked') === 'checked') {
					localStorage['trackMe'] = '1';
					ga.init();
				} else {
					// LocalStorage wird dort gesetzt.
					ga.stopTracking();
				}
			});

		},
		unbindEvents: function() {

			// Eventhandler entfernen
			JET.off('.datenschutz');
			el.off('.datenschutz');

			// Button "Abbrechen" weider anzeigen.
			$('#modal-secondary').show();			
		}
	};

	return config;

});