JET.controller('Login', function(el) {

	var config = {
		title: 'Login',
		template: 'Login.html',
		viewWillAppear: function() {

			// Buttontext für den Modalbutton rechts
			$('#modal-primary').text('Login');

			// Lösche ggf. vorhandene Fehlermeldungen
			el.find('.error').remove();
			el.find('.is-error').removeClass('is-error');

		},
		bindEvents: function() {

			// Eventhandler "Login"
			JET.on('modal.primary.click.login', doLogin);
			el.on('submit.login', '#form-login', doLogin);

			// Eventhandler "Passwort vergessen"
			el.on('click.login', '#login-forgotpassword', function(e) {
				e.preventDefault();
				// Zeige Controller
				UI.modal('Password');
			});

			// Eventhandler "Registrieren"
			el.on('click.login', '#login-register', function(e) {
				// Zeige Controller
				UI.modal('Register');
			});

		},
		unbindEvents: function() {

			// Eventhandler löschen
			JET.off('.login');
			el.off('.login');
			// Formular zurücksetzen
			$('#form-login')[0].reset();

			// Fehlermeldungen löschen
			el.find('.error').remove();
			el.find('.is-error').removeClass('is-error');
		}
	};

	/**
	 * Callbackfunction "Login"
	 * @param e
	 * @returns {*}
	 */
	function doLogin(e) {
		
		e.preventDefault();

		var $form = $('#form-login'), 
			params = $form.formParams(),
			errors = {};

		// Tastatur ausblenden
		$form.find('input').blur();

		// Formularevents löschen
		$form.off('focus.login');

		// Fehlerprüfung
		if(params.email == '') {

			errors['#user-email'] = 'Bitte gib Deine E-Mail-Adresse ein!';

		} else if(!isValidEmail(params.email)) {

			errors['#user-email'] = 'Bitte überprüfe Deine E-Mail-Adresse!';

		}
		if(params.password == '') {
			errors['#user-pass'] = 'Bitte gib Dein Passwort ein!';
		}

		// Zeige Fehlermeldung
		if(!_.isEmpty(errors)) {
			$form.on('focus.login', '.is-error', function() {
				$(this).removeClass('is-error');
			})
			return showError(errors, $form);
		}

		// API Parameter
		params.action = 'login';

		// Internet?
		if(!isConnected()) {
			return displayNoConnectionError();
		}

		// Zeige Loadingspinner
		UI.toggleLoading();

		// API Call
		$.post(JET.backendURL, params, 'json').done(function(data) {

			// Sicherheitshalber den Focus noch einmal entfernen
			$form.find('input').blur();

			// Aktualisiere User
			User.setData(data);

			// Registriere Pushnachrichten
			JET.registerPush();

			// Schliesse Modal
			UI.modalClose();

			// Schliesse Navigation
			UI.closeNav();

			// Zeige Preisalarm
			UI.show('Preisalarm');

		}).fail(function(jqXHR, textStatus, errorThrown) {

			// Internet?
			if(isOfflineError(jqXHR)) {
				return displayNoConnectionError();
			}

			// Zeige Fehlermeldung
			var response = JSON.parse(jqXHR.responseText);
			UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />' + response.msg);

		}).always(function() {

			// Verberge Loadingspinner
			UI.toggleLoading();
		});

	}

	return config;

});

JET.controller('Password', function(el) {

	var config = {
		title: 'Passwort vergessen?',
		template: 'Passwort.html',
		bindEvents: function() {

			// Eventhandler "Passwort anfordern"
			JET.on('modal.primary.click.passwort', function(e) {
				e.preventDefault();
				sendForm();
			});

			// Eventhandler Form submit => "Passwort anfordern"
			el.on('submit.passwort', '#form-password', function(e) {
				e.preventDefault();
				sendForm();
			});

			// Eventhandler "Abbrechen"
			JET.on('modal.secondary.click.passwort', function(e) {
                UI.modalClose();
            });
		},
		unbindEvents: function() {

			// Eventhandler entfernen
			JET.off('.passwort');
			el.off('.passwort');

			// Formular zurücksetzen
			$('#form-password')[0].reset();

			// Fehlermeldungen entfernen
			el.find('.error').remove();
			el.find('.is-error').removeClass('is-error');
		}
	};

	/**
	 * Passwort anfordern
	 * @returns {*}
	 */
	function sendForm() {

		var email = el.find('#user-email').val();

		// Fehlerprüfung
		if(email.length < 3 || !isValidEmail(email)) {
			UI.alert('Bitte überprüfe Deine E-Mail-Adresse');
			return;
		}

		// Internet?
		if(!isConnected()) {
			return displayNoConnectionError();
		}

		// Zeige Loadingspinner
		UI.toggleLoading();

		// API Parameter
		var params = {
			action: 'restorePassword',
			email: email
		};

		// API Call
		$.post(JET.backendURL, params, 'json').done(function(data) {

			if(data.status == 'success') {

				// Zeige Hinweistext
				UI.alert(data.msg, function() {
					// Schliesse Modal
					UI.modalClose();
				});
			} else {
				// Zeige Fehlermeldung
				UI.alert(data.msg, function() {});

			}


		}).fail(function(jqXHR, textStatus, errorThrown) {

			// Internet?
			if(isOfflineError(jqXHR)) {
				return displayNoConnectionError();
			}

			// Zeige Fehlermeldung
			var response = JSON.parse(jqXHR.responseText);
			UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />' + response.msg);

		}).always(function() {
			// Focus entfernen
			el.find('#form-password input').blur();

			// Verberge Loadingspinner
			UI.toggleLoading();
		});

	};

	return config;

});

JET.controller('Logout', function(el) {

	var config = {
		title: 'Abmelden',
		template: 'Logout.html',
		viewWillAppear: function() {

			// Setze Hintergrundfarbe des Modalheaders
			/*$('#modal-header').css('backgroundColor', '#e74c3c');*/
            $('#modal-header').css('backgroundColor', '#fbba00');

			// Setze Buttontext
			//$('#modal-primary').text('Logout');
            $('#modal-primary').text('');
            $('#modal-secondary').text('');

		},
		bindEvents: function() {

			// Eventhandler "Logout"
			//JET.on('modal.primary.click.logout', function(e) {
            $("#body-logout").click(function() {

				// Userdaten löschen
				User.setData('');

				// Logintoken löschen
				User.setToken('');

				// Modal schliessen
				UI.modalClose();

				// Navigation schliessen
				UI.closeNav();

				// Zeige Startseite
				UI.show('Startseite');
			});
            
            $(".cancelmania").click(function() {
                UI.modalClose();
			});

		},
		unbindEvents: function() {

			// Entferne Eventhandler
			JET.off('.logout');
			el.off('.logout');

			// Headerfarbe zurücksetzen
			$('#modal-header').css('backgroundColor', '');
		}
	};

	return config;

});

JET.controller('Register', function(el) {

	var formData = false;
	var config = {
		title: 'Registrieren',
		template: 'Register.html',
		viewWillAppear: function(args) {

			// Wenn die AGB / Datenschutzbestimmungen aufgrufen werden, springen wir anschliessend hier hin zurück.
			// Damit dann nicht alle Eingaben futsch sind, übergeben wir die Daten an den anderen Controller und erhalten diese dann wieder zurück.
			if(args) {
				formData = args;
			}

			// Fehlermeldungen entfernen
			el.find('.error').remove();
			el.find('.is-error').removeClass('is-error');

		},
		bindEvents: function() {

			// Im Formular gibt es einen Link auf die AGB / Datenschutz Seite
			// Damit wir unsere Eingaben nicht verlieren, und hierhin zurück kommen, übergeben wir die Daten an den Controller (siehe Controller Datenschutz).
			el.on('click.register', 'a[data-modal]', function(e) {

				// Öffne Modal
				UI.modal(this.getAttribute('data-modal'), {
					returnTo: {
						controller: 'Register',
						type: 'modal',
						args: $('#form-register').formParams()
					}
				});

			});

			// Eventhandler Daten speichern
			el.on('submit.register', '#form-register', doRegister);
			JET.on('modal.primary.click.register', doRegister);

			// Wenn es schon Formulardaten gibt, setze Formularwerte
			if(formData) {
				$('#form-register').formParams(formData);
			}

		},
		unbindEvents: function() {

			// Eventhandler löschen
			JET.off('.register');
			el.off('.register');

			// Fomrular resetten
			$('#form-register')[0].reset();

			// Fehlermeldungen entfernen
			el.find('.error').remove();
			el.find('.is-error').removeClass('is-error');
			formData = false;
		}
	};

	/**
	 * Callbackfunction "Registrieren"
	 * @param e
	 * @returns {*}
	 */
	function doRegister(e) {
		e.preventDefault();

		// Fehlermeldungen entfernen
		el.find('.error').remove();
		el.find('.is-error').removeClass('is-error');

		var $form = $('#form-register'), 
			params = $form.formParams(),
			errors = {};

		// Tastatur ausblenden
		$form.find('input').blur();

		// Formular Eventhandler entfernen
		$form.off('focus.register');

		// Fehlerprüfung
		if(params.email == '') {

			errors['#register-email'] = 'Bitte gib Deine E-Mail-Adresse ein!';

		} else if(!isValidEmail(params.email)) {

			errors['#register-email'] = 'Bitte überprüfe Deine E-Mail-Adresse!';

		}
		if(params.password == '') {
			errors['#register-password'] = 'Bitte gib Dein Passwort ein!';
		} else if(params.password !== params.password2) {
			errors['#register-password2'] = 'Die Passwörter stimmen nicht überein!';
		}

		if(!params.agb) {

			errors['#register-agb'] = 'Du musst die "Allgemeinen Geschäftsbedingungen" und die "Datenschutzerklärung" akzeptieren!';

		}

		// Zeige Fehlermeldung
		if(!_.isEmpty(errors)) {
			$form.on('focus.register', '.is-error', function() {
				$(this).removeClass('is-error');
			});
			return showError(errors, $form);
		}

		// API Parameter
		params.action = 'register';

		// Internet?
		if(!isConnected()) {
			return displayNoConnectionError();
		}

		// Zeige Loadingspinner
		UI.toggleLoading();

		// API Call
		$.post(JET.backendURL, params, 'json').done(function(data) {

			// Aktualisiere Userdaten
			User.setData(data.user);

			// Registriere Pushnachrichten
			JET.registerPush();

			// Zeige nächsten Schritt
			UI.modal('ProfilSettings');


		}).fail(function(jqXHR, textStatus, errorThrown) {

			// Internet?
			if(isOfflineError(jqXHR)) {
				return displayNoConnectionError();
			}

			// Zeige Fehlermeldung
			var response = JSON.parse(jqXHR.responseText);
			UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />' + response.msg);


		}).always(function() {

			// Verberge Loadingspinner
			UI.toggleLoading();
			formData = false;

		});

	}

	return config;

});