JET.controller('PreisalarmNoLogin', function(el) {

	// Statischer Controller, der aufgerufen wird, solange der User nicht eingeloggt ist.
	// Siehe UI.show

	var config = {
		title: 'Preisalarm',
		template: 'Preisalarm_nologin.html'
	};

	return config;

});

JET.controller('Preisalarm', function(el) {

		// Slider für die Kraftstoffpreise
	var swipeViews = [],
		// Hilfsvariable
		isTemplateLoaded = false,

		config = {

			// Seitentitel
			title: 'Preisalarm',

			// View
			template: 'Preisalarm.html',

			// Der Controller ist dem DOM hinzugefügt
			bindEvents: function() {

				// Stellt sicher, das der Backbutton nicht mehr angezeigt wird
				JET.one('UI.idle', function() {
					UI.handle('hide.back');
				});

				// Eventhandler, um die Tankstellendetailseite anzuzeigen
				// Klick auf die Map oder die Adresse
				el.on('click', '.station > img, .address', function(e) {

					UI.show('Tankstellensuche', $(this).parents('.station')[0].getAttribute('data-station-number'));

				});

				// Eventhandler um einen neuen Preisalarm einzurichten
				el.on('click.preisalarm', '.button-create-alarm', function(e) {
					e.preventDefault();

					UI.modal('PreisalarmEdit', this.getAttribute('data-station'));
				});

				// Eventhandler um einen bestehenden Preisalarm zu bearbeiten
				el.on('click.preisalarm', '.button-edit-alarm', function(e) {
					e.preventDefault();
					
					UI.modal('PreisalarmEdit', this.getAttribute('data-station'));
				});

				// Eventhandler um die Tankstelle zu löschen
				el.on('click.preisalarm', '.delete-station', function(e) {

					var station_number = $(this).parents('.station').data('station-number'),
						station = _.find(User.getData('stations'), function(station) { return station.station_id == station_number }),
						hasAlarm = station.has_alarm,
						msg = hasAlarm ? ' Der Preisalarm wird ebenfalls gelöscht.' : '';
				
					// Zeige Alert
					UI.alert('<b>Bist Du sicher?</b><br />Die JET Tankstelle wird aus Deinen Favoriten entfernt.'+msg, function() {
						
						if(!isConnected()) {
							// Kein Internet
							return displayNoConnectionError();
						}

						// API Call
						$.post(JET.backendURL, {
							action: 'removeStation',
							station_number: station_number,
							token: User.getToken()
						}).done(function(data) {

							// Aktualisiere User
							User.setData(data.user);
							
							setTimeout(function() {
								// Aktualisiere View
								config.refresh();

							}, 100);

						}).fail(function(jqXHR) {

							if(isOfflineError(jqXHR)) {
								// Kein Internet
								return displayNoConnectionError();
							}

							// Zeige API Fehlermeldung
							var o = jqXHR.responseJSON || {msg:'API Fehler'};

							UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />'+o.msg);

						});
						
					}, true);

				});

				// Eventhandler um eine neue Stammtankstelle einzurichten
				el.on('click.preisalarm', '.add-station', function() {
					UI.show('Tankstellensuche', 'locate');
				});

			},

			// Der Controller wird gleich entfernt werden
			unbindEvents: function() {

				// Lösche Slider
				for(var i=0; i<swipeViews.length; ++i) {
					swipeViews[i].destroy();
					swipeViews[i] = null;
				}
				swipeViews = [];

				// Lösche Eventhandler
				el.off('.preisalarm');

				

			},

			// Template ist geladen und bereit
			templateLoaded: function() {
				
				isTemplateLoaded = true;

				render();
			},

			// View soll angezeigt werden
			viewWillAppear: function() {
				
				if(isTemplateLoaded) {
					render();
				}
			},

			// View soll aktualisiert werden
			refresh: function() {
				// Resette die SwipeViews, da diese neue erstellt werden
				for(var i=0; i<swipeViews.length; ++i) {
					swipeViews[i].destroy();
					swipeViews[i] = null;
				}
				swipeViews = [];
				render();
			}
		};

	/**
	 * Erstellt die Slider für die Kraftstoffpreise
	 */
	function buildSwipeViews() {

		// Welcher Preis soll angezeigt werden?
		// Wird gesetzt, wenn ein Preisalarm eingerichtet wird.
		var priceToShow = window.localStorage['jet.app.preferredFuelType'] || '';
	
		el.find('.price-wrapper').each(function(index) {

			// Setup SwipeView
			// Doku: http://cubiq.org/swipeview
			var i,
				page,
				$carousel = $(this),
				slides = $carousel.find('.price-item').detach(),
				$indicator = $carousel.parent().find('.indicator'),
				lis = [];

			// Erstelle SwipeView
			swipeViews[index] = new SwipeView($carousel[0], {
				numberOfPages: slides.length,
				hastyPageFlip: true
			});

			// Erstelle 3 Seiten
			for(i=0; i<3; ++i) {
				page = i==0 ? slides.length -1 : i-1;
				swipeViews[index].masterPages[i].innerHTML = slides[page].outerHTML;
			}

			// Erstelle neue Seiten nach jedem Swipe
			swipeViews[index].onFlip(function() {
				var elem, upcoming, i;
				for(i=0; i<3; ++i) {
					upcoming = swipeViews[index].masterPages[i].dataset.upcomingPageIndex;

					if(upcoming != swipeViews[index].masterPages[i].dataset.pageIndex) {
						elem = swipeViews[index].masterPages[i].querySelector('div.price-item').parentNode;
						elem.innerHTML = slides[upcoming].outerHTML;
					}
				}

				// Aktualisiere Pageindicator
				$indicator.find('.is-active').removeClass('is-active');
				$indicator[0].childNodes[swipeViews[index].pageIndex].className = 'is-active';
			});


			// Welches Slide soll als erstes angezeigt werden
			var pageToGo = 0;

			// Erstelle Pageindicator
			for(i=0; i<slides.length; ++i) {
				lis.push('<li'+(i==0 ? ' class="is-active"' : '')+'></li>');

				// Dieser Preis soll angezeigt werden
				if(priceToShow && slides[i].className.indexOf(priceToShow) > -1) {
					pageToGo = i;
				}

			}

			$indicator.html(lis.join(''));

			if(pageToGo > 0) {
				// Zeige Wunschkraftstoff
				swipeViews[index].goToPage(pageToGo);
			}

		});
	}

	/**
	 * Rendert den View
	 */
	function render() {

		var userStations = User.getData('stations'),
			stationsToDisplay = [];

		// Check, ob die Stammtankstelle noch existiert
		// Eventuell ist die Tankstelle zwischenzeitlich geschlossen worden
		// und daher nicht mehr vorhanden.
		_.each(userStations, function(station_info) {
		
			if(!_.isEmpty(Stations.getStation(station_info.station_id))) {
				stationsToDisplay.push(station_info);
			}
		});

		// Haben wir Tankstellen zum anzeigen?
		if(stationsToDisplay.length > 0) {

			// Aktualisiere View
			config.models.userStations({stations: stationsToDisplay});

			setTimeout(function() {
				// Erstelle Slider
				buildSwipeViews();
			}, 0);
			
		} else {

			// Aktualisiere View
			config.models.userStations();			
		}

	}

	return config;

});

JET.controller('PreisalarmEdit', function(el) {

	// Erstelle / Bearbeite Preisalarm

	var station_number, 
		isTemplateLoaded = false,

		config = {
			title: 'Preisalarm',
			template: 'Preisalarm_ErstellenBearbeiten.html',

			// Die Stationsnummer wird durch UI.modal('PreisalarmEdit', 'XJ00000') übergeben
			viewWillAppear: function(station_id) {

				station_number = station_id;

				// Setze Buttentext des rechten Modalbuttons
				$('#modal-primary').text('Speichern');

				// Ggf. sind noch Fehlermeldungen
				el.find('.error').remove();
				el.find('.is-error').removeClass('is-error');

				if(isTemplateLoaded) {
					preisAlarmRender(station_number, config);
				}

			},
			bindEvents: function() {

				// Eventlistener für den Modalbutton rechts (Speichern)
				JET.on('modal.primary.click.preisalarm_edit', preisAlarmSave);
				el.on('submit.preisalarm_edit', '#preisalarm-edit', preisAlarmSave);

				// Eventlistener für die Selectliste "Kraftstoffart"
				el.on('change.preisalarm_edit', '#fueltype', function(e) {

					// Wenn die Kraftstoffart gewählt wurde, wollen wir den aktuellen Preis
					// als Placeholder anzeigen

					var current_price,
						user = User.getData(),
						fuelType = $(this).val();

					for(var i = 0, j = user.stations.length; i<j; ++i) {

						if(user.stations[i].station_id == station_number) {
							
							current_price = user.stations[i].prices[fuelType];
							break;
							
						}

					}

					if(current_price) {
						setPricePlaceholder(current_price);
						// Der Preis muss noch auf 2 Nachkommastellen gekürzt werden
						//el.find('#price').attr('placeholder', current_price.substring(0,current_price.length-1));
					}
					
					

				});

				// Eventhandler um den Preisalarm zu löschen
				el.on('click', '.button-delete', function(e) {

					e.preventDefault();

					UI.alert('<b>Bist Du sicher?</b>', function() {

						if(!isConnected()) {
							return displayNoConnectionError();
						}

						// API Call
						$.post(JET.backendURL, {
							action: 'deleteAlarm',
							token: User.getToken()
						}).done(function(data) {

							// Aktualisiere User
							User.setData(data.user);

							// Schliesse Modal und aktualisiere den darunterliegenden View (Preisalarm)
							UI.modalClose(true);

						}).fail(function(jqXHR) {

							// No Internet?
							if(isOfflineError(jqXHR)) {
								return displayNoConnectionError();
							}

							var response = jqXHR.responseJSON || {msg:"API Fehler"};
							UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />' + response.msg);

						});

					}, true);

				});

				// Manche Androidversionen haben Probleme mit der Darstellung
				// bzw. dem Keyboard (Punkt oder Kommaeingabe nicht möglich)
				// bei input[type=number].
				// Deshalb wandeln wir die Felder in normale input[type=text]
				if(isAndroid()) {
					el.find('input[type="number"]').attr('type', 'text');
				}

			},
			unbindEvents: function() {

				// Entferne Eventhandler
				JET.off('.preisalarm_edit');
				el.off('.preisalarm_edit');

				// Resette Formular
				var $form = $('#preisalarm-edit');
				if($form.length > 0) {
					$form[0].reset();
				}
				// Entferne Formeventhandler
				$form.off('.preisalarm_helper');

				// Entferne Fehlermeldungen
				el.find('.error').remove();
				el.find('.is-error').removeClass('is-error');
			},
			templateLoaded: function() {
				
				isTemplateLoaded = true;

				preisAlarmRender(station_number, config);
			}
		};


	/**
	 * Setzt den Placeholdertext für die ausgewählte Kraftstoffsorte
	 * @param price
	 */
	function setPricePlaceholder(price) {
		el.find('#price').attr('placeholder', price.substring(0, price.length-1));
	}

	/**
	 * Rendert den View
	 * @param station_number
	 * @param config
	 */
	function preisAlarmRender(station_number, config) {

		var station = Stations.getStation(station_number),
			user = User.getData(),
			// Defaults
			alarm = {
				weekday_start: '09:00',
				weekday_end: '20:00',
				weekend_start: '10:00',
				weekend_end: '20:00',
				valid_until: 0,
				station_id: ''
			},
			current_price,
			info;

		// Hole Wunschkraftstoffsorte
		// Wird beim Speichern eines Preisalarms gesetzt
		var priceToShow = window.localStorage['jet.app.preferredFuelType'] || '';

		// Suche aktiven Alarm und setzte aktullen Kraftstoffpreis
		for(var i = 0, j = user.stations.length; i<j; ++i) {

			// Es gibt einen Alarm
			// Der muss nicht zwangsläufig zu dieser Tankstelle gehören.
			if(!!user.stations[i].has_alarm) {
				alarm = $.extend({}, user.stations[i].has_alarm);
				alarm.price = ('' + user.stations[i].has_alarm.price + '').split(',').join('.');
				alarm.station_id = user.stations[i].station_id;
			}

			// Setze den Preis für die Standardkraftstoffsorte
			if(user.stations[i].station_id == station_number) {
				current_price = user.stations[i].prices[priceToShow || 'L003'];
				break;
			}

		}

		// Erstelle Objekt mit Viewvariablen
		info = {station: station, alarm: alarm, user:user};

		// Aktualisiere View
		config.models.stationInfo(info);
		config.models.alarmInfo(info);

		if(isAndroid()) {

			// Android hat Probleme mit der Darstellung von input[type=time]
			// Deshalb dieser Fix

			// Die Zeiten sind inklusive Sekunden (00:00:00)
			// Die brauchen wir in der Darstellung nicht.
			alarm.weekday_start = alarm.weekday_start.substr(0,5);
			alarm.weekday_end = alarm.weekday_end.substr(0,5);
			alarm.weekend_start = alarm.weekend_start.substr(0,5);
			alarm.weekend_end = alarm.weekend_end.substr(0,5);

			// Wandle input-Elemente
			// readonly!
			// Werte werden über das datePicker-Plugin geändert
			$('#preisalarm-edit').find('input[type="time"]').attr({
				'type':'text', 
				'readonly':'readonly'
			}).addClass('pseudo-time');

			// Eventhandler für Clickevent
			el.on('click.preisalarm_edit', '.pseudo-time', function(e) {
				e.preventDefault();

				var element = this,
					presetDate = new Date(),
					currentValue = this.value;

				// Erstelle aktuelle Uhrzeit
				if(currentValue.indexOf(':') > -1) {
					currentValue = currentValue.split(':');
					presetDate.setHours(currentValue[0]);
					presetDate.setMinutes(currentValue[1]);
				}

				// Zeige DatePicker-Plugin
				datePicker.show({
					date: presetDate,
					mode: 'time'
				}, function(date) {

					// Der Datumspart interessiert uns nicht...
					var array = date.split(' '),
						time = array[1].split(':');

					// (manche) Androiden geben "3:12:12 vorm." bzw. "3:12:12 nachm." zurück
					if(array[2] && array[2].indexOf('nachm') > -1) {
						time[0] = parseInt(time[0]) + 12;
					}

					// Setze Wert
					element.value = time[0]+':'+time[1];
					element.blur();
				});
			});
		}

		if(alarm.station_id == station_number) {

			// Aktuelle Tankstelle hat einen Preisalarm

			setTimeout(function() {

				var androidFix = {};
				if(isAndroid()) {
					// Wandle Preis in deutsches Format (Komma als Trennzeichen)
					androidFix.price = alarm.price.split('.').join(',');
				}


				// Setze Formularwerte
				$('#preisalarm-edit').formParams($.extend({
					notification_type: user.flag_push ? 'Push' : (user.flag_sms ? 'SMS' : 'Mail')
				}, alarm, androidFix));

			}, 500);
			

		} else {

			if(priceToShow !== '') {

				// Setze Formularwerte
				$('#preisalarm-edit').formParams({
					fueltype: priceToShow
				});
			}
			
		}

		// Setze Placeholder
		setPricePlaceholder(current_price);
		//$('#preisalarm-edit').find('#price').attr('placeholder', current_price);

	};

	/**
	 * Preisalarm soll gespeichert werden
	 * @param e
	 * @returns {*}
	 */
	function preisAlarmSave(e) {

		// Keyboard ausblenden
		el.find('input').blur();

		// Formsubmit unterbinden
		e.preventDefault();

		var $form = $('#preisalarm-edit'), 
			params = $form.formParams(),
			errors = {},
			userStation = _.find(User.getData('stations'), function(station) { return station.station_id == station_number });

		// Formeventlistener entfernen
		$form.off('focus.preisalarm_helper');

		// Wandle input Value in float
		params.price = string2number(params.price, 3);

		// Errorcheck
		if(params.price == '') {

			errors['#price'] = 'Bitte gib Deinen Wunschpreis ein!';

		} else if((params.price+'').split('.')[1].substr(-1) != '0') {

			errors['#price'] = 'Bitte gib nur ganze Beträge als Wunschpreis ein!';

		} else if(params.price > parseFloat(userStation.prices[params.fueltype].split(',').join('.'))) {

			errors['#price'] = 'Dein Wunschpreis liegt über dem aktuellen Preis!?';

		}

		// Zeige Fehlermeldung
		if(!_.isEmpty(errors)) {
			$form.on('focus.preisalarm_helper', '.is-error', function() {
				$(this).removeClass('is-error');
			})
			return showError(errors, $form);
		}

		// Setze API-Parameter
		params.action = 'editAlarm';
		params.token = User.getToken();

		// Internet?
		if(isConnected()) {

			// Zeige Spinner
			UI.toggleLoading();

			// API Call
			$.post(JET.backendURL, params, 'json').done(function(data) {

				// Aktualisiere User
				User.setData(data.user);

				// Setze Wunschkraftstoff
				window.localStorage['jet.app.preferredFuelType'] = params.fueltype;

				// Aktiviere Pushnachrichten
				JET.registerPush();

				// Schliesse Modal und aktualisiere den darunterliegenden View (Preisalarm)
				UI.modalClose(true);

			}).fail(function(jqXHR, textStatus, errorThrown) {

				// Internet?
				if(isOfflineError(jqXHR)) {
					return displayNoConnectionError();
				}

				// Zeige Fehlermeldung
				var response = jqXHR.responseJSON || {msg:"API Fehler"};
				UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />' + response.msg);
				

			}).always(function() {
				// Hide Spinner
				UI.toggleLoading();
			});
		} else {

			// Kein Internet!
			displayNoConnectionError();

		}

	};

	return config;

});
