JET.controller('Profil', function(el) {

	var isTemplateLoaded = false,
		verify_classes = 'no-phone phone-verified phone-unverified phone-awaiting-verification',
		isDirty = false;

	var config = {
		title: 'Profil',
		template: 'Profil.html',
		templateLoaded: function() {

			isTemplateLoaded = true;
			render();

		},
		viewWillAppear: function() {

			// Setze Buttontext für den rechten Modalbutton
			$('#modal-primary').text('Speichern');

			if(isTemplateLoaded) {
				render();
			}

		},
		bindEvents: function() {

			// Eventhandler zum Speichern der Daten
			el.on('submit.profile', '#edit-profile', saveForm);
			JET.on('modal.primary.click.profile', saveForm);

			// Eventhandler zur Auswahl eines Profilbildes
			el.on('click.profile', 'label[for="picture"]', function(e) {

				// Bildquelle auswählen
				var alertBody = [
					'<b>Profilbild auswählen</b><br /><br />',
					/*'<button type="button" class="picture-choose-button choose-camera"><i class="fa fa-camera"></i> Foto aufnehmen</button>',
					'<button type="button" class="picture-choose-button choose-album"><i class="fa fa-picture-o"></i> Foto auswählen</button>'*/
                    '<button type="button" class="button-primary button-spacer choose-album"><i class="fa fa-picture-o"></i> Foto auswählen</button>',
                    '<button type="button" class="button-secondary choose-camera"><i class="fa fa-camera"></i> Foto aufnehmen</button>'
					
				];


				// Eventhandler für den Alert zur Auswahl der Bildquelle
				$('#alert-body').one('click.profile', 'button', function(e) {

					var $btn = $(this), type;
					if($btn.hasClass('choose-camera')) {
						type = 'CAMERA';
					} else if($btn.hasClass('choose-album')) {
						type = 'PHOTOLIBRARY';
					}

					// Hole Foto
					getPhoto(type).done(function(imageData) {

						// Setze Vorschaubild
						$('#user-profile-picture').attr('src', 'data:image/jpeg;base64,'+imageData);
						isDirty = true;

					}).fail(function() {

						// Fehlermeldung
						UI.alert({
							body: '<b>Das hat leider nicht funktioniert!</b>',
							okbutton:'Erneut versuchen',
							cancelbutton: 'Abbrechen'
						}, function() {

							// Eventhandler für "Nochmal versuchen"
							$('label[for="picture"]', el).click();

						});

					});

					// Schliesse Alert
					$('#alert-ok').trigger('click');

				});

				// Zeige Auswahldialog für die Bildquelle
				UI.alert({
					body: alertBody.join(''),
					okbutton: '<div class="photo-chooser-cancel">Abbrechen</div><br>'
				}, function() {
					$('#alert-body').off('click.profile');
				});



			});

			// Eventhandler Telefonnummer
			el.on('change.profile', '#mobile_phone', function() {

				// Ändert der Benutzer seine Telefonnummer, ist diese ungeprüft.
				// Hier wird die benötigte CSS-Klasse gesetzt.
				$(this).parents('.verify-widget').removeClass(verify_classes).addClass('no-phone');

			});

			// Eventhandler "Telefonnummer prüfen"
			el.on('click.profile', '.verify-phone', function(e) {
				e.preventDefault();

				var $button = $(this);

				// Internet?
				if(!isConnected()) {
					return displayNoConnectionError();
				}

				// Loadingspinner
				UI.toggleLoading();

				// API Call
				// Die Telefonnummer ist bereits in der DB gespeichert.
				$.post(JET.backendURL, {

					action: 'verifyPhone',
					token: User.getToken()

				}).done(function(data) {

					if(data.status == 'success') {

						// Aktualisiere Daten
						User.setData(data.user);

						// Zeige Hinweis aus Bestätigungs-SMS
						UI.alert(data.msg);

						// Setze benötigte CSS-Klassen
						$button.parents('.verify-widget').removeClass(verify_classes).addClass('phone-awaiting-verification');

					} else {

						// Zeige Fehlermeldung
						UI.alert(data.msg);

					}


				}).fail(function(jqXHR) {

					// Internet?
					if(isOfflineError(jqXHR)) {
						return displayNoConnectionError();
					}

					// Zeige Fehlermeldung
					var message = jqXHR.responseJSON.msg || '';
					UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />'+message);

				}).always(function() {

					// Verberge Loadingspinner
					UI.toggleLoading();
				});

				
			});

			// Eventhandler zur Prüfung des Bestätigungscodes
			el.on('click.profile', '.verify-code button', function(e) {

				e.preventDefault();
				var $button = $(this),
					$input = $button.parent().find('#verify-code'),
					code = $input.val();

				// Errorcheck
				if(!(/[0-9]{4}/.test(code))) {
					UI.alert('Bitte gib den Bestätigungscode ein!');
					$input.focus();
					return;
				}

				// Internet?
				if(!isConnected()) {
					return displayNoConnectionError();
				}

				// Zeige Loadingspinner
				UI.toggleLoading();

				// API Call
				$.post(JET.backendURL, {

					action: 'verifyCode',
					code: code,
					token: User.getToken()

				}).done(function(data) {

					if(data.status == 'success') {

						// Aktualisiere User
						User.setData(data.user);

						// Setze CSS-Klassen
						$button.parents('.verify-widget').removeClass(verify_classes).addClass('phone-verified');

						// Zeige Hinweistext
						UI.alert(data.msg);

					} else {

						// Zeige Fehlermeldung
						UI.alert(data.msg);
					}

				}).fail(function(jqXHR) {

					// Internet?
					if(isOfflineError(jqXHR)) {
						return displayNoConnectionError();
					}

					// Fehlermeldung
					var message = jqXHR.responseJSON.msg || '';
					UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />'+message);


				}).always(function() {

					// Verberge Loading
					UI.toggleLoading();
				});

			});


			// Eventhandler "Profil löschen"
			el.on('click.profile', '.button-delete', function(e) {

				e.preventDefault();

				// Confirm
				UI.alert('<b>Bist Du sicher?</b><br>Dein JET KLICKe Profil wird vollständig gelöscht und kann nicht wieder hergestellt werden.', function() {
					// OK

					// Internet?
					if(!isConnected()) {
						return displayNoConnectionError();
					}

					// API Call
					$.post(JET.backendURL, {
						action: 'deleteProfile',
						token: User.getToken()
					}).done(function(data) {

						// Lösche lokale Userdaten
						User.setData('');
						// Lösche Logintoken
						User.setToken('');

						// Schliesse Modal
						UI.modalClose();
						// Schliesse Navigation
						UI.closeNav();

						// Zeige Startseite (NoLogin)
						UI.show('Startseite');

					}).fail(function(jqXHR, textStatus, errorThrown) {

						// Internet?
						if(isOfflineError(jqXHR)) {
							return displayNoConnectionError();
						}

						// Zeige Fehlermeldung
						UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />Dein Profil konnte nicht gelöscht werden.');

					});

				}, true);

			});

			// Wann immer eine Formularfeld bearbeitet wurde, wird der "isDirty" Status gesetzt.
			// Ist isDirty === false wird nicht gespeichert.
			el.on('change.profile', '#edit-profile', function() {
				isDirty = true;
			});

		},
		unbindEvents: function() {

			// Eventhandler entfernen
			JET.off('.profile');
			el.off('.profile');

			// Lösche Fehler- und Erfolgsmeldungen sowie Formularhelper
			var $form = $('#edit-profile');

			$form.off('.form_helper');
			$form[0].reset();
			$form.find('.success').remove();
			$form.find('.error').remove();
			$form.find('.info').remove();

		}
	};

	/**
	 * Rendert den View
	 */
	function render() {

		// Erstelle Kopie der Userdaten, damit wir die nicht aus Versehen überschreiben
		var data = $.extend({}, User.getData());

		// Zeige Profilbild wenn vorhanden
		if(data.picture) {
			$('#user-profile-picture').attr('src', JET.baseURL+'img/288x288/'+data.picture);
		}

		// Lösche Key
		delete data.picture;

		// Prüfe Status der Telefonverifizierung
		checkPhoneStatus(data);

		// Setze Formaulwerte
		$('#edit-profile').formParams(data);

	};

	/**
	 * Prüft den aktuellen Status der Telefonnummerverifizierung
	 * @param user Die Daten des aktuellen Benutzers
	 */
	function checkPhoneStatus(user) {
		if(!user.mobile_phone) {
			// Keine Telfonnummer eingegeben
			$('.verify-widget').removeClass(verify_classes).addClass('no-phone');
		} else if(user.flag_phone_verified == '1') {
			// Telefonnummer verifiziert
			$('.verify-widget').removeClass(verify_classes).addClass('phone-verified');
		} else if(user.has_verification_code == '1') {
			// Code angefordert aber noch nicht eingegeben
			$('.verify-widget').removeClass(verify_classes).addClass('phone-awaiting-verification');
		} else {
			// Telefonnummer noch nicht verifiziert (Code noch nicht angefordert).
			$('.verify-widget').removeClass(verify_classes).addClass('phone-unverified');
		}
	}

	/**
	 * Speichert die Formulardaten
	 * @param e
	 * @returns {*}
	 */
	function saveForm(e) {

		// Sind Änderungen vorgenommen worden?
		if(!isDirty) {
			UI.modalClose();
			return;
		}

		e.preventDefault();

		var $form = $('#edit-profile'),
			params = $form.formParams(),
			data,
			errors = {};

		// Lösche bestehende Fehlermeldungen
		$form.off('focus.form_helper');
		el.find('.error').remove();
		el.find('.is-error').removeClass('is-error');

		// Errorcheck
		if(!params.username) {
			errors['#username'] = 'Btte gib Deinen Benutzernamen ein!';
		}
		if(!params.email) {
			errors['#email'] = 'Bitte gib Deine E-Mail-Adresse ein!';
		} else if(!isValidEmail(params.email)) {
			errors['#email'] = 'Bitte überprüfe Deine E-Mail-Adresse!';
		}
		if(params.password && params.password !== $form.find('#password2').val()) {
			errors['#password'] = 'Bitte gib zweimal das gleiche Passwort ein!';
		}

		// Zeige Fehlermeldungen
		if(!_.isEmpty(errors)) {
			$form.on('focus.form_helper', '.is-error', function() {
				$(this).removeClass('is-error');
			});
			$form.parents('.container')[0].scrollTop = 0;
			return showError(errors, $form);
		}

		// Nutze FormData, um das Profilbild übertragen zu können
		data = new FormData();

		// API Parameter
		data.append('action', 'editProfile');
		data.append('token', User.getToken());

		// Übertrage die Formulardaten in das FormData Objekt
		for(var key in params) {
			data.append(key, params[key]);
		}

		// Übertrage das Profilbild, wenn die Bildquelle eine data-URI ist (also ein neues Bild ausgewählt wurde)
		var img_src = $('#user-profile-picture').attr('src');
		if(img_src.indexOf('data') === 0) {
			data.append('picture', img_src);
		}

		// Internet?
		if(!isConnected()) {
			return displayNoConnectionError();
		}

		// Zeige Loadingspinner
		UI.toggleLoading();

		// API Call
		$.ajax({
			url: JET.backendURL,
			data: data,
			type: 'POST',
			processData: false, // Wir senden ein FormData Objekt
			contentType: false,
			cache: false
		}).done(function(data) {
			
			if(data.status == 'success') {

				// Aktualisiere Userdaten
				User.setData(data.user);

				// Zeige Erfolgsmeldung
				showSuccess('Die Änderungen wurden gespeichert.', $form);

				// Zeige Hinweis aus Telefonnummerverifizierung (soweit vorhanden)
				if(data['msg']) {
					showMessage(data.msg, $form);
				}

				// Form ist sauber!
				isDirty = false;

				// Prüfe Telefonstatus
				checkPhoneStatus(data.user);

			} else {

				// Zeige Fehlermeldung
				showError(data.msg, $form);
			}


		}).fail(function(jqXHR, textStatus, errorThrown) {

			// Internet?
			if(isOfflineError(jqXHR)) {
				return displayNoConnectionError();
			}

			// Fehlermeldung
			UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />Die Änderungen konnten nicht gespeichert werden.');

		}).always(function() {
			// Scrollt to top
			$form.parents('.container')[0].scrollTop = 0;

			// Verberge Loadingspinner
			setTimeout(UI.toggleLoading, 100);
		});

	};

	return config;

});

JET.controller('ProfilSettings', function(el) {

	var isTemplateLoaded = false;
	var verify_classes = 'no-phone phone-verified phone-unverified phone-awaiting-verification';

	var config = {
		title: 'Profil vervollständigen',
		template: 'ProfilSettings.html',
		templateLoaded: function() {

			isTemplateLoaded = true;
			render();

		},
		viewWillAppear: function() {

			// Verberge den "Abbrechen"-Button
			$('#modal-secondary').hide();

			if(isTemplateLoaded) {
				render();
			}

		},
		bindEvents: function() {

			// Eventhandler zum Speichern des Formulars
			el.on('submit.initial', '#edit-profile-initial', saveForm);
			JET.on('modal.primary.click.initial', saveForm);

			// Lösche den roten Rahmen um das Feld Anrede, nach der Auswahl
			el.one('change.initial', '[name="salutation"]', function() {
				$(this).closest('.row').removeAttr('style');
			});

			// Lösche (oder setze) den roten Rahmen um die Formularfelder, nachdem der Benutzer hier etwas eingegeben hat.
			el.on('change.initial', '#firstname, #lastname, #username', function() {
				var $this = $(this);
				if($this.val() == '') {
					$this.closest('.input-wrapper').css('border-color', '#900')
				} else {
					$this.closest('.input-wrapper').css('border-color', '');
				}
			});

			// Eventhandler zur Bildauswahl (siehe Profil)
			el.on('click.initial', 'label[for="picture"]', function(e) {

				var alertBody = [
					'<b>Profilbild auswählen</b><br /><br />',
					'<button type="button" class="picture-choose-button choose-camera"><i class="fa fa-camera"></i> Foto aufnehmen</button>',
					'<button type="button" class="picture-choose-button choose-album"><i class="fa fa-picture-o"></i> Foto auswählen</button>'
				];


				$('#alert-body').one('click.initial', 'button', function(e) {

					var $btn = $(this), type;
					if($btn.hasClass('choose-camera')) {
						type = 'CAMERA';
					} else if($btn.hasClass('choose-album')) {
						type = 'PHOTOLIBRARY';
					}

					getPhoto(type).done(function(imageData) {

						console.log('Got Image');

						$('#user-profile-picture').attr('src', 'data:image/jpeg;base64,'+imageData);
						isDirty = true;

					}).fail(function() {

						UI.alert({
							body: '<b>Das hat leider nicht funktioniert!</b>',
							okbutton:'Erneut versuchen',
							cancelbutton: 'Abbrechen'
						}, function() {

							$('label[for="picture"]', el).click();

						});

					});

					$('#alert-ok').trigger('click');

				});

				UI.alert({
					body: alertBody.join(''),
					okbutton: '<div class="photo-chooser-cancel">Abbrechen</div>'
				}, function() {
					$('#alert-body').off('click.initial');
				});



			});

		},
		unbindEvents: function() {

			// Eventhhandler entfernen
			JET.off('.initial');
			el.off('.initial');
		}
	};

	/**
	 * Rendert den View
	 */
	function render() {

		// Erstelle Kopie der Benutzerdaten, um versehentliches Überschreiben zu verhindern.
		var data = $.extend({}, User.getData());

		// Zeige Profilbild (falls vorhanden)
		if(data.picture) {
			$('#user-profile-picture').attr('src', JET.baseURL+'img/288x288/'+data.picture);
		}

		delete data.picture;

		// Aktualisiere die CSS-Klassen anhand des aktuellen Status der Telefonnummerverifizierung
		if(!data.mobile_phone) {
			$('.verify-widget').removeClass(verify_classes).addClass('no-phone');
		} else if(data.flag_phone_verified == '1') {
			$('.verify-widget').removeClass(verify_classes).addClass('phone-verified');
		} else if(data.has_verification_code == '1') {
			$('.verify-widget').removeClass(verify_classes).addClass('phone-awaiting-verification');
		} else {
			$('.verify-widget').removeClass(verify_classes).addClass('phone-unverified');
		}

		// Setze die Formularwerte
		$('#edit-profile-initial').formParams(data);

	};

	/**
	 * Speichert die Formulardaten
	 * @param e
	 * @returns {*}
	 */
	function saveForm(e) {

		e.preventDefault();

		var $form = el.find('#edit-profile-initial'),
			params = $form.formParams(),
			data,
			errors = {};

		// Entferne Fehlermeldungen
		$form.off('focus.form_helper');
		el.find('.error').remove();
		el.find('.is-error').removeClass('is-error');

		// Errorcheck
		if(!params.salutation) {
			errors['#salutation'] = 'Bitte wähle Deine Anrede!';
		}
		if(!params.firstname) {
			errors['#firstname'] = 'Bitte gib Deinen Vornamen ein!';
		}
		if(!params.lastname) {
			errors['#lastname'] = 'Bitte gib Deinen Nachnamen ein!';
		}
		if(!params.username) {
			errors['#username'] = 'Bitte gib Deinen Benutzernamen ein!';
		}

		// Zeige Fehlermeldungen
		if(!_.isEmpty(errors)) {
			$form.on('focus.form_helper', '.is-error', function() {
				$(this).removeClass('is-error');
			});
	
			$form.parents('.container')[0].scrollTop = 0;
			return showError(errors, $form);
		}

		// Erstelle FormData Objekt
		data = new FormData();

		// API Parameter
		data.append('action', 'editProfileInitial');
		data.append('token', User.getToken());

		// Übertrage die Formulardaten in das FormData Objekt
		for(var key in params) {
			data.append(key, params[key]);
		}

		// Übertrage das Profilbild, wenn die Bildquelle eine data-URI ist (also ein neues Bild ausgewählt wurde)
		var img_src = $('#user-profile-picture').attr('src');
		if(img_src.indexOf('data') === 0) {
			data.append('picture', img_src);
		}

		// Internet?
		if(!isConnected()) {
			return displayNoConnectionError();
		}

		// Zeige Loadingspinner
		UI.toggleLoading();

		// API Call
		$.ajax({
			url: JET.backendURL,
			data: data,
			type: 'POST',
			processData: false,
			contentType: false,
			cache: false
		}).done(function(data) {
			
			if(data.status == 'success') {

				// Aktualisiere User
				User.setData(data.user);

				// Zeige Erfolgsmeldung
				showSuccess('Die Änderungen wurden gespeichert.', $form);

				// Schliesse Modal und zeige Preisalarm
				UI.modalClose().done(function() {
					UI.show('Preisalarm');
				});

			} else {
				// Zeige Fehlermeldung
				showError(data.msg, $form);
			}


		}).fail(function(jqXHR, textStatus, errorThrown) {

			// Internet?
			if(isOfflineError(jqXHR)) {
				return displayNoConnectionError();
			}

			// Zeige Fehlermeldung
			UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />Die Daten konnten nicht gespeichert werden.');

		}).always(function() {

			// Scroll to top
			$form.parents('.container')[0].scrollTop = 0;

			// Verberge Loadingspinner
			setTimeout(UI.toggleLoading, 100);
		});

	};

	return config;

});