JET.controller('Tankstellensuche', function($el) {

	// Controller API
	var config = {
		title: 'JET Tankstellenfinder',
		template: 'Tankstellensuche.html',

		// Wird zuerst aufgerufen, auch wenn Temlate noch nicht geladen
		viewWillAppear: onViewWillAppear,

		// Wird aufgerufen, wenn View angezeigt wird
		bindEvents: onBindEvents,

		// Wird aufgerufen, wenn Template geladen
		templateLoaded: loadMap,

		// Nach Modal
		refresh: onRefresh,

		// Wenn View nicht mehr angezeigt wird
		unbindEvents: onUnbindEvents
	};
	
	// Hilfsvariable, um den Ladestatus des Templates zu prüfen
	var isTemplateLoaded = false;

	// Map Variablen
	var map,
		mapIsInitialised = false,
		initialMapZoom = 5,
		// Mitte von Deutschland
		initialMapLocation = {
			lat: 51.41142798796079,
			lng: 10.429687500000027
		},
		stationMarkers = {},
		stationDetailZoom = 15;

	// StatusVariablen
		// Aktuelle LatLng Position der Map
	var currentMapPosition = null,
		// Aktueller Zoomlevel der Map
		currentMapZoom = initialMapZoom,
		// Keine TS-Suche, wenn Zoom < minimumZoomLevelForStationSearch
		minimumZoomLevelForStationSearch = 7,
		// Zeigen wir gerade eine TS-Detailseite an?
		isInDetailView = false,
		// Ist true, wenn wir aus einem TS-Detailview kommen und es eine currentMapPosition und einen currentMapZoom gibt.
		returnsFromKnownLocationAndZoom = false,
		// Falls eine TS-Nummer übergeben wurde, wird diese hier gespeichert und beim Aufruf von initView wird direkt die Detailseite aufgerufen.
		displayStationOnLoad = null,
		// Sollen wir die Position des Users abfragen
		getUserPositionOnLoad = false,


		// Wenn dieser Parameter true ist und wir TS gefunden haben, wird der Kartenausschnitt so angepasst, dass die nächstgelegene
		// TS und der Usermarker auf jeden Falls sichtbar sind.
		searchNearestStation = false,

		// Falls der Geocoder mehrere Ergebnisse liefert, speichern wir diese hier zwischen
		multipleResults;

		// setTimeoutIDs
	var zoomAndPanTimeout = null;
	var priceMarkerPanTimeout = null;

		// google.maps.Marker
	var priceMarker = null;
		// Falls ein JS Object übergeben wurde, wird dieses hier gespeichert.
	var detailStation = {
		station_number: null,
		address: null
	};
	// Hier wird die owlCarousel Instanz der KLICKe-Mitglieder gespeichert.
	var swipeView;

	// Wenn wir beim Aufruf des Controllers in einem Detailview sind und dieser Parameter ist true,
	// wird die Listenansicht geladen.
	var forceHideDetail = false;

	/**
	 * Der Controller soll angezeigt werden
	 */
	function onViewWillAppear(args) {

		if(typeof args === 'undefined') {

			//console.log('TS onViewWillAppear - no args');

			if(currentMapPosition === null) {
				// Wir haben keine Position, also holen wir uns eine.
				getUserPositionOnLoad = true;

			}

		} else if(args == 'locate') {

			// Bitte finden Sie meine aktuelle Position!

			// Falls in Detailview, beenden!
			forceHideDetail = true;

			// Hole Geolocation
			getUserPositionOnLoad = true;

			// Reset der Variablen
			displayStationOnLoad = null;

		} else if(args === 'locating') {

			// Controller wurde durch Klick auf den Button "Locate" in der TS-Suche aufgerufen,
			// eine Geolocation-Abfrage läuft bereits.
			//console.log('TS onViewWillAppear - Locating');

			// Falls in Detailview, beenden!			
			forceHideDetail = true;

			// Wir fragen bereits nach der Geolocation!
			getUserPositionOnLoad = false;

			// Reset der Variablen
			displayStationOnLoad = null;

		} else if(args === 'searching') {

			// Controller wurde durch Klick auf den Button "Suche" in der TS-Suche aufgerufen,
			// eine Geocoding-Abfrage läuft bereits.
			//console.log('TS onViewWillAppear - Searching');

			// Falls in Detailview, beenden!			
			forceHideDetail = true;

			// Wir fragen bereits nach einer Geoposition!
			getUserPositionOnLoad = false;

			// Reset der Variablen
			displayStationOnLoad = null;

		} else if(_.isString(args)) {

			// Zeige eine Tankstelle 
			// Ausgelöst beispielsweise durch Klick auf eine TS im Bereich Preisalarm
			//console.log('TS onViewWillAppear - StationNumber ', args);

			// Welche TS (Stationsnummer)
			displayStationOnLoad = args;

			// Keine Geolocationabfrage
			getUserPositionOnLoad = false;

			// Unbekannte Geoposition
			currentMapPosition = null;

		} else if(_.isObject(args)) {

			// Zeige eine Tankstelle, ausgehend von einer Geoposition
			// Wird durch das Radar ausgelöst.
			//console.log('TS onViewWillAppear - StationObject', args);

			// Setze aktuelle Position
			setCurrentMapPosition(args.lat, args.lng);

			// Keine Geolocationabfrage, bitte.
			getUserPositionOnLoad = false;

			// Zeige diese Station (Stationsnummer)
			displayStationOnLoad = args.station_number;

		}

		// Falls wir bereits versucht haben, die Maps-Api zu laden, aber noch kein Ergebnis haben,
		// versuchen wir es eben noch einmal (Fingers crossed)
		if(mapTriedToLoad && (typeof google === 'undefined' || typeof google.maps === 'undefined')) {
			loadMap();
		}

	};

	// Haben wir schon versucht, die Map zu laden?
	var mapTriedToLoad = false;
	function loadMap() {

		// Jupp, jetzt gerade...
		mapTriedToLoad = true;

		// Lade die JSAPI asynchron
		$.ajax({
			dataType: 'script',
			cache: true,
			crossDomain: true,
			url: 'https://www.google.com/jsapi'
		}).done(function(script, textStatus, jqXHR) {

			// Lade die Maps-Api asynchron
			google.load('maps', '3', {other_params: 'key=AIzaSyC77GxSiGbRUCYBGwetB71asdpHKx67Wgo', callback: function() {
				initMap();
			}});

		}).fail(function(jqXHR) {

			// Oh no!

			if(!isConnected()) {
				UI.alert('<b>Derzeit keine Internetverbindung!</b><br />Die Karte kann nicht geladen werden.');
			} else {
				UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />Die Karte kann nicht geladen werden.');	
			}
			
			// Verberge mögliche Statusmeldungen
			hideSearchStatus();

		});

	};

	/**
	 * Maps-API ist geladen, init!
	 */
	function initMap() {
		//console.log('TS onTemplateLoaded');

		isTemplateLoaded = true;

		// Verberge mögliche Statusmeldungen
		hideSearchStatus();


		// Google Maps API Options
		var mapOptions = {
			zoom: initialMapZoom,
			center: initialMapLocation, // Deutschland
			mapTypeId: google.maps.MapTypeId.ROADMAP,
			backgroundColor: '#ffffff',
			disableDefaultUI: true,
			zoomControl: true, 
			keyboardShortcuts: false,
			minZoom: initialMapZoom,
			styles: [
				{
					// Keine POIs auf der Karte, bitte (STAR Tankstellen, anyone?)
					featureType: 'poi',
					stylers: [
						{visibility:'off'}
					]
				}
			]
		};
		
		map = new google.maps.Map($('#map', $el)[0], mapOptions);

		// Sobald die Map fertig geladen und initialisiert ist, wird das "idle" Event geworfen.
		// Dann können wir weitermachen
		google.maps.event.addListenerOnce(map, 'idle', function() {
			isTemplateLoaded = true;
			bindMapListeners();
			onBindEvents();
		});


		// Debug
		window['map'] = map;
	};

	
	// Statusvariable
	var eventsAreBound = false;

	/**
	 * Wird aufgerufen, wenn der View bereit ist.
	 */
	function onBindEvents() {

		if(isTemplateLoaded === false) {
			return;
		} else if(eventsAreBound === true) {
			return initView();
		} 


		//console.log('TS onBindEvents', eventsAreBound);
		eventsAreBound = true;

		bindGeocoderListeners();
		bindGeolocationListeners();
		
		bindUIListeners();

		// Aktiviere Backbutton Handling
		JET.on('UI.back.click.ts', function(e) {
			hideStationDetail();
		});

		initView();
		
	};


	/**
	 * Wird aufgerufen, wenn der User einen neuen Preisalarm gesetzt hat.
	 */
	function onRefresh() {
		//console.log('TS onRefresh');

		if(detailStation.station_number !== null) {

			// Aktualisiere das Userinterface		

			var starButton = $el.find('li[data-event="ts-star"]');
			var alarmButton = $el.find('li[data-event="ts-alarm"]');

			var userStation = _.find(User.getData('stations'), function(station) { 
				return station.station_id == detailStation.station_number; 
			});

			if(userStation) {
				starButton.addClass('is-active');

				if(userStation.has_alarm) {
					alarmButton.addClass('is-active');
				} else {
					alarmButton.removeClass('is-active');
				}
			} else {
				starButton.removeClass('is-active');
				alarmButton.removeClass('is-active');
			}

		}

	};

	/**
	 * Wird aufgerufen, wenn ein anderer Controller angezeigt werden soll.
	 */
	function onUnbindEvents() {
		//console.log('TS UnbindEvents');

		// Deregistriere alle Eventhandler, die in bindEvents registriert wurden
		JET.off('.ts');
		$el.off('.ts');

		// Falls noch Timeouts laufen, beende diese
		clearTimeout(zoomAndPanTimeout);
		clearTimeout(priceMarkerPanTimeout);
		eventsAreBound = false;
	};

	/**
	 * Initialisiere den View
	 */
	function initView() {

		//console.log('TS initView');

		if(forceHideDetail) {

			// Beende Detailansicht

			if(isInDetailView) {
				hideStationDetail(true);
			}
			forceHideDetail = false;
			UI.handle('hide.back');

		} else if($el.hasClass('is-detail') && !isInDetailView) {

			// Falls wir in einem Detailview sind, stelle sicher, das der Back-Button auch angezeigt wird.
			JET.one('UI.idle', function() {
				UI.handle('show.back');
			});

		} else if($el.hasClass('is-detail') && isInDetailView) {

			// Falls wir in einem Detailview sind, stelle sicher, das der Back-Button auch angezeigt wird.
			UI.handle('show.back');
		}

		// Soll eine TS angezeigt werden?
		if(displayStationOnLoad !== null) {
			
			setTimeout(function() {

				showStationDetail(displayStationOnLoad);

				if(currentMapPosition) {
					getUserMarker().setPosition(currentMapPosition);
				}

				displayStationOnLoad = null;

			}, 100);

			return;

		} else if(getUserPositionOnLoad === true) {

			// Hole Userposition
			LocationService.locate();
			getUserPositionOnLoad = false;

		} 

		// Trigger resize Event der Google Map, um einen Darstellungsfehler unter Android zu umgehen.
		// Manchmal wurde die Map nicht korrekt angezeigt. Dieser "Trick" hilft.
		google.maps.event.trigger(map, 'resize');

		// Startet die Geolocation bzw Geocoding Abfrage (falls angefragt)
		JET.trigger('tssuche.isReady');

	};

	/**
	 * Registriert alle Eventlistener des google.maps Objects
	 */
	function bindMapListeners() {

		// Immer wenn der Kartenausschnitt geändert wird (debounced, damit wir nicht überflutet werden).
		google.maps.event.addListener(map, 'bounds_changed', _.debounce(function() {

			//console.log('BoundsChanged');

			var bounds = map.getBounds(),
				stations,
				center,
				nearestStations;

			// Siehe LocationService.js
			JET.trigger('map.bounds', bounds);

			if(isInDetailView) {
				
				// Wenn wir in einem Detailview sind, passiert nichts.

				if(returnsFromKnownLocationAndZoom) {
					returnsFromKnownLocationAndZoom = false;
					isInDetailView = false;
				}

				return;

			} else if(map.getZoom() < minimumZoomLevelForStationSearch) {

				// Zoomlevel zu klein
				return;
			}

			// Suche Stationen, die auf der Karte enthalten sind.

			stations = Stations.getStationsInBounds(bounds);

			// Keine Stationen gefunden?
			if(stations.length === 0 && searchNearestStation === true) {
				
				// suche nächstgelegene station im Umkreis von 400 km

				center = map.getCenter();
				nearestStations = Stations.findNearestStation(center.lat(), center.lng(), 400);

				if(nearestStations.length > 0) {

					searchNearestStation = false;

					// Zeige TS und Kartenmittelpunkt (Userlocation)
					bounds.extend(new google.maps.LatLng(nearestStations[0].lat, nearestStations[0].lng));
					map.fitBounds(bounds);

					return;

				}

			}


			searchNearestStation = false;

			// Zeige Stationsliste
			showStationList(stations);

		}, 500));

		// Speichere aktuellen Kartenmittelpunkt, um nach Detailview wieder auf die letzte Position zu gelangen
		google.maps.event.addListener(map, 'center_changed', function() {

			if(isInDetailView) {
				// ausser wir zeigen eine Tankstellen-Detailseite an
				return false; 
			}
			console.log('CenterChanged - setting center');

			var center = map.getCenter();
			setCurrentMapPosition(center.lat(), center.lng());

		});

		// Speichere aktuelle Zoomstufe, um nach Detailview wieder auf die letzte Zoomstufe zu gelangen
		google.maps.event.addListener(map, 'zoom_changed', function() {

			// ausser wir zeigen eine Tankstellen-Detailseite an
			if(isInDetailView) {
				return;
			} else if(returnsFromKnownLocationAndZoom && $el.find('.ts-results > li').length > 0) {

				isInDetailView = true;
				
				return;

			}

			currentMapZoom = map.getZoom();
			if(currentMapZoom < minimumZoomLevelForStationSearch) {

				// Zoomlevel zu klein

				// Zeige Warnhinweis
				config.models.tsList({zoomwarning: true});

				// Entferne Marker
				updateStationMarkers([]);

			}

		});

	};

	// Geocoder Events
	function bindGeocoderListeners() {

		// Geocoding Request wurde gestartet
		JET.on('geocoder.start.ts', function(e, search) {

			// Zeige Hinweistext
			var escaped = $('<div />').html(search).text();
			showSearchStatus('<b>'+escaped+'</b> wird gesucht. Einen Moment bitte&hellip;');

		});

		// Geocoder Request hat Ergebnisse
		JET.on('geocoder.result.ts', function(e, result) {
			
			var _location = result.location;

			// verberge Hinweis
			hideSearchStatus();

			// Erlaube die nächstgelegene Station zu suchen, falls keine Ergebnisse im Viewport sind.
			searchNearestStation = true;

			// Setze Viewport
			map.fitBounds(result.viewport);
			map.panTo(_location);

			// Zeige Usermarker an gefundener Position
			showUserMarker(_location.lat(), _location.lng());

		});

		// Geocoding Request hat mehrere Ergebnisse gefunden
		JET.on('geocoder.multiple_results.ts', function(e, results) {

			// verberge Hinweis
			hideSearchStatus();

			// speichere Result für später
			multipleResults = results;

			// aktualisiere View
			config.models.tsList({locations:results});

		});

		// Geocoder hat Fehler
		JET.on('geocoder.error.ts', function(e, msg) {

			// verberge Hinweis
			hideSearchStatus();

			// Zeige Fehler
			UI.alert(msg);

		});

		// Binde Eventlistener
		$el.on('click.ts', '.ts-locations', function(e) {

			// Die Ergebnisse des Geocoders waren nicht eindeutig,
			// es wird eine Liste der "Meinten Sie..." Orte angezeigt.
			// Klickt der User auf einen dieser Orte, wird diese Auswahl übernommen

			var $list = $(this), 
				$item = $(e.target),
				result = multipleResults[$item.index()];

			// Erlaube Suche der nächstgelegenen Tankstelle, falls keine Ergebnisse in den aktuellen Viewbounds vorhanden sind.
			searchNearestStation = true;

			// Setzte Mapbounds
			map.fitBounds(result.viewport);

			// Zeige Usermarker
			showUserMarker(result.location.lat(), result.location.lng());

			// Bewege Karte, um bounds_changed Event auszulösen
			map.panBy(0,1);

			// Leere die Liste
			config.models.tsList();

			// resette die Variable
			multipleResults = null;

		});

	};

	// Geolocation Events
	function bindGeolocationListeners() {

		// Geolocation Request ist gestartet
		JET.on('geolocation.start.ts', function() {
			//console.log('geolocation.start.ts');

			// Zeige Hinweis
			showSearchStatus('Deine Position wird gesucht. Einen Moment bitte&hellip;');
		});

		// Geolocationabfrage war erfolgreich
		JET.on('geolocation.success.ts', function(e, coords) {
			//console.log('geolocation.success.ts', coords);

			// Erlaube Suche der nächstgelegenen Tankstelle, falls keine Ergebnisse in den aktuellen Viewbounds vorhanden sind.
			searchNearestStation = true;

			// Zeige Usermarker an gefundener Position
			showUserMarker(coords.latitude, coords.longitude);

			// Setze den Kartenausschnitt so, das ein Umkreis von 2km um die gefundene Position dargestellt wird.
			var ll = new google.maps.LatLng(coords.latitude, coords.longitude);

			// Hierzu erzeugen wir ein google.maps.Circle Object
			var _circle = getCircle();
			// setzten den Centerpunkt
			_circle.setCenter(ll);
			// den Radius
			_circle.setRadius(2000);

			// und setzen die Mapbounds so, das dieser Circle angezeigt werden kann.
			// Der Circle selber wird aber nicht auf der Karte gezeigt.
			map.fitBounds(_circle.getBounds());

			// verberge Suchhinweis
			hideSearchStatus();

		});

		// Geolocation Abfrage hat einen Fehler gemeldet
		JET.on('geolocation.error.ts', function(e, msg, error) {
			//console.log('geolocation.error.ts', msg, error);

			// verberge Hinweis
			hideSearchStatus();

			if(error.code == 3 || error.code == 2) {

				// Fehler ist entweder POSITION_UNAVAILABLE oder TIMEOUT
				// Zeige Fehlermeldung und erlaube es dem User die Abfrage erneut zu starten.

				UI.alert({
					body: msg,
					okbutton:'Nochmal versuchen',
					cancelbutton:'Abbrechen'
				}, function() {

					LocationService.locate();

				});

			} else {

				// Fehler ist PERMISSION_DENIED
				// Zeige Meldung
				UI.alert(msg);
			}

		});

	};

	
	// Userinterface Events
	function bindUIListeners() {

		// Finde mich bzw. bringe Station wieder in Focus Button
		$el.on('click.ts', '.relocateStation', function(e) {

			e.preventDefault();

			clearTimeout(zoomAndPanTimeout);
			zoomAndPanTimeout = null;

			if(isInDetailView && priceMarker.getMap() !== null) {
				// Wenn wir eine Tankstellendetailseite anzeigen
				// bringe den Preismast wieder in den View
				locatePriceMarker();

			} else {

				// Versuche den Usermarker zu lokalisieren
				var userPosition = getUserMarker().getPosition();

				// dies schlägt fehl, wenn der Usermarker noch nicht auf der Karte ist
				if(userPosition) {
					// falls die Position gefunden wurde, setze Kartenmittelpunkt dorthin
					map.panTo(userPosition);
				}

				// Starte Geolocationabfrage
				LocationService.locate();
			}

			// entferne Focus vom Button
			this.blur();

		});

		// ------------------------------------------------------------------------------------------

		// Suchergebnisliste
		$el.on('click.ts', '.ts-results li', function() {

			var $li = $(this),
				station_number = $li.data('station');

			// Zeige ausgewählte Tankstelle
			showStationDetail(station_number);

		});

		// ------------------------------------------------------------------------------------------

		// Tankstellenfilter
		// Die dargestellten Tankstellen können nach einigen Features gefiltert werden.
		// Die Reihenfolge der Eventlistener ist relevant!

		// TS Features (Filter) auf/zu
		$el.on('click.ts', 'li[data-event="ts-suche-features"]', function(e) {
			
			$(this).toggleClass('is-active');


		});

		// TS Features (Filter) auswählen
		// Bei einem Click auf eine Checkbox, "bubbelt" das Event nach oben.
		// Wenn wir das hier nicht anhalten, wird die Box wieder geschlossen, da der oben definierte Listener
		// greifen würde.
		// Das ist aber unschön, da man dann immer nur ein Feature zur Zeit auswählen kann.
		$el.on('click.ts', 'li[data-event="ts-suche-features"] form', function(e) {
			// Dieses Event anhalten, damit die Featurebox nicht geschlossen wird.
			e.stopPropagation();

		});

		// TS Features (Filter) auswählen
		$el.on('change.ts', 'li[data-event="ts-suche-features"] input', function(e) {

			var form = this.form, 
				inputs = form.elements,
				hasFeatures = false,
				stations = [],
				feature_name = this.id,
				feature_is_active = this.checked;

			// Setze Filter
			Stations.setFeature(feature_name, feature_is_active);

			if(feature_is_active === true) {

				hasFeatures = true;

			} else {

				for(var i=0, j=inputs.length; i<j; ++i) {
					if(inputs[i].checked) {
						hasFeatures = true;
						break;
					}
				}

			}

			// Zeige Status (Icon ist gelb oder weiss)
			if(hasFeatures) {
				$('li[data-event="ts-suche-features"]').addClass('has-features');
			} else {
				$('li[data-event="ts-suche-features"]').removeClass('has-features');
			}

			// Kartenauschnitt zu gross
			if(map.getZoom() < minimumZoomLevelForStationSearch) {

				return;
				
			}

			stations = Stations.getStationsInBounds(map.getBounds());

			showStationList(stations);


		});


		// ------------------------------------------------------------------------------------------

		// "Da will ich hin" auf/zu
		$el.on('click.ts', 'li[data-event="ts-navi"]', function(e) {
			
			$(this).toggleClass('is-active');

		});
			
		// Stammtankstelle setzen
		$el.on('click.ts', 'li[data-event="ts-star"]', function(e) {

			var alertMessage;

			if(!User.isLoggedIn()) {

				// Das geht nur bei eingeloggten Usern
				alertMessage = '<b>JET Stammtankstelle speichern</b><br />';
				alertMessage += 'Für diese Funktion musst Du dich zunächst anmelden oder registrieren.';

				UI.alert(alertMessage);
				return;
			}

			// Finde Stammtankstellen
			var stammTS = User.getData('stations'),
				$li = $(this);

			if($li.hasClass('is-active')) {

				// Tankstelle ist bereits Stammtankstelle

				// Confirm delete
				alertMessage = '<b>Dies ist eine Deiner Stammtankstellen</b><br />';
				alertMessage += 'Möchtest Du diese Tankstelle nicht mehr als Stammtankstelle gespeichert haben?';

				UI.alert(alertMessage, function() {

					if(!isConnected()) {
						return displayNoConnectionError();
					}

					// API Call
					$.post(JET.backendURL, {
						action: 'removeStation',
						station_number: detailStation.station_number,
						token: User.getToken()
					}).done(function(data) {

						User.setData(data.user);
						$li.removeClass('is-active');
						$el.find('li[data-event="ts-alarm"]').removeClass('is-active');

					}).fail(function(jqXHR) {

						if(isOfflineError(jqXHR)) {
							return displayNoConnectionError();
						}
						
						UI.alert('<b>Ein Fehler ist aufgetreten!</b>');


					});

				}, true);

			} else if(stammTS && stammTS.length == 2) {

				// User hat bereits 2 Stammstankstellen (mehr geht nicht)
				// Zeige Dialog zur Auswahl der Tankstelle, die ersetzt werden soll.
				stationChooser(stammTS);
				
			} else {

				// Setze Stammtankstelle
				setStation(detailStation.station_number);
			}

		});

		// Richte Preisalarm für diese TS ein
		$el.on('click.ts', 'li[data-event="ts-alarm"]', function(e) {

			if(!User.isLoggedIn()) {
				// No login, no alarm
				UI.alert('<b>JET Preisalarm</b><br />Für diese Funktion musst Du dich zunächst anmelden oder registrieren.');
				return;
			}

			// Ein Preisalarm kann nur für eine der Stammtankstellen gesetzt werden.
			// Prüfe, ob dem so ist.
			var userStations = User.getData('stations'),
				isUserStation = false;

			if(userStations) {
				for(var i=0, j=userStations.length; i<j; ++i) {
					if(userStations[i].station_id == detailStation.station_number) {
						isUserStation = true;
					}
				}
			}

			if(isUserStation) {

				// alles gut, dies ist eine der Stammtankstellen des Users
				// Zeige Preisalarmeingabemaske
				UI.modal('PreisalarmEdit', detailStation.station_number);

			} else if(userStations && userStations.length == 2) {

				// Dies ist keine der Stammtankstellen des Users,
				// aber er hat schon zwei.
				// Also muss er erst eine TS ersetzen
				$.when(stationChooser(userStations, detailStation.station_number)).then(function() {
					// Zeige Preisalarmeingabemaske
					UI.modal('PreisalarmEdit', detailStation.station_number);

				});

			} else {
				// Dies ist keine der Stammtankstellen des Users,
				// aber er hat erst eine.
				// Setze diese TS als zweite Stamm-TS
				setStation(detailStation.station_number).done(function() {
					// Zeige Preisalarmeingabemaske
					UI.modal('PreisalarmEdit', detailStation.station_number);					
				});

			}

		});

		// Klick auf ein Profilbild der KLICKe-Mitglieder auf der TS-Detailseite
		$el.on('click.ts', '.ts-detail-klicke-item', function(e) {

			// Prüfe, ob dies ein "Freund" des Users ist
			var user_id = this.getAttribute('data-userid'),
				is_friend = _.find(User.getData('friends'), function(friend) {

					return friend.friend_id == user_id;

				}); 

			// Zeige Userdetailseite als Modal
			UI.modal('KlickeUserDetail', {
				user_id: user_id,
				is_friend: is_friend
			});

		});

	};

	// Zeigt den "Usermarker" an der angegebenen Position
	// und setzt die aktuelle Kartenposition
	function showUserMarker(lat, lng) {

		setCurrentMapPosition(lat, lng);

		getUserMarker().setPosition({
			lat:lat,
			lng:lng
		});

	};

	// Zeige die Liste der gefundenen Tankstellen
	function showStationList(stations) {

		// Keine Stationen?
		if(stations.length === 0) {

			var data = {};

			if($el.find('li[data-event="ts-suche-features"]').hasClass('has-features')) {
				data.msg = 'Sie haben Filter gesetzt. Eventuell werden mehr JET Tankstellen gefunden, wenn Sie den Filter deaktivieren.';
			}

			config.models.tsList(data);

			return;
		}

		// Entfernung zum Usermarker messen
		// Stationen nach Entfernung sortieren
		// Marker erstellen
		updateStationMarkers(stations);

		// Liste anzeigen
		config.models.tsList({stations: stations.sort(stationSortCallback)});

	};

	// Zeigt die Marker aller gefundenen Stationen auf der Map an.
	// Ist ein Marker bereits vorhanden, wird dieser aktualisiert,
	// andernfalls wird der Marker neu erstellt.
	// Vorhandene Marker, die nicht mehr in der Liste enthalten sind,
	// werden entfernt.
	function updateStationMarkers(stations) {

		var station_id,
			// die Marker, die auf der Karte angezeigt werden sollen
			newMarkers = [],
			userMarker = getUserMarker(),

			// Berechne die Entfernung jeder Tankstelle zu diesem Punkt.
			userPosition = userMarker.getPosition();

		if(!userPosition) {
			userPosition = map.getCenter();
		}

		for(var i=0, station; station = stations[i]; ++i) {

			station_id = station.station_number;

			if(stationMarkers[station_id]) {
				// diese Tankstelle (bzw. der Marker) ist bereits auf der Karte vorhanden
				newMarkers[station_id] = stationMarkers[station_id];

			} else {
				// Marker noch nicht vorhanden, neu erstellen
				newMarkers[station_id] = createStationMarker(station);
				
			}

			// Berechne Distanz der Tankstelle zum gegebenen Punkt
			station.distance = calculateDistance(newMarkers[station_id].getPosition(), userPosition);

		}

		// Prüfe, welche Marker von der Karte entfernt werden können
		for(station_id in stationMarkers) {

			if(!newMarkers[station_id]) {
				
				google.maps.event.clearInstanceListeners(stationMarkers[station_id]);
				stationMarkers[station_id].setMap(null);
				
			}
		}

		// speichere neues MarkerArray
		stationMarkers = newMarkers;

	};

	// Erstellt einen Marker und "bindet" das Clickevent zur Auswahl der Tankstelle
	function createStationMarker(aStation) {

		var _marker = new google.maps.Marker({
			map: map,
			position: new google.maps.LatLng(aStation.lat, aStation.lng),
			//icon: 'css/chrome/marker_jet_small.png'
			icon: {
				url: 'css/chrome/JET_TS_marker.png',
				size: new google.maps.Size(79, 79),
				scaledSize: new google.maps.Size(40, 40),
				anchor: new google.maps.Point(20, 34)
			}
		});

		addStationMarkerEvent(_marker, aStation.station_number);

		return _marker;

	};

	// Erstellt den Eventlistener für das Clickevent zur Auswahl der Tankstelle
	function addStationMarkerEvent(aMarker, aStation) {

		google.maps.event.addListener(aMarker, 'click', function() {

			showStationDetail(aStation);

		});

	}

	// Zeigt einen Hinweistext auf der Karte an.
	// Wird genutzt, um dem User anzuzeigen, das eine Suche durchgeführt wird (Geolocation oder Geoposition)
	function showSearchStatus(msg, speed) {

		if(typeof msg === 'undefined') {
			msg = 'Einen Moment bitte&hellip;';
		}

		if(typeof speed === 'undefined') {
			speed = 'normal';
		}

		$el.find('.search-status')
		   .html(msg)
		   .fadeIn(speed);

	};

	// Verbirgt den Hinweistext
	function hideSearchStatus(speed) {

		if(typeof speed === 'undefined') {
			speed = 'fast';
		}

		$el.find('.search-status').fadeOut(speed);
	};

	/**
	 * Callbackfunktion zur Sortierung der Tankstellen nach Entfernung
	 * @param object a Tankstelle 1
	 * @param object b Tankstelle 2
	 * @returns {number}
	 */
	function stationSortCallback(a, b) {
		return (a.distance < b.distance ? -1 : (a.distance > b.distance ? 1 : 0));
	};

	/**
	 * * Berechnet die Entfernung zwischen 2 Punkten in km
	 * http://www.movable-type.co.uk/scripts/latlong.html#cosine-law
	 * @param LatLng ll1
	 * @param LatLng ll2
	 * @returns {number}
	 */
	function calculateDistance(ll1, ll2) {
		var R = 6371, // Earth Radius in KM
			sin = Math.sin,
			cos = Math.cos,
			pi = Math.PI / 180,
			lat1 = ll1.lat() * pi,
			lon1 = ll1.lng() * pi,
			lat2 = ll2.lat() * pi,
			lon2 = ll2.lng() * pi;

		return Math.acos(sin(lat1)*sin(lat2) + cos(lat1)*cos(lat2) * cos(lon2-lon1)) * R;
	}

	/**
	 * Zeigt eine Tankstellendetailseite an
	 * @param string station_number
	 */
	function showStationDetail(station_number) {

		// Beende Timeouts (falls vorhanden)
		if(zoomAndPanTimeout !== null) {
			clearTimeout(zoomAndPanTimeout);
			zoomAndPanTimeout = null;
		}

		if(priceMarkerPanTimeout !== null) {
			clearTimeout(priceMarkerPanTimeout);
			priceMarkerPanTimeout = null;
		}

		// Hole Detailinformationen
		var station = Stations.getStation(station_number);

		if(typeof station['lat'] === 'undefined') {
			return;
		}

		// Zeige Marker, falls noch nicht auf der Karte
		if(typeof stationMarkers[station_number] === 'undefined') {
			stationMarkers[station_number] = createStationMarker(station);
		} else if(stationMarkers[station_number].getMap() === null) {
			stationMarkers[station_number].setMap(map);
		}

		// Zeige Detailview (siehe /tpl/Tankstellensuche.html)
		$el.find('.detail-view')[0].scrollTop=0;
		$el.addClass('is-detail');

		// Setze Informationen für das detailStation Objekt
		detailStation.address = station.zip + ' '+station.city;
		detailStation.station_number = station_number;

		// Setze Status
		isInDetailView = true;

		// Aktualisiere UI
		UI.setTitle('JET Tankstelle');
		UI.handle('show.back');

		// Zeige Tankstellenposition als Kartenmittelpunkt
		map.panTo(station);
		// Setze den Zoomlevel
		// Dies ist in setTimeout gewrappt, damit das flüssig läuft.
		zoomAndPanTimeout = setTimeout(function() {

			map.setZoom(stationDetailZoom);

			// Lade den Preismast
			loadPriceMarker(station);
            //updateStationMarkers(station); //XXX

			zoomAndPanTimeout = null;

		}, 400);

		// Aktualisiere View mit den Stationsdaten
		config.models.tsDetail({station:station});

		// Aktualisiere die Navigationslinks
		$('#ts-navi-options a', $el).each(function(index, el) {

			var dataHref = el.getAttribute('data-href');

			if(dataHref.indexOf('comgooglemaps') === 0) {
				// Navigiere mit Google Maps

				if(!JET.googleMaps) {
					// Google Maps nicht vorhanden
					$(el).hide();
				} else if(device.platform == 'Android' || device.platform == 'android') {
					// Auf Android müssen wir die Google Maps Anwendung über ein Plugin starten

					var locationString = station.street+', '+station.zip+' '+station.city;

					$el.on('click', 'a[data-href^="comgooglemaps"]', function(e) {
						e.preventDefault();
						navigator.google_navigate.navigate(locationString);
					});

				} else {
					// Unter iOS können wir einfach einen Link öffnen
					el.href = _.template(dataHref, station);
				}
			} else if(dataHref.indexOf('tomtom') === 0) {
				// Navigiere mit TomTom (iOS only)
				if(!JET.tomtom) {
					$(el).hide();
				} else {
					el.href = _.template(dataHref, station);
				}
			} else if(dataHref.indexOf('navigon') === 0) {
				// Navigiere mit Navigon (iOS only)
				if(!JET.navigon) {
					$(el).hide();
				} else {
					el.href = _.template(dataHref, station);
				}
			} else if(dataHref.indexOf('maps://') === 0) {
				// Navigiere mit Apple Maps (iOS only)
				if(!JET.appleMaps) {
					$(el).hide();
				} else {
					el.href = _.template(dataHref, station);
				}
			}

		});

		// Lösche eventuell vorhandene KLICKe-Userliste
		var klicke = $el.find('.ts-detail-klicke');
		swipeView && swipeView.destroy();
		klicke.empty();

		// Resette Icons für Stammtankstelle und Preisalarm
		$el.find('li[data-event="ts-star"]').removeClass('is-active');
		$el.find('li[data-event="ts-alarm"]').removeClass('is-active');

		// Nur wenn der User eingeloggt (und Online) ist, laden wir die KLICKe-Liste und können feststellen,
		// ob dies eine Stammtankstelle des User ist
		if(User.isLoggedIn() && isConnected()) {

			// Hole die Stammtankstellen des Users
			var stations = User.getData('stations') || [];

			for(var i=0; i<stations.length; ++i) {
				
				if(stations[i].station_id == station.station_number) {
					// Ist Stammtankstelle!
					$el.find('li[data-event="ts-star"]').addClass('is-active');

					if(stations[i].has_alarm) {
						// Hat aktiven Preisalarm!
						$el.find('li[data-event="ts-alarm"]').addClass('is-active');
					}

					break;
				}
			}

			// Lade KLICKe-User für diese Tankstelle
			$.get(JET.backendURL, {
				action: 'getKlicke',
				station_number: station.station_number,
				token: User.getToken()
			}).done(function(data) {

				// Erstelle KLICKe-Usercontainer
				var string = ['<div class="klicke-items"><p>Diese JET Tankstelle hat '+data.klicke.length+' Mitglieder.</p><div id="owl-wrapper" class="owl-carousel">'];

				// Erstelle KLICKe-Useritem
				_.each(data.klicke, function(member) {

					string.push('<div class="ts-detail-klicke-item" data-userid="'+member.id+'">');
					string.push('<img src="'+JET.baseURL+(member.picture && 'img/200x200/'+member.picture || JET.defaultProfilePicture)+'" />');
					string.push(member.username);

					string.push('</div>');

				});

				string.push('</div>');
				string.push('</div>');

				// Zeige KLICKe-Infos
				klicke.html(string.join(''));

				setTimeout(function() {
					// Erstelle Carousel
					swipeView = klicke.find('#owl-wrapper').owlCarousel({
						items:4,
						itemsDesktop: [1000, 4],
						itemsDesktopSmall: [900, 4],
						itemsTablet: [600, 4],
						itemsMobile: [400, 4],
						scrollPerPage: true,
						pagination: true
					}).data('owlCarousel');

				}, 0);

			});

		}
	};

	/**
	 * Lädt den Preismast für eine Tankstelle
	 * @param string station
	 */
	function loadPriceMarker(station) {

		// Der Marker wird gecacht
		if(priceMarker === null) {

			priceMarker = new google.maps.Marker({
				clickable: false,
				zIndex: 1000
			});

		}

		// Zeige Marker auf Karte
		priceMarker.setMap(map);
		priceMarker.setPosition(station);

		// Kein Internet?
		if(!isConnected()) {
			priceMarker.setIcon('css/chrome/price_marker_error.png');
			var message = 'Die Preise können nicht angezeigt werden, da aktuell keine ausreichende Internetverbindung besteht.';
			UI.alert('<b>Derzeit keine Internetverbindung!</b><br />' + message);
		} else {
			// Setze Markericon
			// Die URL ruft die API auf, welche den Marker als Grafik erstellt und zurückgibt.
			priceMarker.setIcon(JET.backendURL + '?action=getMarker&version=2&station='+station.station_number+'&cc='+(new Date()).getTime());
            //priceMarker.setIcon('css/chrome/price_marker.png');
            stationMarkers[station.station_number].setMap(null);
		}

		// Setze Kartenmittelpunkt, so dass der Marker komplett sichtbar ist.
		priceMarkerPanTimeout = setTimeout(function() {

			map.panBy(0, -124); // Magic Number = Iconhoehe / 2
			priceMarkerPanTimeout = null;

		}, 400);
	};

	/**
	 * Setze Kartenposition, so dass der Preismarker komplett sichtbar ist
	 */
	function locatePriceMarker() {

		map.panTo(priceMarker.getPosition());
		map.panBy(0, -124);
	};

	/**
	 * Beende Detailansicht einer Tankstelle und gehe zurück zur Liste
	 * @param bool doNotSearchPosition Soll die aktuelle Position des User gesucht werden? Wird hier irgendein Wert übergeben, wird die Position nicht gesucht.
	 */
	function hideStationDetail(doNotSearchPosition) {

		// Beende Timeouts
		if(zoomAndPanTimeout !== null) {
			clearTimeout(zoomAndPanTimeout);
			zoomAndPanTimeout = null;
		}

		if(priceMarkerPanTimeout !== null) {
			clearTimeout(priceMarkerPanTimeout);
			priceMarkerPanTimeout = null;
		}

		// Default setzen
		returnsFromKnownLocationAndZoom = false;

		// Status setzen
		isInDetailView = false;

		// Verberge Preismast
		priceMarker.setMap(null);
		stationMarkers[detailStation.station_number].setMap(map);

		if(typeof doNotSearchPosition === 'undefined') {

			if(currentMapPosition === null) {

				// Wir haben keine letzte Position gecacht und wissen daher nicht, welche Position wir anzeigen sollen.
				// Suche Adresse (PLZ und Ort) der Tankstelle, die wir gerade angezeigt haben. Dadurch können wir dem User
				// eine Karte zeigen, welche die gerade angesehene TS beinhaltet, aber den Kartenmittelpunkt nicht exakt
				// auf dieser Position hat (das würde doof aussehen)
				LocationService.search(detailStation.address);


			} else if(currentMapZoom === null || currentMapZoom <= initialMapZoom) {

				// Wir haben ein letzte Position, aber keine Zoomstufe (oder die Zoomstufe ist zu klein)
				// Also setzten wir den Kartenmittelpunkt und Zoomstufe so, dass ein Radius von 2km um den letzten Mittelpunkt angezeigt wird.
				var _circle = getCircle();
				_circle.setCenter(currentMapPosition);
				_circle.setRadius(2000);
				
				map.fitBounds(_circle.getBounds());
				

			} else {

				// Wir haben eine letzte Position und eine Zoomstufe!
				returnsFromKnownLocationAndZoom = true;

				// Setze Zoomlevel
				map.setZoom(currentMapZoom);

				// Warte einen Moment und setze Kartenmittelpunkt
				// Timeout, damit das flüssig läuft.
				zoomAndPanTimeout = setTimeout(function() {
					map.panTo(currentMapPosition);
					zoomAndPanTimeout = null;
				}, 400);
			}
		}

		// Aktualisiere UI
		UI.setTitle('JET Tankstellenfinder');
		UI.handle('hide.back');
		
		// Resette detailStation
		detailStation.address = null;

		// Beende Detailview
		$el.removeClass('is-detail');

	};

	/**
	 * Wird aufgerufen, wenn der User bereits 2 Stamm-TS hat und eine neue Tankstelle aufnehmen will.
	 * Es wird ein Dialog mit seinen aktuellen Stamm-TS angezeigt, in dem der User wählen kann, welche TS er ersetzen möchte.
	 *
	 * @param array stammTS
	 * @returns jQuery.promise
	 */
	function stationChooser(stammTS) {

		var s1, s2, msg, defered = $.Deferred();

		s1 = Stations.getStation(stammTS[0].station_id);
		s2 = Stations.getStation(stammTS[1].station_id);

		msg = '<b>Du hast bereits 2 Stammtankstellen.</b><br />';
		msg += 'Wenn Du diese Tankstelle als neue Stammtankstelle oder für einen Preisalarm anlegen möchtest, ';
		msg += 'wähle eine der beiden aus, die damit gelöscht und durch die neue Stammtankstelle ersetzt wird.<br />';
		msg += '<button class="change-station-button" data-station="'+s1.station_number+'">'+s1.street + '<br>'+s1.zip+' '+s1.city+'</button>';
		msg += '<br />';
		msg += '<button class="change-station-button" data-station="'+s2.station_number+'">'+s2.street + '<br>'+s2.zip+' '+s2.city+'</button>';

		// Clickhandler für die Auswahlbuttons
		$('#alert-body').one('click.ts', 'button', function(e) {
			

			var station = this.getAttribute('data-station');
			
			if(!isConnected()) {
				return displayNoConnectionError();
			}

			// Setzte Stammtankstelle
			setStation(detailStation.station_number, station).done(function() {

				defered.resolve();

				// Schließe Fenster
				$('#alert-ok').trigger('click');
			
			});


		});

		// Zeige Dialog
		// Der "OK"-Button hat hier die Funktion "Abbrechen".
		// In diesem Fall wird keine neue Stammtankstelle gespeichert.
		UI.alert({
			body:msg,
			okbutton:'Abbrechen'
		}, function() {
			// Lösche Eventhandler
			$('#alert-body').off('click.ts');
			defered.reject();
		}, false);

		return defered.promise();

	};

	/**
	 * Speichert eine neue Stammtankstelle und ersetzt eine bestehende
	 * @param string newStation Stationsnummer der neuen Tankstelle
	 * @param string oldStation Stationsnummer der zu ersetzenden Tankstelle
	 * @returns jqXHR
	 */
	function setStation(newStation, oldStation) {

		var args = {
			action: 'setStation',
			station_number: newStation,
			oldStation: oldStation,
			token: User.getToken()
		};
		

		return $.post(JET.backendURL, args).done(function(data) {


			User.setData(data.user);

			$el.find('li[data-event="ts-star"]').addClass('is-active');

		}).fail(function(jqXHR, textStatus, errorThrown) {

			if(isOfflineError(jqXHR)) {
				return displayNoConnectionError();
			}

			var message = 'Bitte versuche es erneut.';
			
			UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />'+message);

		});

	}

	/**
	 * Speichert die aktuelle Kartenposition
	 * @param lat
	 * @param lng
	 */
	function setCurrentMapPosition(lat, lng) {

		if(currentMapPosition === null) {
			currentMapPosition = {
				lat: 0,
				lng: 0
			};
		}

		currentMapPosition.lat = lat;
		currentMapPosition.lng = lng;

	};


	/** @var google.maps.Circle circle */
	var circle = null;

	/**
	 * Gibt ein google.maps.Circle Objekt zurück
	 * @returns google.maps.Circle
	 */
	function getCircle() {
		if(circle === null) {
			circle = new google.maps.Circle({
				visible:false,
				clickable:false,
				map:map
			});
		}

		return circle;
	};

	/** @var google.maps.Marker userMarker */
	var userMarker = null;

	/**
	 * Gibt ein google.maps.Marker Objekt zurück
	 * @returns google.maps.Marker
	 */
	function getUserMarker() {

		if(userMarker === null) {
			userMarker = new google.maps.Marker({
				map:map,
				clickable: false
			});
		}

		return userMarker;

	};

	return config;

});