JET.controller('Tankstellensuche', function($el) {

	var showDetail = null,
		isLocating = false;

	var config = {
		title: 'JET Tankstellenfinder',
		template: 'Tankstellensuche.html',
		templateLoaded: onTemplateLoaded,
		bindEvents: bindEvents,
		unbindEvents: unbindEvents,
		refresh: function() {

			if(detailStation) {

				var starButton = $el.find('li[data-event="ts-star"]');
				var alarmButton = $el.find('li[data-event="ts-alarm"]');

				var userStation = _.find(User.getData('stations'), function(station) { 
					return station.station_id == detailStation.station_number; 
				});

				if(userStation) {
					starButton.addClass('is-active');

					if(userStation.has_alarm) {
						alarmButton.addClass('is-active');
					} else {
						alarmButton.removeClass('is-active');
					}
				} else {
					starButton.removeClass('is-active');
					alarmButton.removeClass('is-active');
				}

			}

		},
		viewWillAppear: function(station_number) {

			if(typeof station_number === 'undefined') {
				return;
			}

			if(station_number === 'locating') {
				isLocating = true;
			} else if(station_number === 'searching') {
				isLocating = true;
			} else {
				showDetail = station_number;
			}
			
			
		},
		showStation: function(detailsObject) {

			// Das hier wird aufgerufen, wenn das Radar eine neue TS gemeldet hat
			// und die Seite schon da ist

			if(detailsObject === 'locating' || detailsObject === 'searching') {
				return;
			}

			showDetail = detailsObject;


			if(_.isString(detailsObject)) {

				var station_number = detailsObject,
				station = Stations.getStation(station_number);

				var _circle = getCircle();
				_circle.setRadius(5000);
				_circle.setCenter(station);
				mapFitBounds(_circle.getBounds());
				showUserMarker(_circle.getCenter());

			} else {
			

				var ll = new google.maps.LatLng(detailsObject.lat, detailsObject.lng),
				bounds = new google.maps.LatLngBounds(),
				station_number = detailsObject.station_number,
				station = Stations.getStation(station_number);

				bounds.extend(ll);
				bounds.extend(new google.maps.LatLng(station.lat, station.lng));
				mapFitBounds(bounds);
				showUserMarker(ll);
			}
			

			if(detailMarker) {
				hideStationDetail();
			}

			setTimeout(function() {

				showStationDetail(stationMarkers[station_number], station);

			}, 10);
		}
	},
	map,
	userMarker,
	getUserMarker = function() {
		if(!userMarker) {
			userMarker = new google.maps.Marker({
				map:map, 
				clickable: false
			});
		}

		return userMarker;
	},
	circle,
	getCircle = function() {
		if(!circle) {
			circle = new google.maps.Circle({
				visible:false,
				clickable:false
			});
		}

		return circle;
	},
	stationMarkers = [],
	multipleResults = false,

	mapPanTimeout,
	detailMarker,

	isInitialResult = true,
	omitThisUpdate = false,

	initialZoom = 5,
	initialLat = 51.41142798796079,
	initialLng = 10.429687500000027;

	function onTemplateLoaded() {

		var mapOptions = {
			zoom: initialZoom,
			center: new google.maps.LatLng(initialLat, initialLng), // Deutschland
			mapTypeId: google.maps.MapTypeId.ROADMAP,
			backgroundColor: '#ffffff',
			disableDefaultUI: true,
			zoomControl: true, // Brauchen wir das? Müssen wir das selber machen?
			styles: [
				{
					featureType: 'poi',
					stylers: [
						{visibility:'off'}
					]
				}
			]
		};
		
		map = new google.maps.Map($('#map', $el)[0], mapOptions);

		window['map'] = map;

		bindEvents();
	};
	
	var isBound = false;
	function bindEvents() {

		if(!map || isBound) return;
		

		isBound = true;

		if(detailMarker) {

			console.log('BindEvents:DetailMarker');

			UI.setTitle('JET Tankstelle');
			setTimeout(function() {
				UI.handle('show.back');
			}, 0);

			// Check, ob TS noch StammTS
			var stations = User.getData('stations');
			if(!stations) {
				$el.find('[data-event="ts-star"]').removeClass('is-active');
				$el.find('[data-event="ts-alarm"]').removeClass('is-active');
			} 

		}

		if(showDetail) {

			console.log('BindEvents:ShowDetail', showDetail);

			if(detailMarker) {
				hideStationDetail();
			}

			var myStationNumber, myStation;

			if(_.isString(showDetail)) {

				myStationNumber = showDetail;
				myStation = Stations.getStation(myStationNumber);

				if(myStation['lat']) {

					var _circle = getCircle();
					_circle.setRadius(5000);
					_circle.setCenter(myStation);
					mapFitBounds(_circle.getBounds());
					showUserMarker(_circle.getCenter());


				} else {
					myStation = null;
				}


			} else {

				myStationNumber = showDetail.station_number;
				myStation = Stations.getStation(myStationNumber);

				if(myStation['lat']) {

					var ll = new google.maps.LatLng(showDetail.lat, showDetail.lng),
					bounds = new google.maps.LatLngBounds();

					bounds.extend(ll);
					bounds.extend(new google.maps.LatLng(myStation.lat, myStation.lng));

					mapFitBounds(bounds);
					showUserMarker(ll);
				} else {
					myStation = null;
				}

			}
			
			if(myStation !== null) {

				isInitialResult = false;

				setTimeout(function() {

					showStationDetail(stationMarkers[myStationNumber], myStation);

				}, 100);
				
			}
		}

		if(!userMarker && isLocating !== true) {
			setTimeout(function() {
				
				console.log('BindEvents:GetPosition');
				LocationService.getPosition();

			}, 200);
		}

		// Geocoder (Adresssuche)

		// Es wurde ein Ergebnis gefunden
		JET.on('geocoder.result.ts', function(e, result) {
			
			mapFitBounds(result.viewport);
			map.panTo(result.location);
			map.setZoom(map.getZoom() + 2);
			showUserMarker(result.location);
			$el.find('.search-status').fadeOut();

		});

		// Es wurden mehrere Ergebnisse gefunden.
		JET.on('geocoder.multiple_results.ts', function(e, results) {

			multipleResults = results;

			config.models.tsList({locations:results});
			$el.find('.search-status').fadeOut();

		});

		// Ein Fehler ist aufgetreten
		JET.on('geocoder.error.ts', function(e, msg) {
			UI.alert(msg);
			
			$el.find('.search-status').fadeOut();
		});

		JET.on('geocoder.start.ts', function(e, searchstring) {
			var escaped = $('<div />').text(searchstring).html();
			$el.find('.search-status').html('Einen Moment bitte, <b>'+escaped+'</b> wird gesucht&hellip;').show();
		});


		// Geolocation (GPS)

		// Punkt gefunden
		JET.on('geolocation.success.ts', function(e, coords) {



			var center = new google.maps.LatLng(coords.latitude, coords.longitude),
				circle, 
				bounds;

			if(detailStation == null) {

				circle = getCircle();
				circle.setCenter(center);
				circle.setRadius(2000);
				bounds = circle.getBounds();

				mapFitBounds(bounds);
				//isInitialResult = false;
			}
			
			showUserMarker(center);
			$el.find('.search-status').fadeOut();

		});

		JET.on('geolocation.stop.ts', function() {
			isInitialResult = true;
			$el.find('.search-status').fadeOut();
		});
		JET.on('geolocation.start.ts', function() {
			$el.find('.search-status').html('Einen Moment bitte, Deine Position wird gesucht&hellip;').show();
		});

		// Kein Punkt gefunden
		JET.on('geolocation.error.ts', function(e, msg, error) {

			$el.find('.search-status').hide();

			var $initialText = $('#initial-location-text');
			
			$initialText.hide();

			if(error.code == 3 || error.code == 2) {

				UI.alert({
					body: msg,
					okbutton:'Nochmal versuchen',
					cancelbutton:'Abbrechen'
				}, function() {

					LocationService.locate();
					//$initialText.html(_text);

				}, function() {

					// noop
/*					var infoText = 'Über das Lupensymbol <i class="fa fa-search"></i> kannst Du jederzeit nach einer Adresse oder Deinem Standort suchen.';
					$initialText.html(infoText).show();
*/
				});

				

			} else {
				UI.alert(msg);
			}

		});


		var debounced = _.debounce(function() {
			JET.trigger('map.bounds', map.getBounds());
			
			updateLastPosition(map.getCenter());
			

		}, 1000);

		// Wenn die Map verschoben wird,
		// suchen wir nach TS in den Bounds.
		google.maps.event.addListener(map, 'bounds_changed', debounced);

		google.maps.event.addListener(map, 'zoom_changed', function() {

			updateLastZoom(map.getZoom());

		});

		// Wenn neue TS in den Bounds gefunden wurden
		JET.on('stations.inBounds.ts', function(e, stations, center) {

			if(detailMarker || !getUserMarker().getPosition()) return;

			if(omitThisUpdate) {
				omitThisUpdate = false;
				return;
			}

			updateStationMarkers(stations);

			if(stations.length == 0) {

				var data = {},
					$featureLi = $el.find('li[data-event="ts-suche-features"]'),
					filtered,
					bounds = new google.maps.LatLngBounds();

				

				filtered = Stations.findNearestStation(center.lat(), center.lng(), 400);

				if(filtered.length > 0) {
					bounds.extend(center);
					bounds.extend(new google.maps.LatLng(filtered[0].lat, filtered[0].lng));

					map.fitBounds(bounds);
				} else {

					if($featureLi.hasClass('has-features')) {
						data.msg = 'Sie haben Filter gesetzt. Eventuell werden mehr JET Tankstellen gefunden, wenn Sie den Filter deaktivieren.';
					}

					config.models.tsList(data);
				}

			} else {
				config.models.tsList({stations: stations.sort(stationSortCallback)});
			}

		});


		$el.on('click.ts', '.ts-locations', function(e) {

			var $list = $(this), 
				$item = $(e.target),
				result = multipleResults[$item.index()];

			
			mapFitBounds(result.viewport);
			showUserMarker(result.location);

			config.models.tsList();

		});

		$el.on('click.ts', '.ts-results li', function() {

			var $li = $(this),
				station_number = $li.data('station');

			showStationDetail(stationMarkers[station_number], Stations.getStation(station_number));

		});
		

		// TS Features auswählen
		$el.on('click.ts', 'li[data-event="ts-suche-features"] form', function(e) {
			// Dieses Event anhalten,
			// damit die Featurebox nicht geschlossen wird.
			e.stopPropagation();

		});
		$el.on('change.ts', 'li[data-event="ts-suche-features"] input', function(e) {
			
			Stations.setFeature(this.id, this.checked);

			var form = this.form, 
				inputs = form.elements,
				hasFeatures = false;

			for(var i=0, j=inputs.length; i<j; ++i) {
				if(inputs[i].checked) {
					hasFeatures = true;
					break;
				}
			}

			if(hasFeatures) {
				$('li[data-event="ts-suche-features"]').addClass('has-features');
			} else {
				$('li[data-event="ts-suche-features"]').removeClass('has-features');
			}

			Stations.getStationsInBounds(map.getBounds());

		});
		// TS Features auf/zu
		$el.on('click.ts', 'li[data-event="ts-suche-features"]', function(e) {
			
			$(this).toggleClass('is-active');


		});

		// "Da will ich hin" auf/zu
		$el.on('click.ts', 'li[data-event="ts-navi"]', function(e) {
			
			$(this).toggleClass('is-active');

		});


		// Back Button

		JET.on('UI.back.click.ts', function(e) {
			hideStationDetail(!!showDetail);
		});

		for(var station_id in stationMarkers) {

			stationMarkers[station_id].setMap(map);
			addStationMarkerEvent(stationMarkers[station_id], Stations.getStation(station_id));
		}		

		$el.on('click.ts', 'li[data-event="ts-star"]', function(e) {

			var alertMessage;

			if(!User.isLoggedIn()) {

				alertMessage = '<b>JET Stammtankstelle speichern</b><br />';
				alertMessage += 'Für diese Funktion musst Du dich zunächst anmelden oder registrieren.';

				UI.alert(alertMessage);
				return;
			}

			var stammTS = User.getData('stations'),
				msg, 
				s1, 
				s2,
				$li = $(this);

			if($li.hasClass('is-active')) {

				alertMessage = '<b>Dies ist eine Deiner Stammtankstellen</b><br />';
				alertMessage += 'Möchtest Du diese Tankstelle nicht mehr als Stammtankstelle gespeichert haben?';

				UI.alert(alertMessage, function() {

					$.post(JET.backendURL, {
						action: 'removeStation',
						station_number: detailStation.station_number,
						token: User.getToken()
					}).done(function(data) {

						User.setData(data.user);
						$li.removeClass('is-active');
						$el.find('li[data-event="ts-alarm"]').removeClass('is-active');

					}).fail(function() {

					});

				}, true);

			} else if(stammTS && stammTS.length == 2) {

				stationChooser(stammTS);
				
			} else {
				setStation(detailStation.station_number);
			}

		});

		$el.on('click.ts', 'li[data-event="ts-alarm"]', function(e) {

			if(!User.isLoggedIn()) {
				UI.alert('<b>JET Preisalarm</b><br />Für diese Funktion musst Du dich zunächst anmelden oder registrieren.');
				return;
			}

			var userStations = User.getData('stations'),
				isUserStation = false;

			if(userStations) {
				for(var i=0, j=userStations.length; i<j; ++i) {
					if(userStations[i].station_id == detailStation.station_number) {
						isUserStation = true;
					}
				}
			}

			if(isUserStation) {

				UI.modal('PreisalarmEdit', detailStation.station_number);

			} else if(userStations && userStations.length == 2) {

				$.when(stationChooser(userStations, detailStation.station_number)).then(function() {

					UI.modal('PreisalarmEdit', detailStation.station_number);

				});

			} else {

				setStation(detailStation.station_number).done(function() {
					UI.modal('PreisalarmEdit', detailStation.station_number);					
				});

			}

		});

		$el.on('click.ts', '.relocateStation', function(e) {

			if(!detailStation) {

				LocationService.locate();

			} else {

				map.panTo(detailStation);
				setTimeout(function() {
					map.panBy(0, -124); // Magic Number = Iconhoehe / 2
				}, 0);
				
			}

		});

		$el.on('click.ts', '.ts-detail-klicke-item', function(e) {

			var user_id = this.getAttribute('data-userid'),
				is_friend = _.find(User.getData('friends'), function(friend) {

					return friend.friend_id == user_id;

				}); 

			UI.modal('KlickeUserDetail', {
				user_id: user_id,
				is_friend: is_friend
			});

		});

		JET.on('station.detail.ts', function(e, station_number) {
			console.log('TSSuche:Station.Detail.ST', station_number);
			config.showStation(station_number);
		});

		JET.on('view.didAppear.ts', function(e, which) {
			if(which === 'Tankstellensuche') {
				resizeMap();		
			}
		});

		resizeMap();

	};

	var resizeTimeOut = null;
	function resizeMap() {
		clearTimeout(resizeTimeOut);
		resizeTimeOut = setTimeout(function() {
			console.log('ResizeMap');
			google.maps.event.trigger(map, 'resize');
		}, 100);
	};

	function stationChooser(stammTS) {

		var s1, s2, msg, defered = $.Deferred();

		s1 = Stations.getStation(stammTS[0].station_id);
		s2 = Stations.getStation(stammTS[1].station_id);

		msg = '<b>Du hast bereits 2 Stammtankstellen.</b><br />';
		msg += 'Wenn Du diese Tankstelle als neue Stammtankstelle oder für einen Preisalarm anlegen möchtest, ';
		msg += 'wähle eine der beiden aus, die damit gelöscht und durch die neue Stammtankstelle ersetzt wird.<br />';
		msg += '<button class="change-station-button" data-station="'+s1.station_number+'">'+s1.street + '<br>'+s1.zip+' '+s1.city+'</button>';
		msg += '<br />';
		msg += '<button class="change-station-button" data-station="'+s2.station_number+'">'+s2.street + '<br>'+s2.zip+' '+s2.city+'</button>';

		$('#alert-body').one('click.ts', 'button', function(e) {
			

			var station = this.getAttribute('data-station');
			
			setStation(detailStation.station_number, station).done(function() {

				defered.resolve();

				$('#alert-ok').trigger('click');
			
			});


		});

		UI.alert({
			body:msg,
			okbutton:'Abbrechen'
		}, function() {
			$('#alert-body').off('click.ts');
			defered.reject();
		}, false);

		return defered.promise();

	};

	function setStation(newStation, oldStation) {

		var args = {
			action: 'setStation',
			station_number: newStation,
			oldStation: oldStation,
			token: User.getToken()
		};
		

		return $.post(JET.backendURL, args).done(function(data) {


			User.setData(data.user);

			$el.find('li[data-event="ts-star"]').addClass('is-active');

		}).fail(function(jqXHR, textStatus, errorThrown) {

			UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />Bitte versuche es erneut.');

		});

	}

	function unbindEvents() {

		JET.off('.ts');
		$el.off('.ts');

		google.maps.event.clearInstanceListeners(map);

		for(var station_id in stationMarkers) {
			google.maps.event.clearInstanceListeners(stationMarkers[station_id]);
			stationMarkers[station_id].setMap(null);
		}
		isInitialResult = true;
		isBound = false;
	};

	function stationSortCallback(a, b) {
		return (a.distance < b.distance ? -1 : (a.distance > b.distance ? 1 : 0));
	};


	function updateStationMarkers(stations) {

		var newStations, 
			station_id,
			newMarkers = [],
			userMarker = getUserMarker(),
			userPosition = userMarker.getPosition(),
			round = Math.round;

		if(!userPosition) {
			userPosition = map.getCenter();
		}

		for(var i=0, station; station = stations[i]; ++i) {

			station_id = station.station_number;

			if(stationMarkers[station_id]) {
				
				newMarkers[station_id] = stationMarkers[station_id];
			} else {
				
				newMarkers[station_id] = createStationMarker(station);

				/*
				Distanz zum Usermarker
				Aber nur, wenn der auch vorhanden ist.
				Wenn die Map "nur so" angezeigt wird,
				gibt es keinen Usermarker!
				*/
				
			}
			
			station.distance = calculateDistance(newMarkers[station_id].getPosition(), userPosition);

		}

		for(station_id in stationMarkers) {
			if(!newMarkers[station_id]) {
				
				google.maps.event.clearInstanceListeners(stationMarkers[station_id]);
				stationMarkers[station_id].setMap(null);
			}
		}

		stationMarkers = newMarkers;

	};

	function createStationMarker(aStation) {

		var _marker = new google.maps.Marker({
			map: map,
			position: new google.maps.LatLng(aStation.lat, aStation.lng),
			icon: 'css/chrome/marker_jet_small.png'
		});

		addStationMarkerEvent(_marker, aStation);

		return _marker;

	};

	function addStationMarkerEvent(aMarker, aStation) {

		google.maps.event.addListener(aMarker, 'click', function() {

			showStationDetail(aMarker, aStation);

		});

	}

	/**
	 * Berechnet die Entfernung zwischen 2 Punkten in km
	 *
	 * http://www.movable-type.co.uk/scripts/latlong.html#cosine-law
	 */
	function calculateDistance(ll1, ll2) {
		var R = 6371, // Earth Radius in KM
			sin = Math.sin,
			cos = Math.cos,
			pi = Math.PI / 180,
			lat1 = ll1.lat() * pi,
			lon1 = ll1.lng() * pi,
			lat2 = ll2.lat() * pi,
			lon2 = ll2.lng() * pi;

		return Math.acos(sin(lat1)*sin(lat2) + cos(lat1)*cos(lat2) * cos(lon2-lon1)) * R;
	}

	var detailStation = null, swipeView;

	function showStationDetail(marker, station) {

		$el.find('.search-status').fadeOut();

		detailStation = station;

		if(!marker) {
			stationMarkers[station.station_number] = marker = createStationMarker(station);
		}
		

		UI.handle('show.back');

		map.panTo(marker.getPosition());
		map.setZoom(14);

		detailMarker = marker;

		detailMarker.setIcon(JET.backendURL + '?action=getMarker&station='+station.station_number);
		detailMarker.setClickable(false);

		getUserMarker().setVisible(false);

		mapPanTimeout = setTimeout(function() {

			map.panBy(0, -124); // Magic Number = Iconhoehe / 2
			

		}, 200);

		config.models.tsDetail({station:station});


		$el.find('.detail-view')[0].scrollTop=0;
		$el.addClass('is-detail');

		var klicke = $el.find('.ts-detail-klicke');
		swipeView && swipeView.destroy();
		klicke.empty();

		for(var station_id in stationMarkers) {
			if(marker == stationMarkers[station_id]) continue;
			stationMarkers[station_id].setVisible(false);
		}

		$('#ts-navi-options a', $el).each(function(index, el) {

			var dataHref = el.getAttribute('data-href');

			if(dataHref.indexOf('comgooglemaps') === 0) {
				
				if(!JET.googleMaps) {
					$(el).hide();
				} else if(device.platform == 'Android' || device.platform == 'android') {
					 
					var locationString = station.street+', '+station.zip+' '+station.city;

					 $el.on('click', 'a[data-href^="comgooglemaps"]', function(e) {
					 	e.preventDefault();
					 	navigator.google_navigate.navigate(locationString);

					 });

				} else {
					el.href = _.template(dataHref, station);
				}
			} else if(dataHref.indexOf('tomtom') === 0) {
				if(!JET.tomtom) {
					$(el).hide();
				} else {
					el.href = _.template(dataHref, station);
				}
			} else if(dataHref.indexOf('navigon') === 0) {
				if(!JET.navigon) {
					$(el).hide();
				} else {
					el.href = _.template(dataHref, station);
				}
			} else if(dataHref.indexOf('maps://') === 0) {
				if(!JET.appleMaps) {
					$(el).hide();
				} else {
					el.href = _.template(dataHref, station);
				}
			}

		});

		UI.setTitle('JET Tankstelle');



		

		if(User.isLoggedIn()) {


			var stations = User.getData('stations') || [];

			for(var i=0; i<stations.length; ++i) {
				if(stations[i].station_id == station.station_number) {
					$el.find('li[data-event="ts-star"]').addClass('is-active');

					if(stations[i].has_alarm) {
						$el.find('li[data-event="ts-alarm"]').addClass('is-active');
					}

					break;
				}
			}

			$.get(JET.backendURL, {
				action: 'getKlicke',
				station_number: station.station_number,
				token: User.getToken()
			}).done(function(data) {

				var string = ['<div class="klicke-items"><p>Diese JET Tankstelle hat '+data.klicke.length+' Mitglieder</p><div id="owl-wrapper" class="owl-carousel">'];

				_.each(data.klicke, function(member) {

					string.push('<div class="ts-detail-klicke-item" data-userid="'+member.id+'">');
					string.push('<img src="'+JET.baseURL+(member.picture && 'img/200x200/'+member.picture || JET.defaultProfilePicture)+'" />');
					string.push(member.username);

					string.push('</div>');

				});

				string.push('</div>');
				string.push('</div>');


				klicke.html(string.join(''));

				setTimeout(function() {

					swipeView = klicke.find('#owl-wrapper').owlCarousel({
						items:4,
						itemsDesktop: [1000, 4],
						itemsDesktopSmall: [900, 4],
						itemsTablet: [600, 4],
						itemsMobile: [400, 4],
						scrollPerPage: true,
						pagination: true
					}).data('owlCarousel');

				}, 0);

			}).fail(function(jqXHR, textStatus, errorThrown) {

			});

		}

	};

	function hideStationDetail(isDirectDetailView) {
		
		clearTimeout(mapPanTimeout);

		detailStation = null;

		$el.removeClass('is-detail');

		if(lastKnownPosition === null || lastKnownPosition.lat() == initialLat && lastKnownPosition.lng() == initialLng && lastKnownZoom == initialZoom) {

			setTimeout(function() {
				LocationService.getPosition();
			}, 200);

		} else if(lastKnownZoom === null || lastKnownZoom === initialZoom) {
			var _circle = getCircle();
			_circle.setCenter(lastKnownPosition);
			_circle.setRadius(5000);
			mapFitBounds(_circle.getBounds());
		} else {
			map.panTo(lastKnownPosition);
			setTimeout(function() {
				map.setZoom(lastKnownZoom);
			}, 100);
		}

		
		detailMarker.setClickable(true);
		detailMarker.setIcon('css/chrome/marker_jet_small.png');
		detailMarker = null;
		getUserMarker().setVisible(true);
		

		

		for(var station_id in stationMarkers) {
			stationMarkers[station_id].setVisible(true);
		}

		UI.handle('hide.back');
		UI.setTitle('JET Tankstellenfinder');

		if(!isDirectDetailView) {
			// Wir wollen keine neue Tankstellenliste
			omitThisUpdate = true;
		} else {
			showDetail = null;	
		}
		omitThisUpdate = false;


		$el.find('li[data-event="ts-star"]').removeClass('is-active');
		$el.find('li[data-event="ts-alarm"]').removeClass('is-active');

		
	}

	

	function mapFitBounds(bounds) {

		var _circle = getCircle();
		_circle.setCenter(bounds.getCenter());
		_circle.setRadius(2000);
		var circleBounds = _circle.getBounds();

		if(bounds.contains(circleBounds.getNorthEast()) && bounds.contains(circleBounds.getSouthWest())) {

			map.fitBounds(bounds);
		} else {
			map.fitBounds(circleBounds);
		}

	

		updateLastZoom(map.getZoom());

	};

	var lastKnownPosition = null, 
		lastKnownZoom = null;


	function showUserMarker(ll) {

		updateLastPosition(ll);	

		getUserMarker().setPosition(ll);

	};

	function updateLastPosition(ll) {

		if(detailStation === null) {
			lastKnownPosition = ll;
			console.log('LastKnownPosition %o', lastKnownPosition);			
		}

	};

	function updateLastZoom(zoom) {
		if(detailStation === null) {
			lastKnownZoom = zoom;
			console.log('LastKnownZoom %o', lastKnownZoom);
		}
	};

	return config;

});