JET.controller('TermineNoLogin', function(el) {

	var config = {
		title: 'Termine',
		template: 'Termine_nologin.html'
	};

	return config;

});

JET.controller('Termine', function(el) {

	var isTemplateLoaded = false;
	var events = [];

	// jQuery.clndr Instanz
	// https://kylestetz.github.io/CLNDR/
	var clndr;

	// Kalendertemplate
	var clndrTemplate = ""+
  	"<table class='clndr-table' border='0' cellspacing='0' cellpadding='0'>" +
    "<thead>" +
    "<tr class='header-days'>" +
    "<% for(var i = 0; i < data.daysOfTheWeek.length; i++) { %>" +
    "<td class='header-day'><%= data.daysOfTheWeek[i] %></td>" +
    "<% } %>" +
    "</tr>" +
    "</thead>" +
    "<tbody>" +
    "<% for(var i = 0; i < data.numberOfRows; i++){ %>" +
    "<tr>" +
    "<% for(var j = 0; j < 7; j++){ %>" +
    "<% var d = j + i * 7; %>" +
    "<td class='<%= data.days[d].classes %>'><div class='day-contents'><%= data.days[d].day %>" +
    "</div></td>" +
    "<% } %>" +
    "</tr>" +
    "<% } %>" +
    "</tbody>" +
	"</table>";

	var config = {
		title: 'Termine',
		template: 'Termine.html',
		templateLoaded: function() {
			isTemplateLoaded = true;

			var clndr_container = el.find('.container'),

				// Monatsname und Navigationspfeile
				clndr_header = _.unescape(clndr_container.find('.clndr-controls').remove()[0].outerHTML).split(/[\t\n]/).join(''),

				// Terminliste
				clndr_footer = _.unescape(clndr_container.find('.event-listing').remove()[0].outerHTML).split(/[\t\n]/).join(''),

				// Filter und "Neuer Termin"
				clndr_actions = _.unescape(clndr_container.find('.event-actions').remove()[0].outerHTML).split(/[\t\n]/).join('');

			// Hole die Termine des Users
			// Die Termine des aktuellen Monats sowie des vorherigen und des nächsten Monats
			// werden bei jeder Aktualisierung des Users mitgeladen
			// Daher können wir hier einfach auf die Termine zugreifen
			events = User.getData('events');

			// Sortiere Termine
			sortEvents();

			// Erstelle CLNDR Instanz
			clndr = el.find('.container').clndr({
				template: clndr_header + clndrTemplate + clndr_actions + clndr_footer,
				startWithMonth: moment(),
				events: events,
				adjacentDaysChangeMonth:true,
				dateParameter: 'event_date',
				clickEvents: {
					onMonthChange: function(month) {
						// Wird der Monat gewchselt, laden wir neue Termine
						updateEvents(month);
					},
					click: function(target) {

						// Klick auf einen Tag im Kalender öffnet "Neuer Termin" für diesen Tag
						UI.modal(
							'TermineEdit',
							{
								event_date: target.date.format('YYYY-MM-DD')
							}
						);

					}
				}
			});
		
			

		},
		viewWillAppear: function() {
			if(isTemplateLoaded) {
				// Binde Eventhandler des CLNDR Objects
				clndr.bindEvents();
			}
		},
		bindEvents: function() {

			// Eventhandler um einen Termin aus der Liste zu bearbeiten
			el.on('click.termine', '.event-item', function(e) {

				e.preventDefault();

				var $this = $(this);

				if($this.hasClass('event-item-user')) {
					
					// Termin bearbeiten
					UI.modal('TermineEdit', {
						event_id: this.getAttribute('data-eventid')
					});

					return;
				}

				if($this.hasClass('event-item-jet') || $this.hasClass('event-type-noevents')) {
					// JET Termine sind nicht bearbeitbar
					return;
				}

				// Auto bearbeiten
				UI.modal('BoxenstoppEditCar', this.getAttribute('data-carid'));

				return;

			});

			// Eventhandler für die Filterfunktion
			el.on('click.termine', '.event-actions span', function(e) {

				var $this = $(this);

				if($this.hasClass('termine-alle')) {

					// Alle Termine
					clndr.setEvents(events);
					el.find('#clndr-wrapper').removeClass('filter-user filter-jet');

				} else if($this.hasClass('termine-user')) {

					// Usertermine 
					clndr.setEvents(filterEvents(events, 'user'));
					el.find('#clndr-wrapper').removeClass('filter-jet').addClass('filter-user');

				} else {

					// JET Termine
					clndr.setEvents(filterEvents(events, 'jet'));
					el.find('#clndr-wrapper').removeClass('filter-user').addClass('filter-jet');

				}


			});

			// Eventhandler "Neuer Termin"
			el.on('click.termine', '.event-actions .button-primary', function(e) {

				// Datum auf aktuellen Tag einstellen
				var theDate = '', today = moment();
				if(clndr.month.isSame(today, 'month')) {
					theDate = today;
				} else {
					// Ausser, wenn der Kalender nicht den aktuellen Monat anzeigt.
					// In diesem Fall ist der 1. des angezeigten Monats voreingestellt.
					theDate = clndr.month;
				}

				UI.modal(
					'TermineEdit',
					{
						event_date: theDate.format('YYYY-MM-DD')
					}
				);
			});

			// Setze CSS-Klasse, um die Filterbuttons anzuzeigen / zu verbergen
			// Je nach Userpräferenz
			if(User.getData('flag_calender_ads') == '1') {

				el.removeClass('ads-not-allowed');

			} else {

				el.addClass('ads-not-allowed');

			}


		},
		unbindEvents: function() {
			// CLDNR Event löschen
			$(clndr.element).off();

			// Eventhandler löschen
			JET.off('.termine');
			el.off('.termine');
		},
		refresh: function() {

			// Lade neue Termine
			updateEvents(clndr.month);
		}
	};

	/**
	 * Lädt neue Termine
	 * @param month moment.js Datum
	 * @returns {*}
	 */
	function updateEvents(month) {

		// Default: aktueller Tag
		if(!month) {
			month = moment();
		}

		// Internet?
		if(!isConnected()) {
			return displayNoConnectionError(true);
		}

		// API Call
		$.ajax({
			type: 'POST',
			url: JET.backendURL,
			data: {
				token: User.getToken(),
				action: 'getEvents',
				date: month.format('YYYY-MM-DD 12:00:00'),
				ads: User.getData('flag_calender_ads')
			},
			dataType: 'json'
		}).done(function(data) {

			// Hole Filtereinstellungen
			var $clndrWrapper = el.find('#clndr-wrapper');

			if(data.events) {

				// Lokaler Cache
				events = data.events;

				// Termine sortieren
				sortEvents();

				// Setze Termine je nach Filter
				if($clndrWrapper.hasClass('filter-user')) {
					clndr.setEvents(filterEvents(events, 'user'));
				} else if($clndrWrapper.hasClass('filter-jet')) {
					clndr.setEvents(filterEvents(events, 'jet'));
				} else {
					clndr.setEvents(events);	
				}

				// Aktualisiere User
				User.setData('events', events);
			}

		}).fail(function(jqXHR) {

			// Internet?
			if(isOfflineError(jqXHR)) {
				return displayNoConnectionError(true);
			}

			// Zeige Fehlermeldung
			UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />Die Daten konnten nicht geladen werden.');

		});
		
	};

	/**
	 * Sortiert die Termine
	 * 1. nach Eventdatum
	 * 2. JET Termine erscheinen vor den Userterminen
	 * 3. nach Eintragsdatum
	 */
	function sortEvents() {

		events.sort(function(a, b) {


			if(a.event_date < b.event_date) return -1;
			if(a.event_date > b.event_date) return 1;

			if(a.event_type == 'jet' && b.event_type != 'jet') return -1;
			if(a.event_type != 'jet' && b.event_type == 'jet') return 1;

			if(a.create_date < b.create_date) return -1;
			if(a.create_date > b.create_date) return 1;

			return 0;

		});	

	};

	/**
	 * Filter die Termine
	 * @param _events
	 * @param filter
	 * @returns array mit den gefilterten Terminen
	 */
	function filterEvents(_events, filter) {

		return _.filter(_events, function(item) {
			if(filter == 'jet') {
				return item.event_type == 'jet';
			} else {
				return item.event_type != 'jet';
			}	
		});

	}
	

	return config;

});