
JET.controller('TermineEdit', function(el) {

	var isTemplateLoaded = false,
		info_obj,
		$form;

	var config = {
		title: 'Termin',
		template: 'Termine_edit.html',
		templateLoaded: function() {
			isTemplateLoaded = true;

			$form = el.find('#termin-edit');

			renderForm();
		},
		viewWillAppear: function(info) {

			// Text für den Modalbutton rechts
			$('#modal-primary').text('Speichern');

			// Lösche ggf. vorhandene Fehlermeldungen
			el.find('.error').remove();
			el.find('.is-error').removeClass('is-error');

			// Wenn info übergeben wurde (Datum des Termins oder komplettes Terminobject {id, title, description, date, type}
			if(info) {
				info_obj = info;
			}

			if(isTemplateLoaded) {
				renderForm();
			}

		},
		bindEvents: function() {

			// Eventhandler zum Speichern der Daten
			el.on('submit.termin_edit', '#termin-edit', saveForm);
			JET.on('modal.primary.click.termin_edit', saveForm);

			// Eventhandler zum löschen eines Termins
			el.on('click.termin_edit', '.button-delete', function(e) {

				UI.alert('<b>Bist Du sicher?</b><br>Der Termin wird unwiderruflich gelöscht!', deleteEvent, true);

			});
		},
		unbindEvents: function() {
			// Eventhandler löschen
			el.off('.termin_edit');
			JET.off('.termin_edit');

			// Form zurücksetzen
			$form[0].reset();
		}
	};

	/**
	 * Rendert den View
	 */
	function renderForm() {

		var event_info, btn_delete = el.find('.button-delete');

		if(!info_obj) {

			// Nichts übergeben (Neuer Termin), Default:
			event_info = {
				event_date: moment().format('YYYY-MM-DD'),
				event_id: ''
			};
		} else if(info_obj['event_date']) {

			// Nur Datum übergeben (Neuer Termin)
			event_info = {
				event_date: info_obj['event_date'],
				event_id: ''
			};
		} else if(info_obj['event_id']) {
			// Kompletter Termin übergeben (Edit)
			event_info = _.find(User.getData('events'), function(_event) { return _event.id == info_obj['event_id']; });
			event_info.event_id = event_info.id;
		}

		// Nur dann löschen, wenn wir auch eine ID haben
		if(event_info['id']) {
			btn_delete.show();
		} else {
			btn_delete.hide();
		}

		setTimeout(function() {

			if(isAndroid()) {

				// Android input[type=date] Fix
				var original_date = event_info.event_date;

				event_info = $.extend({}, event_info, {
					event_date: moment(original_date).format('DD.MM.YYYY')
				});

				$form.find('#event_date').attr({
					'type':'text', 
					'readonly':'readonly'
				});

				// Eventhandler zum Auswählen eines Datums
				el.on('click.termin_edit', '#event_date', function(e) {
					e.preventDefault();

					var element = this;

					// Zeige DatePicker
					datePicker.show({
						date: new Date(original_date),
						mode: 'date'
					}, function(date) {

						// Wandele 2015/12/24 in 24.12.2015

						var array = date.split('/');
						if(array[1] < 10) {
							array[1] = '0'+array[1];
						}

						if(array[2] < 10) {
							array[2] = '0'+array[2];
						}

						original_date = array.join('-');

						// Setze Formularvalue
						element.value = array.reverse().join('.');
						element.blur();
					});
				});
			}

			// Setze Formularwerte
			$form.formParams(event_info);

		}, 0);

	};

	/**
	 * Speichert die Daten
	 * @param e
	 * @returns {*}
	 */
	function saveForm(e) {

		// Formsubmit unterbinden
		e.preventDefault();

		// Hole Formulardaten
		var params = $form.formParams(),
			errors = {};
	
		// Formevents löschen
		$form.off('focus.form_helper');

		// Errorcheck
		if(params.title == '') {
			errors['#title'] = 'Bitte gib einen Titel ein!';
		}
		if(params.event_date == '') {
			errors['#event_date'] = 'Bitte gib das Datum ein!';
		}

		// Zeige Fehlermeldungen
		if(!_.isEmpty(errors)) {
			$form.on('focus.form_helper', '.is-error', function() {
				$(this).removeClass('is-error');
			})
			return showError(errors, $form);
		}

		// API Parameter
		params.action = 'editEvent';
		params.token = User.getToken();

		// Internet?
		if(!isConnected()) {
			return displayNoConnectionError();
		}

		// Zeige Loadingspinner
		UI.toggleLoading();

		// API Call
		$.post(JET.backendURL, params, 'json').done(function(data) {

			if(data['user']) {
				// Aktualisiere User
				User.setData(data.user);
			}

			// Schliesse Modal und aktualisiere den darunterliegenden View
			UI.modalClose(true);

		}).fail(function(jqXHR, textStatus, errorThrown) {

			// Internet?
			if(isOfflineError(jqXHR)) {
				return displayNoConnectionError();
			}

			// Zeige Fehlermeldung
			var response = jqXHR.responseText && JSON.parse(jqXHR.responseText) || {msg: "API Fehler " + errorThrown};
			UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />' + response.msg);

		}).always(function() {
			// Verberge Loadingspinner
			UI.toggleLoading();
		});


	}

	/**
	 * Löscht ein Event
	 * @returns {*}
	 */
	function deleteEvent() {

		// Hole Formularwerte
		var params = $form.formParams();

		// API Parameter
		var params = {
			action: 'deleteEvent',
			event_id: params.event_id,
			token: User.getToken()
		};

		// Internet?
		if(!isConnected()) {
			return displayNoConnectionError();
		}

		// Zeige Loadingspinner
		UI.toggleLoading();

		// API Call
		$.post(JET.backendURL, params, 'json')

			.done(function(data) {

				// Aktualisiere User
				if(data['user']) {
					User.setData(data.user);
				}

				// Schliesse Modal und aktualisiere den darunterliegenden View
				UI.modalClose(true);

			})

			.fail(function(jqXHR, textStatus, errorThrown) {

				// Internet?
				if(isOfflineError(jqXHR)) {
					return displayNoConnectionError();
				}

				// Zeige Fehlermeldung
				var response = jqXHR.responseText && JSON.parse(jqXHR.responseText) || {msg: "API Fehler " + errorThrown};
				UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />' + response.msg);

			})

			.always(function() {

				// Verberge Loadingspinner
				UI.toggleLoading();

			});
			
	}

	return config;

});