var Radar = (function() {

	var currentStation = null;

	function bgLocationSuccessCallback(location) {

		console.log('Location Update '+location.latitude+' '+location.longitude);

		var R = 6371,  // km
			P = Math.PI / 180,

			sin = Math.sin,
			cos = Math.cos,
			sqrt = Math.sqrt,

			dLat, dLng, lat_a, lat_b, a, c,

			filtered = [],

			stations = Stations.stations(),
			station_number,
			station;

		for(station_number in stations) {

			station = stations[station_number];

			dLat = sin(((station.lat - location.latitude) * P) / 2);
			dLng = sin(((station.lng - location.longitude) * P) / 2);
			lat_a = cos(location.latitude * P);
			lat_b = cos(station.lat * P);
			
			a = dLat * dLat + dLng * dLng * lat_a * lat_b;
			c = 2 * Math.atan2(sqrt(a), sqrt(1-a));

			station.distance = R * c;

			if(station.distance < 50) {
				filtered.push(station);
			}
		}

		if(filtered.length > 0) {

			filtered.sort(function(a, b) {

				return a.distance < b.distance ? -1 : a.distance > b.distance ? 1 : 0;

			});

			if(filtered[0].station_number != currentStation) {

				window.plugin.notification.local.add({
					id: parseInt(filtered[0].station_number.split('XJ0')[1]).toString(),
					message: filtered[0].street + ' ('+format('#.##0,0', filtered[0].distance)+' km)',
					title: 'JET Tankstelle',
					autoCancel: true,
					json: JSON.stringify({
						type: 'radar',
						id: filtered[0].station_number,
						lat: location.latitude, 
						lng: location.longitude
					})
				});

				if(currentStation) {
					window.plugin.notification.local.cancel(parseInt(currentStation.split('XJ0')[1]).toString());
				}

				currentStation = filtered[0].station_number;
				
			}
		}

		//window.plugins.backgroundGeoLocation.finish();

	};

	function bgLocationErrorCallback(error) {
		console.log('BackgroundLocationError', error);
	};

	function configureBGLocation(isThisForAndroid) {

		var options = {
			desiredAccuracy: 10,
			stationaryRadius: 10,
			distanceFilter: 30,
			debug: true
		};

		if(isThisForAndroid) {
			options.url = JET.backendURL+'?action=updateLocation&uuid='+window.device.uuid+((new Date()).getTime());
			options.params = {
				'authToken': window.localStorage['gcmpushtoken']
			};
		}

		window.plugins.backgroundGeoLocation.configure(
			Radar.success, 
			Radar.error, 
			options
		);

	}
	function configureBGLocationAndroid(token) {

		var options = {
			desiredAccuracy: 10,
			stationaryRadius: 10,
			distanceFilter: 30
		};

		options.url = JET.backendURL+'?action=updateLocation&uuid='+window.device.uuid+((new Date()).getTime());
		options.params = {
			'authToken': token
		};
		
		options.debug = true;

		window.plugins.backgroundGeoLocation.configure(
			Radar.success, 
			Radar.error, 
			options
		);

	}

	document.addEventListener('deviceready', function() {
	
		if(isAndroid()) {
			return;
		} else {
			configureBGLocation(false);
		}

	}, false);

	
	

	function enable() {

		if(isAndroid()) {

			// Android kann die BG Location Updates nicht per JS verarbeiten,
			// Die neuen Positionen werden an den Server geschickt, dort verarbeitet
			// und ggf. per Push an das Gerät gemeldet.
			// Daher brauchen wir zunächst das Pushtoken, falls nicht bereits registriert.
/*
			if(window.localStorage['gcmpushtoken']) {

				window.plugins.backgroundGeoLocation.start();

			} else {

				JET.one('GCM.token', function() {
					configureBGLocation(true);
					window.plugins.backgroundGeoLocation.start();
				});
				JET.registerPush();

			}
*/
			window.plugins.pushNotification.register(

				function successHandler(result) {
					// Noop
					console.log('Android Register Push Success', result);
				},
				function errorHandler(error) {

					console.log('Android Register Push Error', error);	
				},
				{
					"senderID":"616614095654",
					"ecb": 'Radar.pushCallbackGCM'
				}

			);

		} else {
			
			// Für iOS muss sichergestellt sein, das der User Locationdata freigegeben hat,
			// deshalb wird der Aufruf in ein schnelles getCurrentPostion gewrappt.

			navigator.geolocation.getCurrentPosition(
				function geosuccess() {

					console.log('Position found, starting bg location');
					window.plugins.backgroundGeoLocation.start();

				}, 
				function geoerror(error) {
					console.log('Initial position error '+error.code);
					if(error.code == 1) {
						UI.alert('Bitte erlaube den Zugriff auf die Positionsdaten');
					}
				}, 
				{
					enableHighAccuracy: false,
					maximumAge: Infinity
				}
			);

		}

		var $radarButton = $('#nav ul').find('[data-page="Tankstellenradar"] button'),
			$radarIcon = $radarButton.find('.fa'),
			$radarText = $radarButton.find('span');

		$radarIcon.removeClass('fa-square-o').addClass('fa-check-square');
		$radarText.text('an');
		
		localStorage['radar'] = 'enabled';
		
	};

	function disable() {
		try {

			window.plugins.backgroundGeoLocation.stop();

		} catch(e) {
			
			alert('Could not stop backgroundGeoLocation');

		}

		var $radarButton = $('#nav ul').find('[data-page="Tankstellenradar"] button'),
			$radarIcon = $radarButton.find('.fa'),
			$radarText = $radarButton.find('span');

		$radarIcon.removeClass('fa-check-square').addClass('fa-square-o');
		$radarText.text('aus');

		localStorage['radar'] = 'disabled';

	};

	function pushCallbackGCM(e) {
		switch(e.event) {

			case 'registered':

				configureBGLocationAndroid(e.regid);
				window.plugins.backgroundGeoLocation.start();

			break;

			case 'message':
				
				if(e.payload.type == 'radar') {
					if(e.foreground) {
						UI.alert({
							body:'<b>JET Tankstelle</b><br>'+e.payload.message,
							okbutton: 'Anzeigen',
							cancelbutton: 'Abbrechen'
						}, function() {
							UI.closeNav();
							UI.show('Tankstellensuche', {
								station_number: e.payload.id,
								lat: e.payload.lat,
								lng: e.payload.lng
							});	
						}, function() {
							// Noop
						});	
					} else {
						UI.closeNav();
						UI.show('Tankstellensuche', e.payload.id);
					}
					
				} else {
					UI.alert(e.payload.message);
				}
			break;

			case 'error':
				UI.alert('<b>Ein Fehler ist aufgetreten!</b><br />'+e.msg);
			break;

			default:

				UI.alert('<b>Bei dem Versuch Ihr Gerät für Pushnachrichten zu registrieren, ist ein unbekannter Fehler aufgetreten!</b>');

			break;

		}
	};

	return {

		enable: enable,
		disable: disable,
		success: bgLocationSuccessCallback,
		error: bgLocationErrorCallback,
		pushCallbackGCM: pushCallbackGCM,
		status: function() {
			var isRadarEnabled = localStorage['radar'];

			return isRadarEnabled === 'enabled' ? 'enabled' : 'disabled';
		}

	};

}());